@@ -1,5 +1,4 @@
-#!/usr/bin/python
-# Copyright(C) 2012-2020 Open Information Security Foundation
+# Copyright(C) 2012-2023 Open Information Security Foundation
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +21,7 @@ import readline
 import select
 import sys
 from socket import AF_UNIX, error, socket
+from inspect import currentframe
 
 from suricata.sc.specs import argsd
 
@@ -30,6 +30,11 @@ VERSION = "0.2"
 INC_SIZE = 1024
 
 
+def get_linenumber():
+    cf = currentframe()
+    return cf.f_back.f_lineno
+
+
 class SuricataException(Exception):
     """
     Generic class for suricatasc exception
@@ -107,6 +112,9 @@ class SuricataSC:
                 "memcap-show",
                 "dataset-add",
                 "dataset-remove",
+                "get-flow-stats-by-id",
+                "dataset-clear",
+                "dataset-lookup",
                 ]
         self.cmd_list = self.basic_commands + self.fn_commands
         self.sck_path = sck_path
@@ -133,7 +141,7 @@ class SuricataSC:
 
     def send_command(self, command, arguments=None):
         if command not in self.cmd_list and command != 'command-list':
-            raise SuricataCommandException("Command not found: {}".format(command))
+            raise SuricataCommandException("L{}: Command not found: {}".format(get_linenumber(), command))
 
         cmdmsg = {}
         cmdmsg['command'] = command
@@ -153,7 +161,7 @@ class SuricataSC:
         else:
             cmdret = None
         if not cmdret:
-            raise SuricataReturnException("Unable to get message from server")
+            raise SuricataReturnException("L{}: Unable to get message from server".format(get_linenumber))
 
         if self.verbose:
             print("RCV: "+ json.dumps(cmdret))
@@ -166,7 +174,7 @@ class SuricataSC:
                 self.socket = socket(AF_UNIX)
             self.socket.connect(self.sck_path)
         except error as err:
-            raise SuricataNetException(err)
+            raise SuricataNetException("L{}: {}".format(get_linenumber(), err))
 
         self.socket.settimeout(10)
         #send version
@@ -184,13 +192,13 @@ class SuricataSC:
             cmdret = None
 
         if not cmdret:
-            raise SuricataReturnException("Unable to get message from server")
+            raise SuricataReturnException("L{}: Unable to get message from server".format(get_linenumber()))
 
         if self.verbose:
             print("RCV: "+ json.dumps(cmdret))
 
         if cmdret["return"] == "NOK":
-            raise SuricataReturnException("Error: %s" % (cmdret["message"]))
+            raise SuricataReturnException("L{}: Error: {}".format(get_linenumber(), cmdret["message"]))
 
         cmdret = self.send_command("command-list")
 
@@ -220,9 +228,9 @@ class SuricataSC:
                 except IndexError:
                     phrase = " at least" if required_args_count != len(cmd_specs) else ""
                     msg = "Missing arguments: expected{} {}".format(phrase, required_args_count)
-                    raise SuricataCommandException(msg)
+                    raise SuricataCommandException("L{}: {}".format(get_linenumber(), msg))
                 except ValueError as ve:
-                    raise SuricataCommandException("Erroneous arguments: {}".format(ve))
+                    raise SuricataCommandException("L{}: Erroneous arguments: {}".format(get_linenumber(), ve))
             elif c < len(full_cmd):
                 arguments[spec["name"]] = spec_type(full_cmd[c])
         return cmd, arguments
@@ -234,7 +242,7 @@ class SuricataSC:
             if cmd in self.fn_commands:
                 cmd, arguments = getattr(self, "execute")(command=command)
         else:
-            raise SuricataCommandException("Unknown command {}".format(command))
+            raise SuricataCommandException("L{}: Unknown command: {}".format(get_linenumber(), command))
         return cmd, arguments
 
     def interactive(self):
@@ -250,6 +258,8 @@ class SuricataSC:
                     command = input(">>> ").strip()
                 if command == "quit":
                     break
+                if len(command.strip()) == 0:
+                    continue
                 try:
                     cmd, arguments = self.parse_command(command)
                 except SuricataCommandException as err:
@@ -263,10 +273,13 @@ class SuricataSC:
                     try:
                         self.close()
                         self.connect()
-                    except SuricataNetException as err:
-                        print("Can't reconnect to suricata socket, discarding command")
+                    except (SuricataNetException, SuricataReturnException) as err:
+                        print(err.value)
                         continue
                     cmdret = self.send_command(cmd, arguments)
+                except (SuricataCommandException, SuricataReturnException) as err:
+                    print("An exception occured: " + str(err.value))
+                    continue
                 #decode json message
                 if cmdret["return"] == "NOK":
                     print("Error:")
@@ -276,3 +289,4 @@ class SuricataSC:
                     print(json.dumps(cmdret["message"], sort_keys=True, indent=4, separators=(',', ': ')))
         except KeyboardInterrupt:
             print("[!] Interrupted")
+            sys.exit(0)
