@@ -30,21 +30,23 @@ pub struct DHCPLogger {
 
 impl DHCPLogger {
     
-    pub fn new(conf: ConfNode) -> DHCPLogger {
-        return DHCPLogger{
+    pub fn new(conf: ConfNode) -> Self {
+        return Self {
             extended: conf.get_child_bool("extended"),
-        };
+        }
     }
 
     fn get_type(&self, tx: &DHCPTransaction) -> Option<u8> {
         let options = &tx.message.options;
         for option in options {
             let code = option.code;
+            #[allow(clippy::single_match)]
             match &option.option {
-                &DHCPOptionWrapper::Generic(ref option) => {
+                DHCPOptionWrapper::Generic(option) => {
+                    #[allow(clippy::single_match)]
                     match code {
                         DHCP_OPT_TYPE => {
-                            if option.data.len() > 0 {
+                            if !option.data.is_empty() {
                                 return Some(option.data[0]);
                             }
                         }
@@ -59,16 +61,8 @@ impl DHCPLogger {
 
     pub fn do_log(&self, tx: &DHCPTransaction) -> bool {
         if !self.extended {
-            match self.get_type(tx) {
-                Some(t) => {
-                    match t {
-                        DHCP_TYPE_ACK => {
-                            return true;
-                        }
-                        _ => {}
-                    }
-                }
-                _ => {}
+            if let Some(DHCP_TYPE_ACK) = self.get_type(tx){
+                return true;
             }
             return false;
         }
@@ -110,12 +104,12 @@ impl DHCPLogger {
         
         for option in options {
             let code = option.code;
-            match &option.option {
-                &DHCPOptionWrapper::ClientId(ref clientid) => {
+            match option.option {
+                DHCPOptionWrapper::ClientId(ref clientid) => {
                     js.set_string("client_id",
                                   &format_addr_hex(&clientid.data))?;
                 }
-                &DHCPOptionWrapper::TimeValue(ref time_value) => {
+                DHCPOptionWrapper::TimeValue(ref time_value) => {
                     match code {
                         DHCP_OPT_ADDRESS_TIME => {
                             if self.extended {
@@ -136,7 +130,7 @@ impl DHCPLogger {
                         _ => {}
                     }
                 }
-                &DHCPOptionWrapper::Generic(ref option) => {
+                DHCPOptionWrapper::Generic(ref option) => {
                     match code {
                         DHCP_OPT_SUBNET_MASK => {
                             if self.extended {
@@ -145,7 +139,7 @@ impl DHCPLogger {
                             }
                         }
                         DHCP_OPT_HOSTNAME => {
-                            if option.data.len() > 0 {
+                            if !option.data.is_empty() {
                                 js.set_string_from_bytes("hostname",
                                                          &option.data)?;
                             }
@@ -187,7 +181,7 @@ impl DHCPLogger {
     }
 
     fn log_opt_type(&self, js: &mut JsonBuilder, option: &DHCPOptGeneric) -> Result<(), JsonError> {
-        if option.data.len() > 0 {
+        if !option.data.is_empty() {
             let dhcp_type = match option.data[0] {
                 DHCP_TYPE_DISCOVER => "discover",
                 DHCP_TYPE_OFFER => "offer",
@@ -218,7 +212,7 @@ impl DHCPLogger {
                 DHCP_PARAM_TFTP_SERVER_IP => "tftp_server_ip",
                 _ => ""
             };
-            if param.len() > 0 {
+            if !param.is_empty() {
                 js.append_string(param)?;
             }
         }
@@ -248,7 +242,7 @@ impl DHCPLogger {
 
 }
 
-fn format_addr_hex(input: &Vec<u8>) -> String {
+fn format_addr_hex(input: &[u8]) -> String {
     let parts: Vec<String> = input.iter()
         .map(|b| format!("{:02x}", b))
         .collect();
@@ -259,16 +253,16 @@ fn format_addr_hex(input: &Vec<u8>) -> String {
 pub extern "C" fn rs_dhcp_logger_new(conf: *const c_void) -> *mut std::os::raw::c_void {
     let conf = ConfNode::wrap(conf);
     let boxed = Box::new(DHCPLogger::new(conf));
-    return unsafe{std::mem::transmute(boxed)};
+    return Box::into_raw(boxed) as *mut _;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_dhcp_logger_free(logger: *mut std::os::raw::c_void) {
-    let _: Box<DHCPLogger> = unsafe{std::mem::transmute(logger)};
+pub unsafe extern "C" fn rs_dhcp_logger_free(logger: *mut std::os::raw::c_void) {
+    std::mem::drop(Box::from_raw(logger as *mut DHCPLogger));
 }
 
 #[no_mangle]
-pub extern "C" fn rs_dhcp_logger_log(logger: *mut std::os::raw::c_void,
+pub unsafe extern "C" fn rs_dhcp_logger_log(logger: *mut std::os::raw::c_void,
                                      tx: *mut std::os::raw::c_void,
                                      js: &mut JsonBuilder) -> bool {
     let logger = cast_pointer!(logger, DHCPLogger);
@@ -277,7 +271,7 @@ pub extern "C" fn rs_dhcp_logger_log(logger: *mut std::os::raw::c_void,
 }
 
 #[no_mangle]
-pub extern "C" fn rs_dhcp_logger_do_log(logger: *mut std::os::raw::c_void,
+pub unsafe extern "C" fn rs_dhcp_logger_do_log(logger: *mut std::os::raw::c_void,
                                         tx: *mut std::os::raw::c_void)
                                         -> bool {
     let logger = cast_pointer!(logger, DHCPLogger);
