@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Open Information Security Foundation
+/* Copyright (C) 2017-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -22,9 +22,8 @@ use crate::core::*;
 
 // Defined in util-file.h
 extern {
-    pub fn FileFlowToFlags(flow: *const Flow, flags: u8) -> u16;
+    pub fn FileFlowFlagsToFlags(flow_file_flags: u16, flags: u8) -> u16;
 }
-pub const FILE_USE_DETECT:    u16 = BIT_U16!(13);
 
 #[repr(C)]
 #[derive(Debug)]
@@ -33,17 +32,18 @@ pub struct FileContainer {
     tail: * mut c_void,
 }
 
+impl Default for FileContainer {
+    fn default() -> Self { Self {
+        head: ptr::null_mut(),
+        tail: ptr::null_mut(),
+    }}
+}
+
 impl FileContainer {
-    pub fn default() -> FileContainer {
-        FileContainer { head:ptr::null_mut(), tail:ptr::null_mut() }
-    }
-    pub fn free(&mut self) {
+    pub fn free(&mut self, cfg: &'static SuricataFileContext) {
         SCLogDebug!("freeing self");
-        match unsafe {SC} {
-            None => panic!("BUG no suricata_config"),
-            Some(c) => {
-                (c.FileContainerRecycle)(&self);
-            },
+        if let Some(c) = unsafe {SC} {
+            (c.FileContainerRecycle)(self, cfg.files_sbcfg);
         }
     }
 
@@ -53,7 +53,7 @@ impl FileContainer {
             Some(c) => {
                 SCLogDebug!("FILE {:p} OPEN flags {:04X}", &self, flags);
 
-                let res = (c.FileOpenFile)(&self, cfg.files_sbcfg, track_id,
+                let res = (c.FileOpenFile)(self, cfg.files_sbcfg, track_id,
                         name.as_ptr(), name.len() as u16,
                         ptr::null(), 0u32, flags);
                 res
@@ -61,9 +61,9 @@ impl FileContainer {
         }
     }
 
-    pub fn file_append(&mut self, track_id: &u32, data: &[u8], is_gap: bool) -> i32 {
+    pub fn file_append(&mut self, cfg: &'static SuricataFileContext, track_id: &u32, data: &[u8], is_gap: bool) -> i32 {
         SCLogDebug!("FILECONTAINER: append {}", data.len());
-        if data.len() == 0 {
+        if data.is_empty() {
             return 0
         }
         match unsafe {SC} {
@@ -72,13 +72,13 @@ impl FileContainer {
                 let res = match is_gap {
                     false => {
                         SCLogDebug!("appending file data");
-                        let r = (c.FileAppendData)(&self, *track_id,
+                        let r = (c.FileAppendData)(self, cfg.files_sbcfg, *track_id,
                                 data.as_ptr(), data.len() as u32);
                         r
                     },
                     true => {
                         SCLogDebug!("appending GAP");
-                        let r = (c.FileAppendGAP)(&self, *track_id,
+                        let r = (c.FileAppendGAP)(self, cfg.files_sbcfg, *track_id,
                                 data.as_ptr(), data.len() as u32);
                         r
                     },
@@ -88,35 +88,16 @@ impl FileContainer {
         }
     }
 
-    pub fn file_close(&mut self, track_id: &u32, flags: u16) -> i32 {
+    pub fn file_close(&mut self, cfg: &'static SuricataFileContext, track_id: &u32, flags: u16) -> i32 {
         SCLogDebug!("FILECONTAINER: CLOSEing");
 
         match unsafe {SC} {
             None => panic!("BUG no suricata_config"),
             Some(c) => {
-                let res = (c.FileCloseFile)(&self, *track_id, ptr::null(), 0u32, flags);
+                let res = (c.FileCloseFile)(self, cfg.files_sbcfg, *track_id, ptr::null(), 0u32, flags);
                 res
             }
         }
 
     }
-
-    pub fn files_prune(&mut self) {
-        SCLogDebug!("FILECONTAINER: pruning");
-        match unsafe {SC} {
-            None => panic!("BUG no suricata_config"),
-            Some(c) => {
-                (c.FilePrune)(&self);
-            }
-        }
-    }
-
-    pub fn file_set_txid_on_last_file(&mut self, tx_id: u64) {
-        match unsafe {SC} {
-            None => panic!("BUG no suricata_config"),
-            Some(c) => {
-                (c.FileSetTx)(&self, tx_id);
-            }
-        }
-    }
 }
