@@ -15,11 +15,12 @@
  * 02110-1301, USA.
  */
 
-#[cfg(feature = "decompression")]
+#![allow(clippy::result_unit_err)]
+
 mod decompression;
 pub mod detect;
-pub mod files;
 pub mod http2;
 mod huffman;
 pub mod logger;
 mod parser;
+mod range;
