@@ -1,4 +1,4 @@
-/* Copyright (C) 2017 Open Information Security Foundation
+/* Copyright (C) 2017-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -17,15 +17,29 @@
 
 #![cfg_attr(feature = "strict", deny(warnings))]
 
-// Clippy lints we want to suppress due to style, or simply too noisy
-// and not a priority right now.
-#![allow(clippy::missing_safety_doc)]
+// Allow these patterns as its a style we like.
 #![allow(clippy::needless_return)]
-#![allow(clippy::redundant_field_names)]
-#![allow(clippy::len_zero)]
+#![allow(clippy::let_and_return)]
+#![allow(clippy::uninlined_format_args)]
 
-#[macro_use]
-extern crate nom;
+// We find this makes sense at time.
+#![allow(clippy::module_inception)]
+
+// The match macro is not always more clear. But its use is
+// recommended where it makes sense.
+#![allow(clippy::match_like_matches_macro)]
+
+// Something we should be conscious of, but due to interfacing with C
+// is unavoidable at this time.
+#![allow(clippy::too_many_arguments)]
+
+// This would be nice, but having this lint enables causes
+// clippy --fix to make changes that don't meet our MSRV.
+#![allow(clippy::derivable_impls)]
+
+// TODO: All unsafe functions should have a safety doc, even if its
+// just due to FFI.
+#![allow(clippy::missing_safety_doc)]
 
 #[macro_use]
 extern crate bitflags;
@@ -41,6 +55,9 @@ extern crate kerberos_parser;
 extern crate tls_parser;
 extern crate x509_parser;
 
+#[macro_use]
+extern crate suricata_derive;
+
 #[macro_use]
 pub mod log;
 
@@ -53,9 +70,12 @@ pub mod conf;
 pub mod jsonbuilder;
 #[macro_use]
 pub mod applayer;
+/// cbindgen:ignore
+pub mod frames;
 pub mod filecontainer;
 pub mod filetracker;
 pub mod kerberos;
+pub mod detect;
 
 #[cfg(feature = "lua")]
 pub mod lua;
@@ -66,8 +86,9 @@ pub mod ftp;
 pub mod smb;
 pub mod krb;
 pub mod dcerpc;
+pub mod modbus;
 
-pub mod ikev2;
+pub mod ike;
 pub mod snmp;
 
 pub mod ntp;
@@ -76,6 +97,8 @@ pub mod dhcp;
 pub mod sip;
 pub mod rfb;
 pub mod mqtt;
+pub mod pgsql;
+pub mod telnet;
 pub mod applayertemplate;
 pub mod rdp;
 pub mod x509;
@@ -83,5 +106,9 @@ pub mod asn1;
 pub mod mime;
 pub mod ssh;
 pub mod http2;
+pub mod quic;
+pub mod bittorrent_dht;
 pub mod plugin;
+pub mod lzma;
 pub mod util;
+pub mod ffi;
