@@ -16,13 +16,17 @@
  */
 
 #include "suricata-common.h"
+#include "suricata.h"
 #include "stream.h"
 #include "util-byte.h"
 #include "util-unittest.h"
 #include "util-hashlist.h"
 
 #include "util-print.h"
+#include "util-spm-bs.h"
+#include "util-enum.h"
 
+#include "app-layer.h"
 #include "app-layer-protos.h"
 #include "app-layer-parser.h"
 #include "app-layer-detect-proto.h"
@@ -30,9 +34,6 @@
 #include "app-layer-dnp3.h"
 #include "app-layer-dnp3-objects.h"
 
-/* For hexdump(). */
-#include "app-layer-dcerpc-common.h"
-
 /* Default number of unreplied requests to be considered a flood. */
 #define DNP3_DEFAULT_REQ_FLOOD_COUNT 500
 
@@ -97,9 +98,6 @@ SCEnumCharMap dnp3_decoder_event_table[] = {
     {NULL, -1},
 };
 
-/* Some DNP3 servers start with a banner. */
-static const char banner[] = "DNP3";
-
 /* Calculate the next transport sequence number. */
 #define NEXT_TH_SEQNO(current) ((current + 1) % DNP3_MAX_TRAN_SEQNO)
 
@@ -251,6 +249,9 @@ static int DNP3CheckStartBytes(const DNP3LinkHeader *header)
         header->start_byte1 == DNP3_START_BYTE1;
 }
 
+/* Some DNP3 servers start with a banner. */
+#define DNP3_BANNER "DNP3"
+
 /**
  * \brief Check if a frame contains a banner.
  *
@@ -261,7 +262,7 @@ static int DNP3CheckStartBytes(const DNP3LinkHeader *header)
  */
 static int DNP3ContainsBanner(const uint8_t *input, uint32_t len)
 {
-    return BasicSearch(input, len, (uint8_t *)banner, strlen(banner)) != NULL;
+    return BasicSearch(input, len, (uint8_t *)DNP3_BANNER, strlen(DNP3_BANNER)) != NULL;
 }
 
 /**
@@ -320,7 +321,7 @@ static uint16_t DNP3ProbingParser(Flow *f, uint8_t direction,
 }
 
 /**
- * \brief Caculate the length of the transport layer with CRCs removed.
+ * \brief Calculate the length of the transport layer with CRCs removed.
  *
  * \param input_len The length of the transport layer buffer.
  *
@@ -466,11 +467,11 @@ static void *DNP3StateAlloc(void *orig_state, AppProto proto_orig)
 static void DNP3SetEvent(DNP3State *dnp3, uint8_t event)
 {
     if (dnp3 && dnp3->curr) {
-        AppLayerDecoderEventsSetEventRaw(&dnp3->curr->decoder_events, event);
+        AppLayerDecoderEventsSetEventRaw(&dnp3->curr->tx_data.events, event);
         dnp3->events++;
     }
     else {
-        SCLogWarning(SC_ERR_ALPARSER, "Failed to set event, state or tx pointer was NULL.");
+        SCLogWarning("Failed to set event, state or tx pointer was NULL.");
     }
 }
 
@@ -479,14 +480,14 @@ static void DNP3SetEvent(DNP3State *dnp3, uint8_t event)
  */
 static void DNP3SetEventTx(DNP3Transaction *tx, uint8_t event)
 {
-    AppLayerDecoderEventsSetEventRaw(&tx->decoder_events, event);
+    AppLayerDecoderEventsSetEventRaw(&tx->tx_data.events, event);
     tx->dnp3->events++;
 }
 
 /**
  * \brief Allocation a DNP3 transaction.
  */
-static DNP3Transaction *DNP3TxAlloc(DNP3State *dnp3)
+static DNP3Transaction *DNP3TxAlloc(DNP3State *dnp3, bool request)
 {
     DNP3Transaction *tx = SCCalloc(1, sizeof(DNP3Transaction));
     if (unlikely(tx == NULL)) {
@@ -497,8 +498,13 @@ static DNP3Transaction *DNP3TxAlloc(DNP3State *dnp3)
     dnp3->curr = tx;
     tx->dnp3 = dnp3;
     tx->tx_num = dnp3->transaction_max;
-    TAILQ_INIT(&tx->request_objects);
-    TAILQ_INIT(&tx->response_objects);
+    tx->is_request = request;
+    if (tx->is_request) {
+        tx->tx_data.detect_flags_tc |= APP_LAYER_TX_SKIP_INSPECT_FLAG;
+    } else {
+        tx->tx_data.detect_flags_ts |= APP_LAYER_TX_SKIP_INSPECT_FLAG;
+    }
+    TAILQ_INIT(&tx->objects);
     TAILQ_INSERT_TAIL(&dnp3->tx_list, tx, next);
 
     /* Check for flood state. */
@@ -657,7 +663,7 @@ static void DNP3ObjectFree(DNP3Object *object)
 }
 
 /**
- * \breif Allocate a DNP3 object.
+ * \brief Allocate a DNP3 object.
  */
 static DNP3Object *DNP3ObjectAlloc(void)
 {
@@ -868,12 +874,8 @@ static void DNP3HandleUserDataRequest(DNP3State *dnp3, const uint8_t *input,
 
     if (!DNP3_TH_FIR(th)) {
         TAILQ_FOREACH(ttx, &dnp3->tx_list, next) {
-            if (ttx->request_lh.src == lh->src &&
-                ttx->request_lh.dst == lh->dst &&
-                ttx->has_request &&
-                !ttx->request_done &&
-                NEXT_TH_SEQNO(DNP3_TH_SEQ(ttx->request_th)) == DNP3_TH_SEQ(th))
-            {
+            if (ttx->lh.src == lh->src && ttx->lh.dst == lh->dst && ttx->is_request && !ttx->done &&
+                    NEXT_TH_SEQNO(DNP3_TH_SEQ(ttx->th)) == DNP3_TH_SEQ(th)) {
                 tx = ttx;
                 break;
             }
@@ -885,7 +887,7 @@ static void DNP3HandleUserDataRequest(DNP3State *dnp3, const uint8_t *input,
 
         /* Update the saved transport header so subsequent segments
          * will be matched to this sequence number. */
-        tx->response_th = th;
+        tx->th = th;
     }
     else {
         ah = (DNP3ApplicationHeader *)(input + sizeof(DNP3LinkHeader) +
@@ -897,24 +899,21 @@ static void DNP3HandleUserDataRequest(DNP3State *dnp3, const uint8_t *input,
         }
 
         /* Create a transaction. */
-        tx = DNP3TxAlloc(dnp3);
+        tx = DNP3TxAlloc(dnp3, true);
         if (unlikely(tx == NULL)) {
             return;
         }
-        tx->request_lh = *lh;
-        tx->request_th = th;
-        tx->request_ah = *ah;
-        tx->has_request = 1;
-
+        tx->lh = *lh;
+        tx->th = th;
+        tx->ah = *ah;
     }
 
     if (!DNP3ReassembleApplicationLayer(input + sizeof(DNP3LinkHeader),
-            input_len - sizeof(DNP3LinkHeader),
-            &tx->request_buffer, &tx->request_buffer_len)) {
+                input_len - sizeof(DNP3LinkHeader), &tx->buffer, &tx->buffer_len)) {
 
         /* Malformed, set event and mark as done. */
         DNP3SetEvent(dnp3, DNP3_DECODER_EVENT_MALFORMED);
-        tx->request_done = 1;
+        tx->done = 1;
         return;
     }
 
@@ -923,26 +922,11 @@ static void DNP3HandleUserDataRequest(DNP3State *dnp3, const uint8_t *input,
         return;
     }
 
-    tx->request_done = 1;
+    tx->done = 1;
 
-    /* Some function codes do not expect a reply. */
-    switch (tx->request_ah.function_code) {
-        case DNP3_APP_FC_CONFIRM:
-        case DNP3_APP_FC_DIR_OPERATE_NR:
-        case DNP3_APP_FC_FREEZE_NR:
-        case DNP3_APP_FC_FREEZE_CLEAR_NR:
-        case DNP3_APP_FC_FREEZE_AT_TIME_NR:
-        case DNP3_APP_FC_AUTH_REQ_NR:
-            tx->response_done = 1;
-        default:
-            break;
-    }
-
-    if (DNP3DecodeApplicationObjects(
-            tx, tx->request_buffer + sizeof(DNP3ApplicationHeader),
-                tx->request_buffer_len - sizeof(DNP3ApplicationHeader),
-                &tx->request_objects)) {
-        tx->request_complete = 1;
+    if (DNP3DecodeApplicationObjects(tx, tx->buffer + sizeof(DNP3ApplicationHeader),
+                tx->buffer_len - sizeof(DNP3ApplicationHeader), &tx->objects)) {
+        tx->complete = 1;
     }
 }
 
@@ -967,11 +951,8 @@ static void DNP3HandleUserDataResponse(DNP3State *dnp3, const uint8_t *input,
 
     if (!DNP3_TH_FIR(th)) {
         TAILQ_FOREACH(ttx, &dnp3->tx_list, next) {
-            if (ttx->response_lh.src == lh->src &&
-                ttx->response_lh.dst == lh->dst &&
-                ttx->has_response && !ttx->response_done &&
-                NEXT_TH_SEQNO(DNP3_TH_SEQ(ttx->response_th)) == DNP3_TH_SEQ(th))
-            {
+            if (ttx->lh.src == lh->src && ttx->lh.dst == lh->dst && !ttx->is_request &&
+                    !ttx->done && NEXT_TH_SEQNO(DNP3_TH_SEQ(ttx->th)) == DNP3_TH_SEQ(th)) {
                 tx = ttx;
                 break;
             }
@@ -983,53 +964,27 @@ static void DNP3HandleUserDataResponse(DNP3State *dnp3, const uint8_t *input,
 
         /* Replace the transport header in the transaction with this
          * one in case there are more frames. */
-        tx->response_th = th;
+        tx->th = th;
     }
     else {
         ah = (DNP3ApplicationHeader *)(input + offset);
         offset += sizeof(DNP3ApplicationHeader);
         iin = (DNP3InternalInd *)(input + offset);
 
-        if (ah->function_code == DNP3_APP_FC_UNSOLICITED_RESP) {
-            tx = DNP3TxAlloc(dnp3);
-            if (unlikely(tx == NULL)) {
-                return;
-            }
-
-            /* There is no request associated with an unsolicited
-             * response, so mark the request done as far as
-             * transaction state handling is concerned. */
-            tx->request_done = 1;
+        tx = DNP3TxAlloc(dnp3, false);
+        if (unlikely(tx == NULL)) {
+            return;
         }
-        else {
-            /* Find transaction. */
-            TAILQ_FOREACH(ttx, &dnp3->tx_list, next) {
-                if (ttx->has_request &&
-                    ttx->request_done &&
-                    ttx->request_lh.src == lh->dst &&
-                    ttx->request_lh.dst == lh->src &&
-                    !ttx->has_response &&
-                    !ttx->response_done &&
-                    DNP3_APP_SEQ(ttx->request_ah.control) == DNP3_APP_SEQ(ah->control)) {
-                    tx = ttx;
-                    break;
-                }
-            }
-            if (tx == NULL) {
-                return;
-            }
-        }
-
-        tx->has_response = 1;
-        tx->response_lh = *lh;
-        tx->response_th = th;
-        tx->response_ah = *ah;
-        tx->response_iin = *iin;
+        tx->lh = *lh;
+        tx->th = th;
+        tx->ah = *ah;
+        tx->iin = *iin;
     }
 
+    BUG_ON(tx->is_request);
+
     if (!DNP3ReassembleApplicationLayer(input + sizeof(DNP3LinkHeader),
-            input_len - sizeof(DNP3LinkHeader),
-            &tx->response_buffer, &tx->response_buffer_len)) {
+                input_len - sizeof(DNP3LinkHeader), &tx->buffer, &tx->buffer_len)) {
         DNP3SetEvent(dnp3, DNP3_DECODER_EVENT_MALFORMED);
         return;
     }
@@ -1038,13 +993,12 @@ static void DNP3HandleUserDataResponse(DNP3State *dnp3, const uint8_t *input,
         return;
     }
 
-    tx->response_done = 1;
+    tx->done = 1;
 
     offset = sizeof(DNP3ApplicationHeader) + sizeof(DNP3InternalInd);
-    if (DNP3DecodeApplicationObjects(tx, tx->response_buffer + offset,
-            tx->response_buffer_len - offset,
-            &tx->response_objects)) {
-        tx->response_complete = 1;
+    if (DNP3DecodeApplicationObjects(
+                tx, tx->buffer + offset, tx->buffer_len - offset, &tx->objects)) {
+        tx->complete = 1;
     }
 }
 
@@ -1131,22 +1085,22 @@ error:
  * multiple frames, but not the complete final frame).
  */
 static AppLayerResult DNP3ParseRequest(Flow *f, void *state, AppLayerParserState *pstate,
-    const uint8_t *input, uint32_t input_len, void *local_data,
-    const uint8_t flags)
+        StreamSlice stream_slice, void *local_data)
 {
     SCEnter();
     DNP3State *dnp3 = (DNP3State *)state;
     DNP3Buffer *buffer = &dnp3->request_buffer;
     int processed = 0;
 
+    const uint8_t *input = StreamSliceGetData(&stream_slice);
+    uint32_t input_len = StreamSliceGetDataLen(&stream_slice);
+
     if (input_len == 0) {
         SCReturnStruct(APP_LAYER_OK);
     }
 
     if (buffer->len) {
         if (!DNP3BufferAdd(buffer, input, input_len)) {
-            SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate memory to buffer "
-                "DNP3 request data");
             goto error;
         }
         processed = DNP3HandleRequestLinkLayer(dnp3,
@@ -1171,8 +1125,6 @@ static AppLayerResult DNP3ParseRequest(Flow *f, void *state, AppLayerParserState
         /* Not all data was processed, buffer it. */
         if (input_len) {
             if (!DNP3BufferAdd(buffer, input, input_len)) {
-                SCLogError(SC_ERR_MEM_ALLOC,
-                    "Failed to allocate memory to buffer DNP3 request data");
                 goto error;
             }
         }
@@ -1271,8 +1223,7 @@ error:
  * See DNP3ParseResponsePDUs for DNP3 frame handling.
  */
 static AppLayerResult DNP3ParseResponse(Flow *f, void *state, AppLayerParserState *pstate,
-    const uint8_t *input, uint32_t input_len, void *local_data,
-    const uint8_t flags)
+        StreamSlice stream_slice, void *local_data)
 {
     SCEnter();
 
@@ -1280,10 +1231,11 @@ static AppLayerResult DNP3ParseResponse(Flow *f, void *state, AppLayerParserStat
     DNP3Buffer *buffer = &dnp3->response_buffer;
     int processed;
 
+    const uint8_t *input = StreamSliceGetData(&stream_slice);
+    uint32_t input_len = StreamSliceGetDataLen(&stream_slice);
+
     if (buffer->len) {
         if (!DNP3BufferAdd(buffer, input, input_len)) {
-            SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate memory to buffer "
-                "DNP3 response data");
             goto error;
         }
         processed = DNP3HandleResponseLinkLayer(dnp3,
@@ -1312,8 +1264,6 @@ static AppLayerResult DNP3ParseResponse(Flow *f, void *state, AppLayerParserStat
         /* Not all data was processed, buffer it. */
         if (input_len) {
             if (!DNP3BufferAdd(buffer, input, input_len)) {
-                SCLogError(SC_ERR_MEM_ALLOC,
-                    "Failed to allocate memory to buffer DNP3 response data");
                 goto error;
             }
         }
@@ -1329,11 +1279,6 @@ error:
     SCReturnStruct(APP_LAYER_ERROR);
 }
 
-static AppLayerDecoderEvents *DNP3GetEvents(void *tx)
-{
-    return ((DNP3Transaction *) tx)->decoder_events;
-}
-
 static void *DNP3GetTx(void *alstate, uint64_t tx_id)
 {
     SCEnter();
@@ -1382,22 +1327,17 @@ static void DNP3TxFree(DNP3Transaction *tx)
 {
     SCEnter();
 
-    if (tx->request_buffer != NULL) {
-        SCFree(tx->request_buffer);
+    if (tx->buffer != NULL) {
+        SCFree(tx->buffer);
     }
 
-    if (tx->response_buffer != NULL) {
-        SCFree(tx->response_buffer);
+    AppLayerDecoderEventsFreeEvents(&tx->tx_data.events);
+
+    if (tx->tx_data.de_state != NULL) {
+        DetectEngineStateFree(tx->tx_data.de_state);
     }
 
-    AppLayerDecoderEventsFreeEvents(&tx->decoder_events);
-
-    if (tx->de_state != NULL) {
-        DetectEngineStateFree(tx->de_state);
-    }
-
-    DNP3TxFreeObjectList(&tx->request_objects);
-    DNP3TxFreeObjectList(&tx->response_objects);
+    DNP3TxFreeObjectList(&tx->objects);
 
     SCFree(tx);
     SCReturn;
@@ -1426,11 +1366,10 @@ static void DNP3StateTxFree(void *state, uint64_t tx_id)
             dnp3->curr = NULL;
         }
 
-        if (tx->decoder_events != NULL) {
-            if (tx->decoder_events->cnt <= dnp3->events) {
-                dnp3->events -= tx->decoder_events->cnt;
-            }
-            else {
+        if (tx->tx_data.events != NULL) {
+            if (tx->tx_data.events->cnt <= dnp3->events) {
+                dnp3->events -= tx->tx_data.events->cnt;
+            } else {
                 dnp3->events = 0;
             }
         }
@@ -1489,24 +1428,12 @@ static int DNP3GetAlstateProgress(void *tx, uint8_t direction)
         SCReturnInt(1);
     }
 
-    if (direction & STREAM_TOCLIENT && dnp3tx->response_done) {
+    if (dnp3tx->complete)
         retval = 1;
-    }
-    else if (direction & STREAM_TOSERVER && dnp3tx->request_done) {
-        retval = 1;
-    }
 
     SCReturnInt(retval);
 }
 
-/**
- * \brief App-layer support.
- */
-static int DNP3GetAlstateProgressCompletionStatus(uint8_t direction)
-{
-    return 1;
-}
-
 /**
  * \brief App-layer support.
  */
@@ -1515,8 +1442,9 @@ static int DNP3StateGetEventInfo(const char *event_name, int *event_id,
 {
     *event_id = SCMapEnumNameToValue(event_name, dnp3_decoder_event_table);
     if (*event_id == -1) {
-        SCLogError(SC_ERR_INVALID_ENUM_MAP, "Event \"%s\" not present in "
-            "the DNP3 enum event map table.", event_name);
+        SCLogError("Event \"%s\" not present in "
+                   "the DNP3 enum event map table.",
+                event_name);
         return -1;
     }
 
@@ -1533,8 +1461,9 @@ static int DNP3StateGetEventInfoById(int event_id, const char **event_name,
 {
     *event_name = SCMapEnumValueToName(event_id, dnp3_decoder_event_table);
     if (*event_name == NULL) {
-        SCLogError(SC_ERR_INVALID_ENUM_MAP, "Event \"%d\" not present in "
-            "the DNP3 enum event map table.", event_id);
+        SCLogError("Event \"%d\" not present in "
+                   "the DNP3 enum event map table.",
+                event_id);
         return -1;
     }
 
@@ -1543,31 +1472,18 @@ static int DNP3StateGetEventInfoById(int event_id, const char **event_name,
     return 0;
 }
 
-/**
- * \brief App-layer support.
- */
-static DetectEngineState *DNP3GetTxDetectState(void *vtx)
-{
-    DNP3Transaction *tx = vtx;
-    return tx->de_state;
-}
-
-/**
- * \brief App-layer support.
- */
-static int DNP3SetTxDetectState(void *vtx, DetectEngineState *s)
-{
-    DNP3Transaction *tx = vtx;
-    tx->de_state = s;
-    return 0;
-}
-
 static AppLayerTxData *DNP3GetTxData(void *vtx)
 {
     DNP3Transaction *tx = (DNP3Transaction *)vtx;
     return &tx->tx_data;
 }
 
+static AppLayerStateData *DNP3GetStateData(void *vstate)
+{
+    DNP3State *state = (DNP3State *)vstate;
+    return &state->state_data;
+}
+
 /**
  * \brief Check if the prefix code is a size prefix.
  *
@@ -1629,8 +1545,7 @@ void RegisterDNP3Parsers(void)
 
     const char *proto_name = "dnp3";
 
-    if (AppLayerProtoDetectConfProtoDetectionEnabled("tcp", proto_name))
-    {
+    if (AppLayerProtoDetectConfProtoDetectionEnabledDefault("tcp", proto_name, false)) {
         AppLayerProtoDetectRegisterProtocol(ALPROTO_DNP3, proto_name);
 
         if (RunmodeIsUnittests()) {
@@ -1646,8 +1561,7 @@ void RegisterDNP3Parsers(void)
             }
         }
 
-    }
-    else {
+    } else {
         SCLogConfig("Protocol detection and parser disabled for DNP3.");
         SCReturn;
     }
@@ -1664,11 +1578,6 @@ void RegisterDNP3Parsers(void)
         AppLayerParserRegisterStateFuncs(IPPROTO_TCP, ALPROTO_DNP3,
             DNP3StateAlloc, DNP3StateFree);
 
-        AppLayerParserRegisterGetEventsFunc(IPPROTO_TCP, ALPROTO_DNP3,
-            DNP3GetEvents);
-        AppLayerParserRegisterDetectStateFuncs(IPPROTO_TCP, ALPROTO_DNP3,
-            DNP3GetTxDetectState, DNP3SetTxDetectState);
-
         AppLayerParserRegisterGetTx(IPPROTO_TCP, ALPROTO_DNP3, DNP3GetTx);
         AppLayerParserRegisterGetTxIterator(IPPROTO_TCP, ALPROTO_DNP3, DNP3GetTxIterator);
         AppLayerParserRegisterGetTxCnt(IPPROTO_TCP, ALPROTO_DNP3, DNP3GetTxCnt);
@@ -1677,8 +1586,7 @@ void RegisterDNP3Parsers(void)
 
         AppLayerParserRegisterGetStateProgressFunc(IPPROTO_TCP, ALPROTO_DNP3,
             DNP3GetAlstateProgress);
-        AppLayerParserRegisterGetStateProgressCompletionStatus(ALPROTO_DNP3,
-            DNP3GetAlstateProgressCompletionStatus);
+        AppLayerParserRegisterStateProgressCompletionStatus(ALPROTO_DNP3, 1, 1);
 
         AppLayerParserRegisterGetEventInfo(IPPROTO_TCP, ALPROTO_DNP3,
             DNP3StateGetEventInfo);
@@ -1687,6 +1595,7 @@ void RegisterDNP3Parsers(void)
 
         AppLayerParserRegisterTxDataFunc(IPPROTO_TCP, ALPROTO_DNP3,
             DNP3GetTxData);
+        AppLayerParserRegisterStateDataFunc(IPPROTO_TCP, ALPROTO_DNP3, DNP3GetStateData);
     }
     else {
         SCLogConfig("Parser disabled for protocol %s. "
@@ -2177,7 +2086,7 @@ static int DNP3ParserTestRequestResponse(void)
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
 
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     SCMutexLock(&flow.m);
     FAIL_IF(AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
@@ -2191,20 +2100,22 @@ static int DNP3ParserTestRequestResponse(void)
     FAIL_IF(tx == NULL);
     FAIL_IF(tx->tx_num != 1);
     FAIL_IF(tx != state->curr);
-    FAIL_IF(tx->request_buffer == NULL);
-    FAIL_IF(tx->request_buffer_len != 20);
-    FAIL_IF(tx->request_ah.function_code != DNP3_APP_FC_DIR_OPERATE);
+    FAIL_IF(tx->buffer == NULL);
+    FAIL_IF(tx->buffer_len != 20);
+    FAIL_IF(tx->ah.function_code != DNP3_APP_FC_DIR_OPERATE);
 
     SCMutexLock(&flow.m);
     FAIL_IF(AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
             STREAM_TOCLIENT, response, sizeof(response)));
     SCMutexUnlock(&flow.m);
-    FAIL_IF(DNP3GetTx(state, 0) != tx);
-    FAIL_IF(!tx->response_done);
-    FAIL_IF(tx->response_buffer == NULL);
+    DNP3Transaction *tx0 = DNP3GetTx(state, 1);
+    FAIL_IF(tx0 == NULL);
+    FAIL_IF(tx0 == tx);
+    FAIL_IF(!tx0->done);
+    FAIL_IF(tx0->buffer == NULL);
 
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
@@ -2244,7 +2155,7 @@ static int DNP3ParserTestUnsolicitedResponseConfirm(void)
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
 
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     SCMutexLock(&flow.m);
     FAIL_IF(AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
@@ -2258,28 +2169,31 @@ static int DNP3ParserTestUnsolicitedResponseConfirm(void)
     FAIL_IF(tx == NULL);
     FAIL_IF(tx->tx_num != 1);
     FAIL_IF(tx != state->curr);
-    FAIL_IF(tx->request_buffer != NULL);
-    FAIL_IF(tx->response_buffer == NULL);
-    FAIL_IF(!tx->response_done);
-    FAIL_IF(tx->response_ah.function_code != DNP3_APP_FC_UNSOLICITED_RESP);
+    FAIL_IF(!tx->done);
+    FAIL_IF(tx->ah.function_code != DNP3_APP_FC_UNSOLICITED_RESP);
 
     SCMutexLock(&flow.m);
     FAIL_IF(AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
             STREAM_TOSERVER, confirm, sizeof(confirm)));
     SCMutexUnlock(&flow.m);
-    FAIL_IF(DNP3GetTx(state, 0) != tx);
-    FAIL_IF(!tx->response_done);
-    FAIL_IF(tx->response_buffer == NULL);
-    /* FAIL_IF(tx->iin1 != 0 || tx->iin2 != 0); */
+
+    /* Confirms are ignored currently.  With the move to
+       unidirectional transactions it might be easy to support these
+       now. */
+    DNP3Transaction *resptx = DNP3GetTx(state, 1);
+    FAIL_IF(resptx);
 
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
 
 /**
  * \test Test flood state.
+ *
+ * Note that flood state needs to revisited with the modification to a
+ * unidirectional protocol.
  */
 static int DNP3ParserTestFlooded(void)
 {
@@ -2310,7 +2224,7 @@ static int DNP3ParserTestFlooded(void)
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
 
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     SCMutexLock(&flow.m);
     FAIL_IF(AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
@@ -2324,10 +2238,11 @@ static int DNP3ParserTestFlooded(void)
     FAIL_IF(tx == NULL);
     FAIL_IF(tx->tx_num != 1);
     FAIL_IF(tx != state->curr);
-    FAIL_IF(tx->request_buffer == NULL);
-    FAIL_IF(tx->request_buffer_len != 20);
-    /* FAIL_IF(tx->app_function_code != DNP3_APP_FC_DIR_OPERATE); */
-    FAIL_IF(tx->response_done);
+    FAIL_IF(tx->buffer == NULL);
+    FAIL_IF(tx->buffer_len != 20);
+    FAIL_IF(tx->ah.function_code != DNP3_APP_FC_DIR_OPERATE);
+    FAIL_IF_NOT(tx->done);
+    FAIL_IF_NOT(DNP3GetAlstateProgress(tx, STREAM_TOSERVER));
 
     for (int i = 0; i < DNP3_DEFAULT_REQ_FLOOD_COUNT - 1; i++) {
         SCMutexLock(&flow.m);
@@ -2336,7 +2251,7 @@ static int DNP3ParserTestFlooded(void)
         SCMutexUnlock(&flow.m);
     }
     FAIL_IF(state->flooded);
-    FAIL_IF(DNP3GetAlstateProgress(tx, 0));
+    FAIL_IF_NOT(DNP3GetAlstateProgress(tx, STREAM_TOSERVER));
 
     /* One more request should trip us into flooded state. */
     SCMutexLock(&flow.m);
@@ -2348,11 +2263,8 @@ static int DNP3ParserTestFlooded(void)
     /* Progress for the oldest tx should return 1. */
     FAIL_IF(!DNP3GetAlstateProgress(tx, 0));
 
-    /* But progress for the current state should still return 0. */
-    FAIL_IF(DNP3GetAlstateProgress(state->curr, 0));
-
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
@@ -2415,7 +2327,7 @@ static int DNP3ParserTestPartialFrame(void)
     flow.protoctx = (void *)&ssn;
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     /* Pass in the first partial frame. */
 
@@ -2454,9 +2366,9 @@ static int DNP3ParserTestPartialFrame(void)
     FAIL_IF(tx == NULL);
     FAIL_IF(tx->tx_num != 1);
     FAIL_IF(tx != state->curr);
-    FAIL_IF(tx->request_buffer == NULL);
-    FAIL_IF(tx->request_buffer_len != 20);
-    FAIL_IF(tx->request_ah.function_code != DNP3_APP_FC_DIR_OPERATE);
+    FAIL_IF(tx->buffer == NULL);
+    FAIL_IF(tx->buffer_len != 20);
+    FAIL_IF(tx->ah.function_code != DNP3_APP_FC_DIR_OPERATE);
 
     /* Send partial response. */
     SCMutexLock(&flow.m);
@@ -2466,7 +2378,8 @@ static int DNP3ParserTestPartialFrame(void)
     FAIL_IF(r != 0);
     FAIL_IF(state->response_buffer.len != sizeof(response_partial1));
     FAIL_IF(state->response_buffer.offset != 0);
-    FAIL_IF(tx->response_done);
+    tx = DNP3GetTx(state, 1);
+    FAIL_IF_NOT_NULL(tx);
 
     /* Send rest of response. */
     SCMutexLock(&flow.m);
@@ -2479,13 +2392,14 @@ static int DNP3ParserTestPartialFrame(void)
     FAIL_IF(state->response_buffer.len != 0);
     FAIL_IF(state->response_buffer.offset != 0);
 
-    /* Transaction should be replied to now. */
-    FAIL_IF(!tx->response_done);
-    FAIL_IF(tx->response_buffer == NULL);
-    FAIL_IF(tx->response_buffer_len == 0);
+    /* There should now be a response transaction. */
+    tx = DNP3GetTx(state, 1);
+    FAIL_IF_NULL(tx);
+    FAIL_IF(tx->buffer == NULL);
+    FAIL_IF(tx->buffer_len == 0);
 
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
@@ -2528,7 +2442,7 @@ static int DNP3ParserTestMultiFrame(void)
     flow.protoctx = (void *)&ssn;
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     SCMutexLock(&flow.m);
     r = AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
@@ -2541,7 +2455,7 @@ static int DNP3ParserTestMultiFrame(void)
     FAIL_IF(state->transaction_max != 2);
 
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
@@ -2569,9 +2483,9 @@ static int DNP3ParserTestParsePDU01(void)
     FAIL_IF(pdus < 1);
     DNP3Transaction *dnp3tx = DNP3GetTx(dnp3state, 0);
     FAIL_IF_NULL(dnp3tx);
-    FAIL_IF(!dnp3tx->has_request);
-    FAIL_IF(TAILQ_EMPTY(&dnp3tx->request_objects));
-    DNP3Object *object = TAILQ_FIRST(&dnp3tx->request_objects);
+    FAIL_IF(!dnp3tx->is_request);
+    FAIL_IF(TAILQ_EMPTY(&dnp3tx->objects));
+    DNP3Object *object = TAILQ_FIRST(&dnp3tx->objects);
     FAIL_IF(object->group != 1 || object->variation != 0);
     FAIL_IF(object->count != 0);
 
@@ -2609,8 +2523,8 @@ static int DNP3ParserDecodeG70V3Test(void)
     FAIL_IF(bytes != sizeof(pkt));
     DNP3Transaction *tx = DNP3GetTx(dnp3state, 0);
     FAIL_IF_NULL(tx);
-    FAIL_IF_NOT(tx->has_request);
-    DNP3Object *obj = TAILQ_FIRST(&tx->request_objects);
+    FAIL_IF_NOT(tx->is_request);
+    DNP3Object *obj = TAILQ_FIRST(&tx->objects);
     FAIL_IF_NULL(obj);
     FAIL_IF_NOT(obj->group == 70);
     FAIL_IF_NOT(obj->variation == 3);
@@ -2691,7 +2605,7 @@ static int DNP3ParserIncorrectUserData(void)
     flow.protoctx = (void *)&ssn;
     flow.proto = IPPROTO_TCP;
     flow.alproto = ALPROTO_DNP3;
-    StreamTcpInitConfig(TRUE);
+    StreamTcpInitConfig(true);
 
     int r = AppLayerParserParse(NULL, alp_tctx, &flow, ALPROTO_DNP3,
                                 STREAM_TOCLIENT, packet_bytes, sizeof(packet_bytes));
@@ -2699,7 +2613,7 @@ static int DNP3ParserIncorrectUserData(void)
     FAIL_IF(r == 0);
 
     AppLayerParserThreadCtxFree(alp_tctx);
-    StreamTcpFreeConfig(TRUE);
+    StreamTcpFreeConfig(true);
     FLOW_DESTROY(&flow);
     PASS;
 }
