@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2013 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -439,7 +439,13 @@ const struct DecodeEvents_ DEvents[] = {
             SCTP_PKT_TOO_SMALL,
     },
 
-    /* Fragmentation reasembly events. */
+    /* ESP EVENTS */
+    {
+            "decoder.esp.pkt_too_small",
+            ESP_PKT_TOO_SMALL,
+    },
+
+    /* Fragmentation reassembly events. */
     {
             "decoder.ipv4.frag_pkt_too_large",
             IPV4_FRAG_PKT_TOO_LARGE,
@@ -553,6 +559,31 @@ const struct DecodeEvents_ DEvents[] = {
             CHDLC_PKT_TOO_SMALL,
     },
 
+    /* NSH events */
+    {
+            "decoder.nsh.header_too_small",
+            NSH_HEADER_TOO_SMALL,
+    },
+    {
+            "decoder.nsh.unsupported_version",
+            NSH_UNSUPPORTED_VERSION,
+    },
+    {
+            "decoder.nsh.bad_header_length",
+            NSH_BAD_HEADER_LENGTH,
+    },
+    {
+            "decoder.nsh.reserved_type",
+            NSH_RESERVED_TYPE,
+    },
+    {
+            "decoder.nsh.unsupported_type",
+            NSH_UNSUPPORTED_TYPE,
+    },
+    {
+            "decoder.nsh.unknown_payload",
+            NSH_UNKNOWN_PAYLOAD,
+    },
     {
             "decoder.too_many_layers",
             GENERIC_TOO_MANY_LAYERS,
@@ -787,6 +818,10 @@ const struct DecodeEvents_ DEvents[] = {
             "stream.pkt_retransmission",
             STREAM_PKT_RETRANSMISSION,
     },
+    {
+            "stream.pkt_spurious_retransmission",
+            STREAM_PKT_SPURIOUS_RETRANSMISSION,
+    },
     {
             "stream.pkt_bad_window_update",
             STREAM_PKT_BAD_WINDOW_UPDATE,
@@ -817,6 +852,22 @@ const struct DecodeEvents_ DEvents[] = {
             "stream.reassembly_overlap_different_data",
             STREAM_REASSEMBLY_OVERLAP_DIFFERENT_DATA,
     },
+    {
+            "stream.reassembly_depth_reached",
+            STREAM_REASSEMBLY_DEPTH_REACHED,
+    },
+    {
+            "stream.reassembly_insert_memcap",
+            STREAM_REASSEMBLY_INSERT_MEMCAP,
+    },
+    {
+            "stream.reassembly_insert_limit",
+            STREAM_REASSEMBLY_INSERT_LIMIT,
+    },
+    {
+            "stream.reassembly_insert_invalid",
+            STREAM_REASSEMBLY_INSERT_INVALID,
+    },
 
     { NULL, 0 },
 };
