@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2022 Open Information Security Foundation
+/* Copyright (C) 2007-2023 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -32,19 +32,22 @@
 #include "threadvars.h"
 #include "util-debug.h"
 #include "decode-events.h"
+#ifdef PROFILING
 #include "flow-worker.h"
+#include "app-layer-protos.h"
+#endif
 
 #ifdef HAVE_NAPATECH
 #include "util-napatech.h"
 #endif /* HAVE_NAPATECH */
 
-
 typedef enum {
     CHECKSUM_VALIDATION_DISABLE,
     CHECKSUM_VALIDATION_ENABLE,
     CHECKSUM_VALIDATION_AUTO,
     CHECKSUM_VALIDATION_RXONLY,
     CHECKSUM_VALIDATION_KERNEL,
+    CHECKSUM_VALIDATION_OFFLOAD,
 } ChecksumValidationMode;
 
 enum PktSrcEnum {
@@ -60,6 +63,7 @@ enum PktSrcEnum {
     PKT_SRC_DETECT_RELOAD_FLUSH,
     PKT_SRC_CAPTURE_TIMEOUT,
     PKT_SRC_DECODER_GENEVE,
+    PKT_SRC_SHUTDOWN_FLUSH,
 };
 
 #include "source-nflog.h"
@@ -69,20 +73,20 @@ enum PktSrcEnum {
 #include "source-af-packet.h"
 #include "source-netmap.h"
 #include "source-windivert.h"
+#ifdef HAVE_DPDK
+#include "source-dpdk.h"
+#endif
 #ifdef HAVE_PF_RING_FLOW_OFFLOAD
 #include "source-pfring.h"
 #endif
+#ifdef HAVE_AF_XDP
+#include "source-af-xdp.h"
+#endif
 
-#include "action-globals.h"
-
-#include "decode-erspan.h"
 #include "decode-ethernet.h"
-#include "decode-chdlc.h"
 #include "decode-gre.h"
-#include "decode-geneve.h"
 #include "decode-ppp.h"
 #include "decode-pppoe.h"
-#include "decode-sll.h"
 #include "decode-ipv4.h"
 #include "decode-ipv6.h"
 #include "decode-icmpv4.h"
@@ -90,16 +94,10 @@ enum PktSrcEnum {
 #include "decode-tcp.h"
 #include "decode-udp.h"
 #include "decode-sctp.h"
-#include "decode-raw.h"
-#include "decode-null.h"
+#include "decode-esp.h"
 #include "decode-vlan.h"
-#include "decode-vntag.h"
-#include "decode-vxlan.h"
 #include "decode-mpls.h"
 
-#include "detect-reference.h"
-
-#include "app-layer-protos.h"
 
 /* forward declarations */
 struct DetectionEngineThreadCtx_;
@@ -158,15 +156,6 @@ typedef struct Address_ {
         (a)->addr_data32[3] = 0;                                  \
     } while (0)
 
-/* clear the address structure by setting all fields to 0 */
-#define CLEAR_ADDR(a) do {       \
-        (a)->family = 0;         \
-        (a)->addr_data32[0] = 0; \
-        (a)->addr_data32[1] = 0; \
-        (a)->addr_data32[2] = 0; \
-        (a)->addr_data32[3] = 0; \
-    } while (0)
-
 /* Set the IPv6 addresses into the Addrs of the Packet.
  * Make sure p->ip6h is initialized and validated. */
 #define SET_IPV6_SRC_ADDR(p, a) do {                    \
@@ -215,7 +204,6 @@ typedef struct Address_ {
     } while (0)
 
 
-
 #define GET_IPV4_SRC_ADDR_U32(p) ((p)->src.addr_data32[0])
 #define GET_IPV4_DST_ADDR_U32(p) ((p)->dst.addr_data32[0])
 #define GET_IPV4_SRC_ADDR_PTR(p) ((p)->src.addr_data32)
@@ -237,7 +225,6 @@ typedef struct Address_ {
     (p)->pktlen = (len); \
     } while (0)
 
-
 /* Port is just a uint16_t */
 typedef uint16_t Port;
 #define SET_PORT(v, p) ((p) = (v))
@@ -279,6 +266,7 @@ typedef struct PacketAlert_ {
     uint8_t flags;
     const struct Signature_ *s;
     uint64_t tx_id; /* Used for sorting */
+    int64_t frame_id;
 } PacketAlert;
 
 /* flag to indicate the rule action (drop/pass) needs to be applied to the flow */
@@ -291,6 +279,8 @@ typedef struct PacketAlert_ {
 #define PACKET_ALERT_FLAG_TX            0x08
 /** action was changed by rate_filter */
 #define PACKET_ALERT_RATE_FILTER_MODIFIED   0x10
+/** alert is in a frame, frame_id set */
+#define PACKET_ALERT_FLAG_FRAME 0x20
 
 extern uint16_t packet_alert_max;
 #define PACKET_ALERT_MAX 15
@@ -411,6 +401,8 @@ enum PacketDropReason {
     PKT_DROP_REASON_STREAM_ERROR,
     PKT_DROP_REASON_STREAM_MEMCAP,
     PKT_DROP_REASON_STREAM_MIDSTREAM,
+    PKT_DROP_REASON_NFQ_ERROR,    /**< no nfq verdict, must be error */
+    PKT_DROP_REASON_INNER_PACKET, /**< drop issued by inner (tunnel) packet */
 };
 
 /* forward declaration since Packet struct definition requires this */
@@ -460,13 +452,15 @@ typedef struct Packet_
      * has the exact same tuple as the lower levels */
     uint8_t recursion_level;
 
-    uint16_t vlan_id[2];
+    uint16_t vlan_id[VLAN_MAX_LAYERS];
     uint8_t vlan_idx;
 
     /* flow */
     uint8_t flowflags;
     /* coccinelle: Packet:flowflags:FLOW_PKT_ */
 
+    uint8_t app_update_direction; // enum StreamUpdateDir
+
     /* Pkt Flags */
     uint32_t flags;
 
@@ -476,7 +470,7 @@ typedef struct Packet_
      * hash size still */
     uint32_t flow_hash;
 
-    struct timeval ts;
+    SCTime_t ts;
 
     union {
         /* nfq stuff */
@@ -503,7 +497,15 @@ typedef struct Packet_
 #ifdef WINDIVERT
         WinDivertPacketVars windivert_v;
 #endif /* WINDIVERT */
-
+#ifdef HAVE_DPDK
+        DPDKPacketVars dpdk_v;
+#endif
+#ifdef HAVE_NAPATECH
+        NapatechPacketVars ntpv;
+#endif
+#ifdef HAVE_AF_XDP
+        AFXDPPacketVars afxdp_v;
+#endif
         /* A chunk of memory that a plugin can use for its packet vars. */
         uint8_t plugin_v[PLUGIN_VAR_SIZE];
 
@@ -556,6 +558,8 @@ typedef struct Packet_
 
     SCTPHdr *sctph;
 
+    ESPHdr *esph;
+
     ICMPV4Hdr *icmpv4h;
 
     ICMPV6Hdr *icmpv6h;
@@ -619,11 +623,6 @@ typedef struct Packet_
                            * It should always point to the lowest
                            * packet in a encapsulated packet */
 
-    /** mutex to protect access to:
-     *  - tunnel_rtv_cnt
-     *  - tunnel_tpr_cnt
-     */
-    SCMutex tunnel_mutex;
     /* ready to set verdict counter, only set in root */
     uint16_t tunnel_rtv_cnt;
     /* tunnel packet ref count */
@@ -640,13 +639,19 @@ typedef struct Packet_
 #ifdef PROFILING
     PktProfiling *profile;
 #endif
-#ifdef HAVE_NAPATECH
-    NapatechPacketVars ntpv;
-#endif
+    /* things in the packet that live beyond a reinit */
+    struct {
+        /** lock to protect access to:
+         *  - tunnel_rtv_cnt
+         *  - tunnel_tpr_cnt
+         *  - nfq_v.mark
+         *  - flags
+         */
+        SCSpinlock tunnel_lock;
+    } persistent;
 } Packet;
 
 /** highest mtu of the interfaces we monitor */
-extern int g_default_mtu;
 #define DEFAULT_MTU 1500
 #define MINIMUM_MTU 68      /**< ipv4 minimum: rfc791 */
 
@@ -683,16 +688,20 @@ typedef struct DecodeThreadVars_
     uint16_t counter_udp;
     uint16_t counter_icmpv4;
     uint16_t counter_icmpv6;
+    uint16_t counter_arp;
+    uint16_t counter_ethertype_unknown;
 
     uint16_t counter_sll;
     uint16_t counter_raw;
     uint16_t counter_null;
     uint16_t counter_sctp;
+    uint16_t counter_esp;
     uint16_t counter_ppp;
     uint16_t counter_geneve;
     uint16_t counter_gre;
     uint16_t counter_vlan;
     uint16_t counter_vlan_qinq;
+    uint16_t counter_vlan_qinqinq;
     uint16_t counter_vxlan;
     uint16_t counter_vntag;
     uint16_t counter_ieee8021ah;
@@ -702,18 +711,20 @@ typedef struct DecodeThreadVars_
     uint16_t counter_ipv4inipv6;
     uint16_t counter_ipv6inipv6;
     uint16_t counter_erspan;
+    uint16_t counter_nsh;
 
     /** frag stats - defrag runs in the context of the decoder. */
     uint16_t counter_defrag_ipv4_fragments;
     uint16_t counter_defrag_ipv4_reassembled;
-    uint16_t counter_defrag_ipv4_timeouts;
     uint16_t counter_defrag_ipv6_fragments;
     uint16_t counter_defrag_ipv6_reassembled;
-    uint16_t counter_defrag_ipv6_timeouts;
     uint16_t counter_defrag_max_hit;
 
     uint16_t counter_flow_memcap;
 
+    uint16_t counter_tcp_active_sessions;
+    uint16_t counter_flow_total;
+    uint16_t counter_flow_active;
     uint16_t counter_flow_tcp;
     uint16_t counter_flow_udp;
     uint16_t counter_flow_icmp4;
@@ -772,176 +783,17 @@ void CaptureStatsSetup(ThreadVars *tv, CaptureStats *s);
         }                                           \
     } while(0)
 
-/**
- *  \brief Initialize a packet structure for use.
- */
-#define PACKET_INITIALIZE(p)                                                                       \
-    {                                                                                              \
-        SCMutexInit(&(p)->tunnel_mutex, NULL);                                                     \
-        (p)->alerts.alerts = PacketAlertCreate();                                                  \
-        PACKET_RESET_CHECKSUMS((p));                                                               \
-        (p)->livedev = NULL;                                                                       \
-    }
-
-#define PACKET_RELEASE_REFS(p) do {              \
-        FlowDeReference(&((p)->flow));          \
-        HostDeReference(&((p)->host_src));      \
-        HostDeReference(&((p)->host_dst));      \
-    } while (0)
-
-/**
- *  \brief Recycle a packet structure for reuse.
- */
-#define PACKET_REINIT(p)                                                                           \
-    do {                                                                                           \
-        CLEAR_ADDR(&(p)->src);                                                                     \
-        CLEAR_ADDR(&(p)->dst);                                                                     \
-        (p)->sp = 0;                                                                               \
-        (p)->dp = 0;                                                                               \
-        (p)->proto = 0;                                                                            \
-        (p)->recursion_level = 0;                                                                  \
-        PACKET_FREE_EXTDATA((p));                                                                  \
-        (p)->flags = (p)->flags & PKT_ALLOC;                                                       \
-        (p)->flowflags = 0;                                                                        \
-        (p)->pkt_src = 0;                                                                          \
-        (p)->vlan_id[0] = 0;                                                                       \
-        (p)->vlan_id[1] = 0;                                                                       \
-        (p)->vlan_idx = 0;                                                                         \
-        (p)->ts.tv_sec = 0;                                                                        \
-        (p)->ts.tv_usec = 0;                                                                       \
-        (p)->datalink = 0;                                                                         \
-        (p)->drop_reason = 0;                                                                      \
-        (p)->action = 0;                                                                           \
-        if ((p)->pktvar != NULL) {                                                                 \
-            PktVarFree((p)->pktvar);                                                               \
-            (p)->pktvar = NULL;                                                                    \
-        }                                                                                          \
-        (p)->ethh = NULL;                                                                          \
-        if ((p)->ip4h != NULL) {                                                                   \
-            CLEAR_IPV4_PACKET((p));                                                                \
-        }                                                                                          \
-        if ((p)->ip6h != NULL) {                                                                   \
-            CLEAR_IPV6_PACKET((p));                                                                \
-        }                                                                                          \
-        if ((p)->tcph != NULL) {                                                                   \
-            CLEAR_TCP_PACKET((p));                                                                 \
-        }                                                                                          \
-        if ((p)->udph != NULL) {                                                                   \
-            CLEAR_UDP_PACKET((p));                                                                 \
-        }                                                                                          \
-        if ((p)->sctph != NULL) {                                                                  \
-            CLEAR_SCTP_PACKET((p));                                                                \
-        }                                                                                          \
-        if ((p)->icmpv4h != NULL) {                                                                \
-            CLEAR_ICMPV4_PACKET((p));                                                              \
-        }                                                                                          \
-        if ((p)->icmpv6h != NULL) {                                                                \
-            CLEAR_ICMPV6_PACKET((p));                                                              \
-        }                                                                                          \
-        (p)->ppph = NULL;                                                                          \
-        (p)->pppoesh = NULL;                                                                       \
-        (p)->pppoedh = NULL;                                                                       \
-        (p)->greh = NULL;                                                                          \
-        (p)->payload = NULL;                                                                       \
-        (p)->payload_len = 0;                                                                      \
-        (p)->BypassPacketsFlow = NULL;                                                             \
-        (p)->pktlen = 0;                                                                           \
-        (p)->alerts.cnt = 0;                                                                       \
-        (p)->alerts.discarded = 0;                                                                 \
-        (p)->alerts.suppressed = 0;                                                                \
-        (p)->alerts.drop.action = 0;                                                               \
-        (p)->pcap_cnt = 0;                                                                         \
-        (p)->tunnel_rtv_cnt = 0;                                                                   \
-        (p)->tunnel_tpr_cnt = 0;                                                                   \
-        (p)->events.cnt = 0;                                                                       \
-        AppLayerDecoderEventsResetEvents((p)->app_layer_events);                                   \
-        (p)->next = NULL;                                                                          \
-        (p)->prev = NULL;                                                                          \
-        (p)->root = NULL;                                                                          \
-        (p)->livedev = NULL;                                                                       \
-        PACKET_RESET_CHECKSUMS((p));                                                               \
-        PACKET_PROFILING_RESET((p));                                                               \
-        p->tenant_id = 0;                                                                          \
-        p->nb_decoded_layers = 0;                                                                  \
-    } while (0)
-
-#define PACKET_RECYCLE(p) do { \
-        PACKET_RELEASE_REFS((p)); \
-        PACKET_REINIT((p)); \
-    } while (0)
-
-/**
- *  \brief Cleanup a packet so that we can free it. No memset needed..
- */
-#define PACKET_DESTRUCTOR(p)                                                                       \
-    do {                                                                                           \
-        PACKET_RELEASE_REFS((p));                                                                  \
-        if ((p)->pktvar != NULL) {                                                                 \
-            PktVarFree((p)->pktvar);                                                               \
-        }                                                                                          \
-        PacketAlertFree((p)->alerts.alerts);                                                       \
-        PACKET_FREE_EXTDATA((p));                                                                  \
-        SCMutexDestroy(&(p)->tunnel_mutex);                                                        \
-        AppLayerDecoderEventsFreeEvents(&(p)->app_layer_events);                                   \
-        PACKET_PROFILING_RESET((p));                                                               \
-    } while (0)
-
-/* macro's for setting the action
- * handle the case of a root packet
- * for tunnels */
-
-static inline void PacketSetActionOnCurrentPkt(Packet *p, const uint8_t action)
-{
-    p->action |= action;
-}
-
-static inline void PacketSetActionOnRealPkt(Packet *p, const uint8_t action)
-{
-    if (likely(p->root == NULL)) {
-        p->action |= action;
-    } else {
-        p->root->action |= action;
-    }
-}
-
-static inline void PacketDrop(Packet *p, const uint8_t action, enum PacketDropReason r)
-{
-    if (p->drop_reason == PKT_DROP_REASON_NOT_SET)
-        p->drop_reason = (uint8_t)r;
-
-    if (likely(p->root == NULL)) {
-        p->action |= action;
-    } else {
-        p->root->action |= action;
-    }
-}
-
-static inline uint8_t PacketTestActionOnCurrentPkt(const Packet *p, const uint8_t a)
-{
-    return (p->action & a);
-}
-
-static inline uint8_t PacketTestActionOnRealPkt(const Packet *p, const uint8_t a)
-{
-    if (likely(p->root == NULL)) {
-        return (p->action & a);
-    } else {
-        return (p->root->action & a);
-    }
-}
-
-// Tests on "real" packets. Should only be used to check for ACTION_DROP|ACTION_REJECT*
-#define PACKET_TEST_ACTION(p, a) PacketTestActionOnRealPkt((p), (a))
-
 #define TUNNEL_INCR_PKT_RTV_NOLOCK(p) do {                                          \
         ((p)->root ? (p)->root->tunnel_rtv_cnt++ : (p)->tunnel_rtv_cnt++);          \
     } while (0)
 
-#define TUNNEL_INCR_PKT_TPR(p) do {                                                 \
-        SCMutexLock((p)->root ? &(p)->root->tunnel_mutex : &(p)->tunnel_mutex);     \
-        ((p)->root ? (p)->root->tunnel_tpr_cnt++ : (p)->tunnel_tpr_cnt++);          \
-        SCMutexUnlock((p)->root ? &(p)->root->tunnel_mutex : &(p)->tunnel_mutex);   \
-    } while (0)
+static inline void TUNNEL_INCR_PKT_TPR(Packet *p)
+{
+    Packet *rp = p->root ? p->root : p;
+    SCSpinLock(&rp->persistent.tunnel_lock);
+    rp->tunnel_tpr_cnt++;
+    SCSpinUnlock(&rp->persistent.tunnel_lock);
+}
 
 #define TUNNEL_PKT_RTV(p) ((p)->root ? (p)->root->tunnel_rtv_cnt : (p)->tunnel_rtv_cnt)
 #define TUNNEL_PKT_TPR(p) ((p)->root ? (p)->root->tunnel_tpr_cnt : (p)->tunnel_tpr_cnt)
@@ -963,6 +815,7 @@ enum DecodeTunnelProto {
     DECODE_TUNNEL_IPV6,
     DECODE_TUNNEL_IPV6_TEREDO, /**< separate protocol for stricter error handling */
     DECODE_TUNNEL_PPP,
+    DECODE_TUNNEL_NSH,
     DECODE_TUNNEL_UNSET
 };
 
@@ -1007,18 +860,19 @@ int DecodeICMPV6(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, ui
 int DecodeTCP(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint16_t);
 int DecodeUDP(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint16_t);
 int DecodeSCTP(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint16_t);
+int DecodeESP(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint16_t);
 int DecodeGRE(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeVLAN(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeVNTag(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeIEEE8021ah(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeGeneve(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeVXLAN(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
-int DecodeVNTag(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeMPLS(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeERSPAN(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeERSPANTypeI(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeCHDLC(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 int DecodeTEMPLATE(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
+int DecodeNSH(ThreadVars *, DecodeThreadVars *, Packet *, const uint8_t *, uint32_t);
 
 #ifdef UNITTESTS
 void DecodeIPV6FragHeader(Packet *p, const uint8_t *pkt,
@@ -1034,30 +888,6 @@ void DecodeGlobalConfig(void);
 void PacketAlertGetMaxConfig(void);
 void DecodeUnregisterCounters(void);
 
-/** \brief Set the No payload inspection Flag for the packet.
- *
- * \param p Packet to set the flag in
- */
-#define DecodeSetNoPayloadInspectionFlag(p) do { \
-        (p)->flags |= PKT_NOPAYLOAD_INSPECTION;  \
-    } while (0)
-
-#define DecodeUnsetNoPayloadInspectionFlag(p) do { \
-        (p)->flags &= ~PKT_NOPAYLOAD_INSPECTION;  \
-    } while (0)
-
-/** \brief Set the No packet inspection Flag for the packet.
- *
- * \param p Packet to set the flag in
- */
-#define DecodeSetNoPacketInspectionFlag(p) do { \
-        (p)->flags |= PKT_NOPACKET_INSPECTION;  \
-    } while (0)
-#define DecodeUnsetNoPacketInspectionFlag(p) do { \
-        (p)->flags &= ~PKT_NOPACKET_INSPECTION;  \
-    } while (0)
-
-
 #define ENGINE_SET_EVENT(p, e) do { \
     SCLogDebug("p %p event %d", (p), e); \
     if ((p)->events.cnt < PACKET_ENGINE_EVENT_MAX) { \
@@ -1156,50 +986,75 @@ void DecodeUnregisterCounters(void);
 #define PPP_OVER_GRE         11
 #define VLAN_OVER_GRE        13
 
-/*Packet Flags*/
-#define PKT_NOPACKET_INSPECTION         (1)         /**< Flag to indicate that packet header or contents should not be inspected*/
-#define PKT_NOPAYLOAD_INSPECTION        (1<<2)      /**< Flag to indicate that packet contents should not be inspected*/
-#define PKT_ALLOC                       (1<<3)      /**< Packet was alloc'd this run, needs to be freed */
-#define PKT_HAS_TAG                     (1<<4)      /**< Packet has matched a tag */
-#define PKT_STREAM_ADD                  (1<<5)      /**< Packet payload was added to reassembled stream */
-#define PKT_STREAM_EST                  (1<<6)      /**< Packet is part of established stream */
-#define PKT_STREAM_EOF                  (1<<7)      /**< Stream is in eof state */
-#define PKT_HAS_FLOW                    (1<<8)
-#define PKT_PSEUDO_STREAM_END           (1<<9)      /**< Pseudo packet to end the stream */
-#define PKT_STREAM_MODIFIED             (1<<10)     /**< Packet is modified by the stream engine, we need to recalc the csum and reinject/replace */
-#define PKT_MARK_MODIFIED               (1<<11)     /**< Packet mark is modified */
-#define PKT_STREAM_NOPCAPLOG            (1<<12)     /**< Exclude packet from pcap logging as it's part of a stream that has reassembly depth reached. */
+/* Packet Flags */
 
-#define PKT_TUNNEL                      (1<<13)
-#define PKT_TUNNEL_VERDICTED            (1<<14)
+/** Flag to indicate that packet header or contents should not be inspected */
+#define PKT_NOPACKET_INSPECTION BIT_U32(0)
+// vacancy
 
-#define PKT_IGNORE_CHECKSUM             (1<<15)     /**< Packet checksum is not computed (TX packet for example) */
-#define PKT_ZERO_COPY                   (1<<16)     /**< Packet comes from zero copy (ext_pkt must not be freed) */
+/** Flag to indicate that packet contents should not be inspected */
+#define PKT_NOPAYLOAD_INSPECTION BIT_U32(2)
+// vacancy
 
-#define PKT_HOST_SRC_LOOKED_UP          (1<<17)
-#define PKT_HOST_DST_LOOKED_UP          (1<<18)
+/** Packet has matched a tag */
+#define PKT_HAS_TAG BIT_U32(4)
+/** Packet payload was added to reassembled stream */
+#define PKT_STREAM_ADD BIT_U32(5)
+/** Packet is part of established stream */
+#define PKT_STREAM_EST BIT_U32(6)
+/** Stream is in eof state */
+#define PKT_STREAM_EOF BIT_U32(7)
+#define PKT_HAS_FLOW   BIT_U32(8)
+/** Pseudo packet to end the stream */
+#define PKT_PSEUDO_STREAM_END BIT_U32(9)
+/** Packet is modified by the stream engine, we need to recalc the csum and       \
+                   reinject/replace */
+#define PKT_STREAM_MODIFIED BIT_U32(10)
+/** Packet mark is modified */
+#define PKT_MARK_MODIFIED BIT_U32(11)
+/** Exclude packet from pcap logging as it's part of a stream that has reassembly \
+                   depth reached. */
+#define PKT_STREAM_NOPCAPLOG BIT_U32(12)
 
-#define PKT_IS_FRAGMENT                 (1<<19)     /**< Packet is a fragment */
-#define PKT_IS_INVALID                  (1<<20)
-#define PKT_PROFILE                     (1<<21)
+#define PKT_TUNNEL           BIT_U32(13)
+#define PKT_TUNNEL_VERDICTED BIT_U32(14)
+
+/** Packet checksum is not computed (TX packet for example) */
+#define PKT_IGNORE_CHECKSUM BIT_U32(15)
+/** Packet comes from zero copy (ext_pkt must not be freed) */
+#define PKT_ZERO_COPY BIT_U32(16)
+
+#define PKT_HOST_SRC_LOOKED_UP BIT_U32(17)
+#define PKT_HOST_DST_LOOKED_UP BIT_U32(18)
+
+/** Packet is a fragment */
+#define PKT_IS_FRAGMENT BIT_U32(19)
+#define PKT_IS_INVALID  BIT_U32(20)
+#define PKT_PROFILE     BIT_U32(21)
 
 /** indication by decoder that it feels the packet should be handled by
  *  flow engine: Packet::flow_hash will be set */
-#define PKT_WANTS_FLOW                  (1<<22)
+#define PKT_WANTS_FLOW BIT_U32(22)
 
 /** protocol detection done */
-#define PKT_PROTO_DETECT_TS_DONE        (1<<23)
-#define PKT_PROTO_DETECT_TC_DONE        (1<<24)
+#define PKT_PROTO_DETECT_TS_DONE BIT_U32(23)
+#define PKT_PROTO_DETECT_TC_DONE BIT_U32(24)
 
-#define PKT_REBUILT_FRAGMENT            (1<<25)     /**< Packet is rebuilt from
-                                                     * fragments. */
-#define PKT_DETECT_HAS_STREAMDATA       (1<<26)     /**< Set by Detect() if raw stream data is available. */
+#define PKT_REBUILT_FRAGMENT                                                                       \
+    BIT_U32(25) /**< Packet is rebuilt from                                                        \
+                 * fragments. */
+#define PKT_DETECT_HAS_STREAMDATA                                                                  \
+    BIT_U32(26) /**< Set by Detect() if raw stream data is available. */
 
-#define PKT_PSEUDO_DETECTLOG_FLUSH      (1<<27)     /**< Detect/log flush for protocol upgrade */
+#define PKT_PSEUDO_DETECTLOG_FLUSH BIT_U32(27) /**< Detect/log flush for protocol upgrade */
 
 /** Packet is part of stream in known bad condition (loss, wrong thread),
  *  so flag it for not setting stream events */
-#define PKT_STREAM_NO_EVENTS            (1<<28)
+#define PKT_STREAM_NO_EVENTS BIT_U32(28)
+
+/** We had no alert on flow before this packet */
+#define PKT_FIRST_ALERTS BIT_U32(29)
+#define PKT_FIRST_TAG    BIT_U32(30)
 
 /** \brief return 1 if the packet is a pseudo packet */
 #define PKT_IS_PSEUDOPKT(p) \
@@ -1220,6 +1075,24 @@ static inline bool PacketIncreaseCheckLayers(Packet *p)
     return true;
 }
 
+/** \brief Set the No payload inspection Flag for the packet.
+ *
+ * \param p Packet to set the flag in
+ */
+static inline void DecodeSetNoPayloadInspectionFlag(Packet *p)
+{
+    p->flags |= PKT_NOPAYLOAD_INSPECTION;
+}
+
+/** \brief Set the No packet inspection Flag for the packet.
+ *
+ * \param p Packet to set the flag in
+ */
+static inline void DecodeSetNoPacketInspectionFlag(Packet *p)
+{
+    p->flags |= PKT_NOPACKET_INSPECTION;
+}
+
 /** \brief return true if *this* packet needs to trigger a verdict.
  *
  *  If we have the root packet, and we have none outstanding,
@@ -1233,8 +1106,8 @@ static inline bool PacketIncreaseCheckLayers(Packet *p)
 static inline bool VerdictTunnelPacket(Packet *p)
 {
     bool verdict = true;
-    SCMutex *m = p->root ? &p->root->tunnel_mutex : &p->tunnel_mutex;
-    SCMutexLock(m);
+    SCSpinlock *lock = p->root ? &p->root->persistent.tunnel_lock : &p->persistent.tunnel_lock;
+    SCSpinLock(lock);
     const uint16_t outstanding = TUNNEL_PKT_TPR(p) - TUNNEL_PKT_RTV(p);
     SCLogDebug("tunnel: outstanding %u", outstanding);
 
@@ -1248,7 +1121,7 @@ static inline bool VerdictTunnelPacket(Packet *p)
     } else {
         verdict = false;
     }
-    SCMutexUnlock(m);
+    SCSpinUnlock(lock);
     return verdict;
 }
 
@@ -1277,8 +1150,9 @@ static inline void DecodeLinkLayer(ThreadVars *tv, DecodeThreadVars *dtv,
             DecodeCHDLC(tv, dtv, p, data, len);
             break;
         default:
-            SCLogError(SC_ERR_DATALINK_UNIMPLEMENTED, "datalink type "
-                    "%"PRId32" not yet supported", datalink);
+            SCLogError("datalink type "
+                       "%" PRId32 " not yet supported",
+                    datalink);
             break;
     }
 }
@@ -1308,7 +1182,7 @@ static inline bool DecodeNetworkLayer(ThreadVars *tv, DecodeThreadVars *dtv,
         case ETHERNET_TYPE_VLAN:
         case ETHERNET_TYPE_8021AD:
         case ETHERNET_TYPE_8021QINQ:
-            if (p->vlan_idx >= 2) {
+            if (p->vlan_idx > VLAN_MAX_LAYER_IDX) {
                 ENGINE_SET_EVENT(p,VLAN_HEADER_TOO_MANY_LAYERS);
             } else {
                 DecodeVLAN(tv, dtv, p, data, len);
@@ -1318,6 +1192,7 @@ static inline bool DecodeNetworkLayer(ThreadVars *tv, DecodeThreadVars *dtv,
             DecodeIEEE8021ah(tv, dtv, p, data, len);
             break;
         case ETHERNET_TYPE_ARP:
+            StatsIncr(tv, dtv->counter_arp);
             break;
         case ETHERNET_TYPE_MPLS_UNICAST:
         case ETHERNET_TYPE_MPLS_MULTICAST:
@@ -1333,8 +1208,12 @@ static inline bool DecodeNetworkLayer(ThreadVars *tv, DecodeThreadVars *dtv,
         case ETHERNET_TYPE_VNTAG:
             DecodeVNTag(tv, dtv, p, data, len);
             break;
+        case ETHERNET_TYPE_NSH:
+            DecodeNSH(tv, dtv, p, data, len);
+            break;
         default:
             SCLogDebug("unknown ether type: %" PRIx16 "", proto);
+            StatsIncr(tv, dtv->counter_ethertype_unknown);
             return false;
     }
     return true;
