@@ -34,16 +34,22 @@
 #include "detect-engine-mpm.h"
 #include "conf.h"
 #include "detect-content.h"
+#include "detect-pcre.h"
+#include "detect-bytejump.h"
+#include "detect-bytetest.h"
 #include "detect-flow.h"
 #include "detect-tcp-flags.h"
 #include "feature.h"
 #include "util-print.h"
+#include "util-time.h"
+#include "util-validate.h"
+#include "util-conf.h"
 
 static int rule_warnings_only = 0;
 static FILE *rule_engine_analysis_FD = NULL;
 static FILE *fp_engine_analysis_FD = NULL;
-static pcre *percent_re = NULL;
-static pcre_extra *percent_re_study = NULL;
+static pcre2_code *percent_re = NULL;
+static pcre2_match_data *percent_re_match = NULL;
 static char log_path[PATH_MAX];
 
 typedef struct FpPatternStats_ {
@@ -191,8 +197,8 @@ void EngineAnalysisFP(const DetectEngineCtx *de_ctx, const Signature *s, char *l
     if (list_type == DETECT_SM_LIST_PMATCH)
         fprintf(fp_engine_analysis_FD, "content\n");
     else {
-        const char *desc = DetectBufferTypeGetDescriptionById(de_ctx, list_type);
-        const char *name = DetectBufferTypeGetNameById(de_ctx, list_type);
+        const char *desc = DetectEngineBufferTypeGetDescriptionById(de_ctx, list_type);
+        const char *name = DetectEngineBufferTypeGetNameById(de_ctx, list_type);
         if (desc && name) {
             fprintf(fp_engine_analysis_FD, "%s (%s)\n", desc, name);
         }
@@ -240,7 +246,7 @@ void EngineAnalysisFP(const DetectEngineCtx *de_ctx, const Signature *s, char *l
     uint16_t patlen = fp_cd->content_len;
     uint8_t *pat = SCMalloc(fp_cd->content_len + 1);
     if (unlikely(pat == NULL)) {
-        FatalError(SC_ERR_FATAL, "Error allocating memory");
+        FatalError("Error allocating memory");
     }
     memcpy(pat, fp_cd->content, fp_cd->content_len);
     pat[fp_cd->content_len] = '\0';
@@ -300,8 +306,7 @@ int SetupFPAnalyzer(void)
 
     fp_engine_analysis_FD = fopen(log_path, "w");
     if (fp_engine_analysis_FD == NULL) {
-        SCLogError(SC_ERR_FOPEN, "failed to open %s: %s", log_path,
-                   strerror(errno));
+        SCLogError("failed to open %s: %s", log_path, strerror(errno));
         return 0;
     }
 
@@ -349,7 +354,7 @@ int SetupRuleAnalyzer(void)
             snprintf(log_path, sizeof(log_path), "%s/%s", log_dir, "rules_analysis.txt");
             rule_engine_analysis_FD = fopen(log_path, "w");
             if (rule_engine_analysis_FD == NULL) {
-                SCLogError(SC_ERR_FOPEN, "failed to open %s: %s", log_path, strerror(errno));
+                SCLogError("failed to open %s: %s", log_path, strerror(errno));
                 return 0;
             }
 
@@ -418,6 +423,10 @@ void CleanupRuleAnalyzer(void)
         fclose(rule_engine_analysis_FD);
         rule_engine_analysis_FD = NULL;
     }
+    if (percent_re != NULL) {
+        pcre2_code_free(percent_re);
+    }
+    pcre2_match_data_free(percent_re_match);
 }
 
 /**
@@ -428,22 +437,21 @@ void CleanupRuleAnalyzer(void)
 int PerCentEncodingSetup(void)
 {
 #define DETECT_PERCENT_ENCODING_REGEX "%[0-9|a-f|A-F]{2}"
-    const char *eb = NULL;
-    int eo = 0;
-    int opts = 0;    //PCRE_NEWLINE_ANY??
+    int en;
+    PCRE2_SIZE eo = 0;
+    int opts = 0; // PCRE2_NEWLINE_ANY??
 
-    percent_re = pcre_compile(DETECT_PERCENT_ENCODING_REGEX, opts, &eb, &eo, NULL);
+    percent_re = pcre2_compile((PCRE2_SPTR8)DETECT_PERCENT_ENCODING_REGEX, PCRE2_ZERO_TERMINATED,
+            opts, &en, &eo, NULL);
     if (percent_re == NULL) {
-        SCLogError(SC_ERR_PCRE_COMPILE, "Compile of \"%s\" failed at offset %" PRId32 ": %s",
-                   DETECT_PERCENT_ENCODING_REGEX, eo, eb);
+        PCRE2_UCHAR errbuffer[256];
+        pcre2_get_error_message(en, errbuffer, sizeof(errbuffer));
+        SCLogError("Compile of \"%s\" failed at offset %d: %s", DETECT_PERCENT_ENCODING_REGEX,
+                (int)eo, errbuffer);
         return 0;
     }
 
-    percent_re_study = pcre_study(percent_re, 0, &eb);
-    if (eb != NULL) {
-        SCLogError(SC_ERR_PCRE_STUDY, "pcre study failed: %s", eb);
-        return 0;
-    }
+    percent_re_match = pcre2_match_data_create_from_pattern(percent_re, NULL);
     return 1;
 }
 
@@ -454,18 +462,16 @@ int PerCentEncodingSetup(void)
  * \retval 0 if it doesn't have % encoding
  * \retval -1 on error
  */
-int PerCentEncodingMatch (uint8_t *content, uint8_t content_len)
+static int PerCentEncodingMatch(uint8_t *content, uint16_t content_len)
 {
-#define MAX_ENCODED_CHARS 240
     int ret = 0;
-    int ov[MAX_ENCODED_CHARS];
 
-    ret = pcre_exec(percent_re, percent_re_study, (char *)content, content_len, 0, 0, ov, MAX_ENCODED_CHARS);
+    ret = pcre2_match(percent_re, (PCRE2_SPTR8)content, content_len, 0, 0, percent_re_match, NULL);
     if (ret == -1) {
         return 0;
     }
     else if (ret < -1) {
-        SCLogError(SC_ERR_PCRE_MATCH, "Error parsing content - %s; error code is %d", content, ret);
+        SCLogError("Error parsing content - %s; error code is %d", content, ret);
         return -1;
     }
     return ret;
@@ -488,7 +494,7 @@ static void EngineAnalysisRulesPrintFP(const DetectEngineCtx *de_ctx, const Sign
     uint16_t patlen = fp_cd->content_len;
     uint8_t *pat = SCMalloc(fp_cd->content_len + 1);
     if (unlikely(pat == NULL)) {
-        FatalError(SC_ERR_FATAL, "Error allocating memory");
+        FatalError("Error allocating memory");
     }
     memcpy(pat, fp_cd->content, fp_cd->content_len);
     pat[fp_cd->content_len] = '\0';
@@ -524,8 +530,8 @@ static void EngineAnalysisRulesPrintFP(const DetectEngineCtx *de_ctx, const Sign
                 payload ? (stream ? "payload and reassembled stream" : "payload") : "reassembled stream");
     }
     else {
-        const char *desc = DetectBufferTypeGetDescriptionById(de_ctx, list_type);
-        const char *name = DetectBufferTypeGetNameById(de_ctx, list_type);
+        const char *desc = DetectEngineBufferTypeGetDescriptionById(de_ctx, list_type);
+        const char *name = DetectEngineBufferTypeGetNameById(de_ctx, list_type);
         if (desc && name) {
             fprintf(rule_engine_analysis_FD, "%s (%s)", desc, name);
         } else if (desc || name) {
@@ -535,9 +541,9 @@ static void EngineAnalysisRulesPrintFP(const DetectEngineCtx *de_ctx, const Sign
     }
 
     fprintf(rule_engine_analysis_FD, "\" ");
-    if (de_ctx->buffer_type_map[list_type] && de_ctx->buffer_type_map[list_type]->transforms.cnt) {
-        fprintf(rule_engine_analysis_FD, "(with %d transform(s)) ",
-                de_ctx->buffer_type_map[list_type]->transforms.cnt);
+    const DetectBufferType *bt = DetectEngineBufferTypeGetById(de_ctx, list_type);
+    if (bt && bt->transforms.cnt) {
+        fprintf(rule_engine_analysis_FD, "(with %d transform(s)) ", bt->transforms.cnt);
     }
     fprintf(rule_engine_analysis_FD, "buffer.\n");
 
@@ -610,6 +616,43 @@ static bool LooksLikeHTTPUA(const uint8_t *buf, uint16_t len)
     return false;
 }
 
+static void DumpContent(JsonBuilder *js, const DetectContentData *cd)
+{
+    char pattern_str[1024] = "";
+    DetectContentPatternPrettyPrint(cd, pattern_str, sizeof(pattern_str));
+
+    jb_set_string(js, "pattern", pattern_str);
+    jb_set_uint(js, "length", cd->content_len);
+    jb_set_bool(js, "nocase", cd->flags & DETECT_CONTENT_NOCASE);
+    jb_set_bool(js, "negated", cd->flags & DETECT_CONTENT_NEGATED);
+    jb_set_bool(js, "starts_with", cd->flags & DETECT_CONTENT_STARTS_WITH);
+    jb_set_bool(js, "ends_with", cd->flags & DETECT_CONTENT_ENDS_WITH);
+    jb_set_bool(js, "is_mpm", cd->flags & DETECT_CONTENT_MPM);
+    jb_set_bool(js, "no_double_inspect", cd->flags & DETECT_CONTENT_NO_DOUBLE_INSPECTION_REQUIRED);
+    if (cd->flags & DETECT_CONTENT_OFFSET) {
+        jb_set_uint(js, "offset", cd->offset);
+    }
+    if (cd->flags & DETECT_CONTENT_DEPTH) {
+        jb_set_uint(js, "depth", cd->depth);
+    }
+    if (cd->flags & DETECT_CONTENT_DISTANCE) {
+        jb_set_uint(js, "distance", cd->distance);
+    }
+    if (cd->flags & DETECT_CONTENT_WITHIN) {
+        jb_set_uint(js, "within", cd->within);
+    }
+    jb_set_bool(js, "fast_pattern", cd->flags & DETECT_CONTENT_FAST_PATTERN);
+    jb_set_bool(js, "relative_next", cd->flags & DETECT_CONTENT_RELATIVE_NEXT);
+}
+
+static void DumpPcre(JsonBuilder *js, const DetectPcreData *cd)
+{
+    jb_set_bool(js, "relative", cd->flags & DETECT_PCRE_RELATIVE);
+    jb_set_bool(js, "relative_next", cd->flags & DETECT_PCRE_RELATIVE_NEXT);
+    jb_set_bool(js, "nocase", cd->flags & DETECT_PCRE_CASELESS);
+    jb_set_bool(js, "negated", cd->flags & DETECT_PCRE_NEGATE);
+}
+
 static void DumpMatches(RuleAnalyzer *ctx, JsonBuilder *js, const SigMatchData *smd)
 {
     if (smd == NULL)
@@ -626,25 +669,7 @@ static void DumpMatches(RuleAnalyzer *ctx, JsonBuilder *js, const SigMatchData *
                 const DetectContentData *cd = (const DetectContentData *)smd->ctx;
 
                 jb_open_object(js, "content");
-                jb_set_string_from_bytes(js, "pattern", cd->content, cd->content_len);
-                jb_set_bool(js, "nocase", cd->flags & DETECT_CONTENT_NOCASE);
-                jb_set_bool(js, "negated", cd->flags & DETECT_CONTENT_NEGATED);
-                jb_set_bool(js, "starts_with", cd->flags & DETECT_CONTENT_STARTS_WITH);
-                jb_set_bool(js, "ends_with", cd->flags & DETECT_CONTENT_ENDS_WITH);
-                jb_set_bool(js, "is_mpm", cd->flags & DETECT_CONTENT_MPM);
-                if (cd->flags & DETECT_CONTENT_OFFSET) {
-                    jb_set_uint(js, "offset", cd->offset);
-                }
-                if (cd->flags & DETECT_CONTENT_DEPTH) {
-                    jb_set_uint(js, "depth", cd->depth);
-                }
-                if (cd->flags & DETECT_CONTENT_DISTANCE) {
-                    jb_set_uint(js, "distance", cd->distance);
-                }
-                if (cd->flags & DETECT_CONTENT_WITHIN) {
-                    jb_set_uint(js, "within", cd->within);
-                }
-                jb_set_bool(js, "fast_pattern", cd->flags & DETECT_CONTENT_FAST_PATTERN);
+                DumpContent(js, cd);
                 if (cd->flags & DETECT_CONTENT_FAST_PATTERN_ONLY) {
                     AnalyzerNote(ctx, (char *)"'fast_pattern:only' option is silently ignored and "
                                               "is interpreted as regular 'fast_pattern'");
@@ -659,6 +684,106 @@ static void DumpMatches(RuleAnalyzer *ctx, JsonBuilder *js, const SigMatchData *
                             (char *)"pattern looks like it inspects HTTP, use http.user_agent "
                                     "or http.header for improved performance");
                 }
+                if (cd->flags & DETECT_CONTENT_WITHIN2DEPTH) {
+                    AnalyzerNote(ctx, (char *)"'within' option for pattern w/o previous content "
+                                              "was converted to 'depth'");
+                }
+                if (cd->flags & DETECT_CONTENT_DISTANCE2OFFSET) {
+                    AnalyzerNote(ctx, (char *)"'distance' option for pattern w/o previous content "
+                                              "was converted to 'offset'");
+                }
+                jb_close(js);
+                break;
+            }
+            case DETECT_PCRE: {
+                const DetectPcreData *cd = (const DetectPcreData *)smd->ctx;
+
+                jb_open_object(js, "pcre");
+                DumpPcre(js, cd);
+                jb_close(js);
+                if (cd->flags & DETECT_PCRE_RAWBYTES) {
+                    AnalyzerNote(ctx,
+                            (char *)"'/B' (rawbytes) option is a no-op and is silently ignored");
+                }
+                break;
+            }
+            case DETECT_BYTEJUMP: {
+                const DetectBytejumpData *cd = (const DetectBytejumpData *)smd->ctx;
+
+                jb_open_object(js, "byte_jump");
+                jb_set_uint(js, "nbytes", cd->nbytes);
+                jb_set_uint(js, "offset", cd->offset);
+                jb_set_uint(js, "multiplier", cd->multiplier);
+                jb_set_uint(js, "post_offset", cd->post_offset);
+                switch (cd->base) {
+                    case DETECT_BYTEJUMP_BASE_UNSET:
+                        jb_set_string(js, "base", "unset");
+                        break;
+                    case DETECT_BYTEJUMP_BASE_OCT:
+                        jb_set_string(js, "base", "oct");
+                        break;
+                    case DETECT_BYTEJUMP_BASE_DEC:
+                        jb_set_string(js, "base", "dec");
+                        break;
+                    case DETECT_BYTEJUMP_BASE_HEX:
+                        jb_set_string(js, "base", "hex");
+                        break;
+                }
+                jb_open_array(js, "flags");
+                if (cd->flags & DETECT_BYTEJUMP_BEGIN)
+                    jb_append_string(js, "from_beginning");
+                if (cd->flags & DETECT_BYTEJUMP_LITTLE)
+                    jb_append_string(js, "little_endian");
+                if (cd->flags & DETECT_BYTEJUMP_BIG)
+                    jb_append_string(js, "big_endian");
+                if (cd->flags & DETECT_BYTEJUMP_STRING)
+                    jb_append_string(js, "string");
+                if (cd->flags & DETECT_BYTEJUMP_RELATIVE)
+                    jb_append_string(js, "relative");
+                if (cd->flags & DETECT_BYTEJUMP_ALIGN)
+                    jb_append_string(js, "align");
+                if (cd->flags & DETECT_BYTEJUMP_DCE)
+                    jb_append_string(js, "dce");
+                if (cd->flags & DETECT_BYTEJUMP_OFFSET_BE)
+                    jb_append_string(js, "offset_be");
+                if (cd->flags & DETECT_BYTEJUMP_END)
+                    jb_append_string(js, "from_end");
+                jb_close(js);
+                jb_close(js);
+                break;
+            }
+            case DETECT_BYTETEST: {
+                const DetectBytetestData *cd = (const DetectBytetestData *)smd->ctx;
+
+                jb_open_object(js, "byte_test");
+                jb_set_uint(js, "nbytes", cd->nbytes);
+                jb_set_uint(js, "offset", cd->offset);
+                switch (cd->base) {
+                    case DETECT_BYTETEST_BASE_UNSET:
+                        jb_set_string(js, "base", "unset");
+                        break;
+                    case DETECT_BYTETEST_BASE_OCT:
+                        jb_set_string(js, "base", "oct");
+                        break;
+                    case DETECT_BYTETEST_BASE_DEC:
+                        jb_set_string(js, "base", "dec");
+                        break;
+                    case DETECT_BYTETEST_BASE_HEX:
+                        jb_set_string(js, "base", "hex");
+                        break;
+                }
+                jb_open_array(js, "flags");
+                if (cd->flags & DETECT_BYTETEST_LITTLE)
+                    jb_append_string(js, "little_endian");
+                if (cd->flags & DETECT_BYTETEST_BIG)
+                    jb_append_string(js, "big_endian");
+                if (cd->flags & DETECT_BYTETEST_STRING)
+                    jb_append_string(js, "string");
+                if (cd->flags & DETECT_BYTETEST_RELATIVE)
+                    jb_append_string(js, "relative");
+                if (cd->flags & DETECT_BYTETEST_DCE)
+                    jb_append_string(js, "dce");
+                jb_close(js);
                 jb_close(js);
                 break;
             }
@@ -716,6 +841,42 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     }
     jb_close(ctx.js);
 
+    switch (s->type) {
+        case SIG_TYPE_NOT_SET:
+            jb_set_string(ctx.js, "type", "unset");
+            break;
+        case SIG_TYPE_IPONLY:
+            jb_set_string(ctx.js, "type", "ip_only");
+            break;
+        case SIG_TYPE_LIKE_IPONLY:
+            jb_set_string(ctx.js, "type", "like_ip_only");
+            break;
+        case SIG_TYPE_PDONLY:
+            jb_set_string(ctx.js, "type", "pd_only");
+            break;
+        case SIG_TYPE_DEONLY:
+            jb_set_string(ctx.js, "type", "de_only");
+            break;
+        case SIG_TYPE_PKT:
+            jb_set_string(ctx.js, "type", "pkt");
+            break;
+        case SIG_TYPE_PKT_STREAM:
+            jb_set_string(ctx.js, "type", "pkt_stream");
+            break;
+        case SIG_TYPE_STREAM:
+            jb_set_string(ctx.js, "type", "stream");
+            break;
+        case SIG_TYPE_APPLAYER:
+            jb_set_string(ctx.js, "type", "app_layer");
+            break;
+        case SIG_TYPE_APP_TX:
+            jb_set_string(ctx.js, "type", "app_tx");
+            break;
+        case SIG_TYPE_MAX:
+            jb_set_string(ctx.js, "type", "error");
+            break;
+    }
+
     jb_open_array(ctx.js, "flags");
     if (s->flags & SIG_FLAG_SRC_ANY) {
         jb_append_string(ctx.js, "src_any");
@@ -738,9 +899,6 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     if (s->flags & SIG_FLAG_APPLAYER) {
         jb_append_string(ctx.js, "applayer");
     }
-    if (s->flags & SIG_FLAG_IPONLY) {
-        jb_append_string(ctx.js, "ip_only");
-    }
     if (s->flags & SIG_FLAG_REQUIRE_PACKET) {
         jb_append_string(ctx.js, "need_packet");
     }
@@ -774,9 +932,6 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     if (s->flags & SIG_FLAG_PREFILTER) {
         jb_append_string(ctx.js, "prefilter");
     }
-    if (s->flags & SIG_FLAG_PDONLY) {
-        jb_append_string(ctx.js, "proto_detect_only");
-    }
     if (s->flags & SIG_FLAG_SRC_IS_TARGET) {
         jb_append_string(ctx.js, "src_is_target");
     }
@@ -785,10 +940,13 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     }
     jb_close(ctx.js);
 
+    const DetectEnginePktInspectionEngine *pkt_mpm = NULL;
+    const DetectEngineAppInspectionEngine *app_mpm = NULL;
+
     jb_open_array(ctx.js, "pkt_engines");
     const DetectEnginePktInspectionEngine *pkt = s->pkt_inspect;
     for ( ; pkt != NULL; pkt = pkt->next) {
-        const char *name = DetectBufferTypeGetNameById(de_ctx, pkt->sm_list);
+        const char *name = DetectEngineBufferTypeGetNameById(de_ctx, pkt->sm_list);
         if (name == NULL) {
             switch (pkt->sm_list) {
                 case DETECT_SM_LIST_PMATCH:
@@ -805,8 +963,42 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
         jb_start_object(ctx.js);
         jb_set_string(ctx.js, "name", name);
         jb_set_bool(ctx.js, "is_mpm", pkt->mpm);
+        if (pkt->v1.transforms != NULL) {
+            jb_open_array(ctx.js, "transforms");
+            for (int t = 0; t < pkt->v1.transforms->cnt; t++) {
+                jb_start_object(ctx.js);
+                jb_set_string(ctx.js, "name",
+                        sigmatch_table[pkt->v1.transforms->transforms[t].transform].name);
+                jb_close(ctx.js);
+            }
+            jb_close(ctx.js);
+        }
         DumpMatches(&ctx, ctx.js, pkt->smd);
         jb_close(ctx.js);
+        if (pkt->mpm) {
+            pkt_mpm = pkt;
+        }
+    }
+    jb_close(ctx.js);
+    jb_open_array(ctx.js, "frame_engines");
+    const DetectEngineFrameInspectionEngine *frame = s->frame_inspect;
+    for (; frame != NULL; frame = frame->next) {
+        const char *name = DetectEngineBufferTypeGetNameById(de_ctx, frame->sm_list);
+        jb_start_object(ctx.js);
+        jb_set_string(ctx.js, "name", name);
+        jb_set_bool(ctx.js, "is_mpm", frame->mpm);
+        if (frame->v1.transforms != NULL) {
+            jb_open_array(ctx.js, "transforms");
+            for (int t = 0; t < frame->v1.transforms->cnt; t++) {
+                jb_start_object(ctx.js);
+                jb_set_string(ctx.js, "name",
+                        sigmatch_table[frame->v1.transforms->transforms[t].transform].name);
+                jb_close(ctx.js);
+            }
+            jb_close(ctx.js);
+        }
+        DumpMatches(&ctx, ctx.js, frame->smd);
+        jb_close(ctx.js);
     }
     jb_close(ctx.js);
 
@@ -818,7 +1010,7 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
         jb_open_array(ctx.js, "engines");
         const DetectEngineAppInspectionEngine *app = s->app_inspect;
         for ( ; app != NULL; app = app->next) {
-            const char *name = DetectBufferTypeGetNameById(de_ctx, app->sm_list);
+            const char *name = DetectEngineBufferTypeGetNameById(de_ctx, app->sm_list);
             if (name == NULL) {
                 switch (app->sm_list) {
                     case DETECT_SM_LIST_PMATCH:
@@ -845,8 +1037,22 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
             jb_set_bool(ctx.js, "is_mpm", app->mpm);
             jb_set_string(ctx.js, "app_proto", AppProtoToString(app->alproto));
             jb_set_uint(ctx.js, "progress", app->progress);
+
+            if (app->v2.transforms != NULL) {
+                jb_open_array(ctx.js, "transforms");
+                for (int t = 0; t < app->v2.transforms->cnt; t++) {
+                    jb_start_object(ctx.js);
+                    jb_set_string(ctx.js, "name",
+                            sigmatch_table[app->v2.transforms->transforms[t].transform].name);
+                    jb_close(ctx.js);
+                }
+                jb_close(ctx.js);
+            }
             DumpMatches(&ctx, ctx.js, app->smd);
             jb_close(ctx.js);
+            if (app->mpm) {
+                app_mpm = app;
+            }
         }
         jb_close(ctx.js);
 
@@ -859,13 +1065,58 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     jb_open_object(ctx.js, "lists");
     for (int i = 0; i < DETECT_SM_LIST_MAX; i++) {
         if (s->sm_arrays[i] != NULL) {
-            jb_open_object(ctx.js, DetectSigmatchListEnumToString(i));
+            jb_open_object(ctx.js, DetectListToHumanString(i));
             DumpMatches(&ctx, ctx.js, s->sm_arrays[i]);
             jb_close(ctx.js);
         }
     }
     jb_close(ctx.js);
 
+    if (pkt_mpm || app_mpm) {
+        jb_open_object(ctx.js, "mpm");
+
+        int mpm_list = pkt_mpm ? DETECT_SM_LIST_PMATCH : app_mpm->sm_list;
+        const char *name;
+        if (mpm_list < DETECT_SM_LIST_DYNAMIC_START)
+            name = DetectListToHumanString(mpm_list);
+        else
+            name = DetectEngineBufferTypeGetNameById(de_ctx, mpm_list);
+        jb_set_string(ctx.js, "buffer", name);
+
+        SigMatchData *smd = pkt_mpm ? pkt_mpm->smd : app_mpm->smd;
+        if (smd == NULL && mpm_list == DETECT_SM_LIST_PMATCH) {
+            smd = s->sm_arrays[mpm_list];
+        }
+        do {
+            switch (smd->type) {
+                case DETECT_CONTENT: {
+                    const DetectContentData *cd = (const DetectContentData *)smd->ctx;
+                    if (cd->flags & DETECT_CONTENT_MPM) {
+                        DumpContent(ctx.js, cd);
+                    }
+                    break;
+                }
+            }
+
+            if (smd->is_last)
+                break;
+            smd++;
+        } while (1);
+        jb_close(ctx.js);
+    } else if (s->init_data->prefilter_sm) {
+        jb_open_object(ctx.js, "prefilter");
+        int prefilter_list = SigMatchListSMBelongsTo(s, s->init_data->prefilter_sm);
+        const char *name;
+        if (prefilter_list < DETECT_SM_LIST_DYNAMIC_START)
+            name = DetectListToHumanString(prefilter_list);
+        else
+            name = DetectEngineBufferTypeGetNameById(de_ctx, prefilter_list);
+        jb_set_string(ctx.js, "buffer", name);
+        const char *mname = sigmatch_table[s->init_data->prefilter_sm->type].name;
+        jb_set_string(ctx.js, "name", mname);
+        jb_close(ctx.js);
+    }
+
     if (ctx.js_warnings) {
         jb_close(ctx.js_warnings);
         jb_set_object(ctx.js, "warnings", ctx.js_warnings);
@@ -897,6 +1148,85 @@ void EngineAnalysisRules2(const DetectEngineCtx *de_ctx, const Signature *s)
     SCReturn;
 }
 
+void DumpPatterns(DetectEngineCtx *de_ctx)
+{
+    if (de_ctx->pattern_hash_table == NULL)
+        return;
+
+    JsonBuilder *root_jb = jb_new_object();
+    JsonBuilder *arrays[de_ctx->buffer_type_id];
+    memset(&arrays, 0, sizeof(JsonBuilder *) * de_ctx->buffer_type_id);
+
+    jb_open_array(root_jb, "buffers");
+
+    for (HashListTableBucket *htb = HashListTableGetListHead(de_ctx->pattern_hash_table);
+            htb != NULL; htb = HashListTableGetListNext(htb)) {
+        char str[1024] = "";
+        DetectPatternTracker *p = HashListTableGetListData(htb);
+        DetectContentPatternPrettyPrint(p->cd, str, sizeof(str));
+
+        JsonBuilder *jb = arrays[p->sm_list];
+        if (arrays[p->sm_list] == NULL) {
+            jb = arrays[p->sm_list] = jb_new_object();
+            const char *name;
+            if (p->sm_list < DETECT_SM_LIST_DYNAMIC_START)
+                name = DetectListToHumanString(p->sm_list);
+            else
+                name = DetectEngineBufferTypeGetNameById(de_ctx, p->sm_list);
+            jb_set_string(jb, "name", name);
+            jb_set_uint(jb, "list_id", p->sm_list);
+
+            jb_open_array(jb, "patterns");
+        }
+
+        jb_start_object(jb);
+        jb_set_string(jb, "pattern", str);
+        jb_set_uint(jb, "patlen", p->cd->content_len);
+        jb_set_uint(jb, "cnt", p->cnt);
+        jb_set_uint(jb, "mpm", p->mpm);
+        jb_open_object(jb, "flags");
+        jb_set_bool(jb, "nocase", p->cd->flags & DETECT_CONTENT_NOCASE);
+        jb_set_bool(jb, "negated", p->cd->flags & DETECT_CONTENT_NEGATED);
+        jb_set_bool(jb, "depth", p->cd->flags & DETECT_CONTENT_DEPTH);
+        jb_set_bool(jb, "offset", p->cd->flags & DETECT_CONTENT_OFFSET);
+        jb_set_bool(jb, "endswith", p->cd->flags & DETECT_CONTENT_ENDS_WITH);
+        jb_close(jb);
+        jb_close(jb);
+    }
+
+    for (uint32_t i = 0; i < de_ctx->buffer_type_id; i++) {
+        JsonBuilder *jb = arrays[i];
+        if (jb == NULL)
+            continue;
+
+        jb_close(jb); // array
+        jb_close(jb); // object
+
+        jb_append_object(root_jb, jb);
+        jb_free(jb);
+    }
+    jb_close(root_jb);
+    jb_close(root_jb);
+
+    const char *filename = "patterns.json";
+    const char *log_dir = ConfigGetLogDirectory();
+    char json_path[PATH_MAX] = "";
+    snprintf(json_path, sizeof(json_path), "%s/%s", log_dir, filename);
+
+    SCMutexLock(&g_rules_analyzer_write_m);
+    FILE *fp = fopen(json_path, "a");
+    if (fp != NULL) {
+        fwrite(jb_ptr(root_jb), jb_len(root_jb), 1, fp);
+        fprintf(fp, "\n");
+        fclose(fp);
+    }
+    SCMutexUnlock(&g_rules_analyzer_write_m);
+    jb_free(root_jb);
+
+    HashListTableFree(de_ctx->pattern_hash_table);
+    de_ctx->pattern_hash_table = NULL;
+}
+
 static void EngineAnalysisItemsReset(void)
 {
     for (size_t i = 0; i < ARRAY_SIZE(analyzer_items); i++) {
@@ -918,12 +1248,14 @@ static void EngineAnalysisItemsInit(void)
     for (size_t i = 0; i < ARRAY_SIZE(analyzer_items); i++) {
         DetectEngineAnalyzerItems *analyzer_item = &analyzer_items[i];
 
-        analyzer_item->item_id = DetectBufferTypeGetByName(analyzer_item->item_name);
+        int item_id = DetectBufferTypeGetByName(analyzer_item->item_name);
+        DEBUG_VALIDATE_BUG_ON(item_id < 0 || item_id > UINT16_MAX);
+        analyzer_item->item_id = (uint16_t)item_id;
         if (analyzer_item->item_id == -1) {
             /* Mismatch between the analyzer_items array and what's supported */
-            FatalError(SC_ERR_INITIALIZATION,
-                       "unable to initialize engine-analysis table: detect buffer \"%s\" not recognized.",
-                       analyzer_item->item_name);
+            FatalError("unable to initialize engine-analysis table: detect buffer \"%s\" not "
+                       "recognized.",
+                    analyzer_item->item_name);
         }
         analyzer_item->item_seen = false;
 
@@ -1019,7 +1351,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
         rule_ipv6_only += 1;
     }
 
-    for (list_id = 0; list_id < (int)s->init_data->smlists_array_size; list_id++) {
+    for (list_id = 0; list_id < DETECT_SM_LIST_MAX; list_id++) {
         SigMatch *sm = NULL;
         for (sm = s->init_data->smlists[list_id]; sm != NULL; sm = sm->next) {
             int16_t item_slot = analyzer_item_map[list_id];
@@ -1083,11 +1415,10 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
                     rule_flags = 1;
                 }
             }
-        } /* for (sm = s->sm_lists[list_id]; sm != NULL; sm = sm->next) */
+        } /* for (sm = s->init_data->smlists[list_id]; sm != NULL; sm = sm->next) */
 
     } /* for ( ; list_id < DETECT_SM_LIST_MAX; list_id++) */
 
-
     if (file_store && !RequiresFeature("output::file-store")) {
         rule_warning += 1;
         warn_file_store_not_present = 1;
@@ -1101,8 +1432,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
     if (rule_content_http > 0 && rule_pcre > 0 && rule_pcre_http == 0) {
         rule_warning += 1;
         warn_pcre_http_content = 1;
-    }
-    else if (s->alproto == ALPROTO_HTTP && rule_pcre > 0 && rule_pcre_http == 0) {
+    } else if (s->alproto == ALPROTO_HTTP1 && rule_pcre > 0 && rule_pcre_http == 0) {
         rule_warning += 1;
         warn_pcre_http = 1;
     }
@@ -1111,7 +1441,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
         rule_warning += 1;
         warn_content_http_content = 1;
     }
-    if (s->alproto == ALPROTO_HTTP && rule_content > 0 && rule_content_http == 0) {
+    if (s->alproto == ALPROTO_HTTP1 && rule_content > 0 && rule_content_http == 0) {
         rule_warning += 1;
         warn_content_http = 1;
     }
@@ -1160,7 +1490,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
         rule_warning += 1;
         warn_offset_depth_alproto = 1;
     }
-    if (s->init_data->mpm_sm != NULL && s->alproto == ALPROTO_HTTP &&
+    if (s->init_data->mpm_sm != NULL && s->alproto == ALPROTO_HTTP1 &&
             s->init_data->mpm_sm_list == DETECT_SM_LIST_PMATCH) {
         rule_warning += 1;
         warn_non_alproto_fp_for_alproto_sig = 1;
@@ -1183,8 +1513,39 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
         fprintf(rule_engine_analysis_FD, "== Sid: %u ==\n", s->id);
         fprintf(rule_engine_analysis_FD, "%s\n", line);
 
-        if (s->flags & SIG_FLAG_IPONLY) fprintf(rule_engine_analysis_FD, "    Rule is ip only.\n");
-        if (s->flags & SIG_FLAG_PDONLY) fprintf(rule_engine_analysis_FD, "    Rule is PD only.\n");
+        switch (s->type) {
+            case SIG_TYPE_NOT_SET:
+                break;
+            case SIG_TYPE_IPONLY:
+                fprintf(rule_engine_analysis_FD, "    Rule is ip only.\n");
+                break;
+            case SIG_TYPE_LIKE_IPONLY:
+                fprintf(rule_engine_analysis_FD, "    Rule is like ip only.\n");
+                break;
+            case SIG_TYPE_PDONLY:
+                fprintf(rule_engine_analysis_FD, "    Rule is PD only.\n");
+                break;
+            case SIG_TYPE_DEONLY:
+                fprintf(rule_engine_analysis_FD, "    Rule is DE only.\n");
+                break;
+            case SIG_TYPE_PKT:
+                fprintf(rule_engine_analysis_FD, "    Rule is packet inspecting.\n");
+                break;
+            case SIG_TYPE_PKT_STREAM:
+                fprintf(rule_engine_analysis_FD, "    Rule is packet and stream inspecting.\n");
+                break;
+            case SIG_TYPE_STREAM:
+                fprintf(rule_engine_analysis_FD, "    Rule is stream inspecting.\n");
+                break;
+            case SIG_TYPE_APPLAYER:
+                fprintf(rule_engine_analysis_FD, "    Rule is app-layer inspecting.\n");
+                break;
+            case SIG_TYPE_APP_TX:
+                fprintf(rule_engine_analysis_FD, "    Rule is App-layer TX inspecting.\n");
+                break;
+            case SIG_TYPE_MAX:
+                break;
+        }
         if (rule_ipv6_only) fprintf(rule_engine_analysis_FD, "    Rule is IPv6 only.\n");
         if (rule_ipv4_only) fprintf(rule_engine_analysis_FD, "    Rule is IPv4 only.\n");
         if (packet_buf) fprintf(rule_engine_analysis_FD, "    Rule matches on packets.\n");
@@ -1225,7 +1586,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
             fprintf(rule_engine_analysis_FD, "    Warning: Rule uses content options with http_* and pcre options without http modifiers.\n"
                                              "             -Consider adding http pcre modifier.\n");
         }
-        else if (warn_pcre_http /*s->alproto == ALPROTO_HTTP && rule_pcre > 0 && rule_pcre_http == 0*/) {
+        else if (warn_pcre_http /*s->alproto == ALPROTO_HTTP1 && rule_pcre > 0 && rule_pcre_http == 0*/) {
             fprintf(rule_engine_analysis_FD, "    Warning: Rule app layer protocol is http, but pcre options do not have http modifiers.\n"
                                              "             -Consider adding http pcre modifiers.\n");
         }
@@ -1233,7 +1594,7 @@ void EngineAnalysisRules(const DetectEngineCtx *de_ctx,
             fprintf(rule_engine_analysis_FD, "    Warning: Rule contains content with http_* and content without http_*.\n"
                                          "             -Consider adding http content modifiers.\n");
         }
-        if (warn_content_http /*s->alproto == ALPROTO_HTTP && rule_content > 0 && rule_content_http == 0*/) {
+        if (warn_content_http /*s->alproto == ALPROTO_HTTP1 && rule_content > 0 && rule_content_http == 0*/) {
             fprintf(rule_engine_analysis_FD, "    Warning: Rule app layer protocol is http, but content options do not have http_* modifiers.\n"
                                              "             -Consider adding http content modifiers.\n");
         }
