@@ -1,4 +1,4 @@
-/* Copyright (C) 2015 Open Information Security Foundation
+/* Copyright (C) 2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
 #include "suricata-common.h"
 #include "suricata.h"
 #include "conf.h"
-#include "debug.h"
 #include "detect.h"
 #include "detect-parse.h"
 
@@ -36,12 +35,14 @@
 #include "util-signal.h"
 
 #include "detect-engine-loader.h"
+#include "detect-engine-build.h"
 #include "detect-engine-analyzer.h"
 #include "detect-engine-mpm.h"
 #include "detect-engine-sigorder.h"
 
 #include "util-detect.h"
 #include "util-threshold-config.h"
+#include "util-path.h"
 
 #ifdef HAVE_GLOB_H
 #include <glob.h>
@@ -64,20 +65,26 @@ char *DetectLoadCompleteSigPath(const DetectEngineCtx *de_ctx, const char *sig_f
     char varname[128];
 
     if (sig_file == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENTS,"invalid sig_file argument - NULL");
+        SCLogError("invalid sig_file argument - NULL");
         return NULL;
     }
 
-    if (strlen(de_ctx->config_prefix) > 0) {
+    /* If we have a configuration prefix, only use it if the primary configuration node
+     * is not marked as final, as that means it was provided on the command line with
+     * a --set. */
+    ConfNode *default_rule_path = ConfGetNode("default-rule-path");
+    if ((!default_rule_path || !default_rule_path->final) && strlen(de_ctx->config_prefix) > 0) {
         snprintf(varname, sizeof(varname), "%s.default-rule-path",
                 de_ctx->config_prefix);
-    } else {
-        snprintf(varname, sizeof(varname), "default-rule-path");
+        default_rule_path = ConfGetNode(varname);
+    }
+    if (default_rule_path) {
+        defaultpath = default_rule_path->val;
     }
 
     /* Path not specified */
     if (PathIsRelative(sig_file)) {
-        if (ConfGet(varname, &defaultpath) == 1) {
+        if (defaultpath) {
             SCLogDebug("Default path: %s", defaultpath);
             size_t path_len = sizeof(char) * (strlen(defaultpath) +
                           strlen(sig_file) + 2);
@@ -93,7 +100,7 @@ char *DetectLoadCompleteSigPath(const DetectEngineCtx *de_ctx, const char *sig_f
                 strlcat(path, "/", path_len);
 #endif
             strlcat(path, sig_file, path_len);
-       } else {
+        } else {
             path = SCStrdup(sig_file);
             if (unlikely(path == NULL))
                 return NULL;
@@ -128,8 +135,9 @@ static int DetectLoadSigFile(DetectEngineCtx *de_ctx, char *sig_file,
 
     FILE *fp = fopen(sig_file, "r");
     if (fp == NULL) {
-        SCLogError(SC_ERR_OPENING_RULE_FILE, "opening rule file %s:"
-                   " %s.", sig_file, strerror(errno));
+        SCLogError("opening rule file %s:"
+                   " %s.",
+                sig_file, strerror(errno));
         return -1;
     }
 
@@ -181,12 +189,14 @@ static int DetectLoadSigFile(DetectEngineCtx *de_ctx, char *sig_file,
             good++;
         } else {
             if (!de_ctx->sigerror_silent) {
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "error parsing signature \"%s\" from "
-                        "file %s at line %"PRId32"", line, sig_file, lineno - multiline);
+                SCLogError("error parsing signature \"%s\" from "
+                           "file %s at line %" PRId32 "",
+                        line, sig_file, lineno - multiline);
 
                 if (!SigStringAppend(&de_ctx->sig_stat, sig_file, line, de_ctx->sigerror, (lineno - multiline))) {
-                    SCLogError(SC_ERR_MEM_ALLOC, "Error adding sig \"%s\" from "
-                            "file %s at line %"PRId32"", line, sig_file, lineno - multiline);
+                    SCLogError("Error adding sig \"%s\" from "
+                               "file %s at line %" PRId32 "",
+                            line, sig_file, lineno - multiline);
                 }
                 if (de_ctx->sigerror) {
                     de_ctx->sigerror = NULL;
@@ -220,7 +230,7 @@ static int ProcessSigFiles(DetectEngineCtx *de_ctx, char *pattern,
     int r = 0;
 
     if (pattern == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "opening rule file null");
+        SCLogError("opening rule file null");
         return -1;
     }
 
@@ -229,13 +239,12 @@ static int ProcessSigFiles(DetectEngineCtx *de_ctx, char *pattern,
     r = glob(pattern, 0, NULL, &files);
 
     if (r == GLOB_NOMATCH) {
-        SCLogWarning(SC_ERR_NO_RULES, "No rule files match the pattern %s", pattern);
+        SCLogWarning("No rule files match the pattern %s", pattern);
         ++(st->bad_files);
         ++(st->total_files);
         return -1;
     } else if (r != 0) {
-        SCLogError(SC_ERR_OPENING_RULE_FILE, "error expanding template %s: %s",
-                 pattern, strerror(errno));
+        SCLogError("error expanding template %s: %s", pattern, strerror(errno));
         return -1;
     }
 
@@ -301,9 +310,8 @@ int SigLoadSignatures(DetectEngineCtx *de_ctx, char *sig_file, int sig_file_excl
         rule_files = ConfGetNode(varname);
         if (rule_files != NULL) {
             if (!ConfNodeIsSequence(rule_files)) {
-                SCLogWarning(SC_ERR_INVALID_ARGUMENT,
-                    "Invalid rule-files configuration section: "
-                    "expected a list of filenames.");
+                SCLogWarning("Invalid rule-files configuration section: "
+                             "expected a list of filenames.");
             }
             else {
                 TAILQ_FOREACH(file, &rule_files->head, next) {
@@ -326,12 +334,12 @@ int SigLoadSignatures(DetectEngineCtx *de_ctx, char *sig_file, int sig_file_excl
         }
     }
 
-    /* If a Signature file is specified from commandline, parse it too */
+    /* If a Signature file is specified from command-line, parse it too */
     if (sig_file != NULL) {
         ret = ProcessSigFiles(de_ctx, sig_file, sig_stat, &good_sigs, &bad_sigs);
 
         if (ret != 0) {
-            if (de_ctx->failure_fatal == 1) {
+            if (de_ctx->failure_fatal) {
                 exit(EXIT_FAILURE);
             }
         }
@@ -344,7 +352,8 @@ int SigLoadSignatures(DetectEngineCtx *de_ctx, char *sig_file, int sig_file_excl
     /* now we should have signatures to work with */
     if (sig_stat->good_sigs_total <= 0) {
         if (sig_stat->total_files > 0) {
-           SCLogWarning(SC_ERR_NO_RULES_LOADED, "%d rule files specified, but no rules were loaded!", sig_stat->total_files);
+            SCLogWarning(
+                    "%d rule files specified, but no rules were loaded!", sig_stat->total_files);
         } else {
             SCLogInfo("No signatures supplied.");
             goto end;
@@ -364,7 +373,10 @@ int SigLoadSignatures(DetectEngineCtx *de_ctx, char *sig_file, int sig_file_excl
     SCSigOrderSignatures(de_ctx);
     SCSigSignatureOrderingModuleCleanup(de_ctx);
 
-    SCThresholdConfInitContext(de_ctx);
+    if (SCThresholdConfInitContext(de_ctx) < 0) {
+        ret = -1;
+        goto end;
+    }
 
     /* Setup the signature group lookup structure and pattern matchers */
     if (SigGroupBuild(de_ctx) < 0)
@@ -452,7 +464,7 @@ int DetectLoadersSync(void)
 
     }
     if (errors) {
-        SCLogError(SC_ERR_INITIALIZATION, "%d loaders reported errors", errors);
+        SCLogError("%d loaders reported errors", errors);
         return -1;
     }
     SCLogDebug("done");
@@ -472,8 +484,7 @@ void DetectLoadersInit(void)
     (void)ConfGetInt("multi-detect.loaders", &setting);
 
     if (setting < 1 || setting > 1024) {
-        SCLogError(SC_ERR_INVALID_ARGUMENTS,
-                "invalid multi-detect.loaders setting %"PRIdMAX, setting);
+        SCLogError("invalid multi-detect.loaders setting %" PRIdMAX, setting);
         exit(EXIT_FAILURE);
     }
     num_loaders = (int32_t)setting;
@@ -571,6 +582,7 @@ static TmEcode DetectLoader(ThreadVars *th_v, void *thread_data)
     DetectLoaderThreadData *ftd = (DetectLoaderThreadData *)thread_data;
     BUG_ON(ftd == NULL);
 
+    TmThreadsSetFlag(th_v, THV_INIT_DONE | THV_RUNNING);
     SCLogDebug("loader thread started");
     while (1)
     {
@@ -608,6 +620,10 @@ static TmEcode DetectLoader(ThreadVars *th_v, void *thread_data)
         SCLogDebug("woke up...");
     }
 
+    TmThreadsSetFlag(th_v, THV_RUNNING_DONE);
+    TmThreadWaitForFlag(th_v, THV_DEINIT);
+    TmThreadsSetFlag(th_v, THV_CLOSED);
+
     return TM_ECODE_OK;
 }
 
