@@ -37,6 +37,8 @@ enum DetectKeywordId {
     DETECT_SEQ,
     DETECT_WINDOW,
     DETECT_IPOPTS,
+    DETECT_IPADDR_SRC,
+    DETECT_IPADDR_DST,
     DETECT_FLAGS,
     DETECT_FRAGBITS,
     DETECT_FRAGOFFSET,
@@ -105,6 +107,10 @@ enum DetectKeywordId {
 
     DETECT_BSIZE,
 
+    DETECT_FRAME,
+
+    DETECT_FLOW_AGE,
+
     DETECT_AL_TLS_VERSION,
     DETECT_AL_TLS_SUBJECT,
     DETECT_AL_TLS_ISSUERDN,
@@ -114,6 +120,7 @@ enum DetectKeywordId {
     DETECT_AL_TLS_VALID,
     DETECT_AL_TLS_FINGERPRINT,
     DETECT_AL_TLS_STORE,
+    DETECT_AL_TLS_CHAIN_LEN,
 
     DETECT_AL_HTTP_COOKIE,
     DETECT_HTTP_COOKIE,
@@ -180,13 +187,16 @@ enum DetectKeywordId {
     DETECT_HTTP2_SIZEUPDATE,
     DETECT_HTTP2_SETTINGS,
     DETECT_HTTP2_HEADERNAME,
-    DETECT_HTTP2_HEADER,
+    DETECT_HTTP_REQUEST_HEADER,
+    DETECT_HTTP_RESPONSE_HEADER,
 
     DETECT_DCE_IFACE,
     DETECT_DCE_OPNUM,
     DETECT_DCE_STUB_DATA,
     DETECT_SMB_NAMED_PIPE,
     DETECT_SMB_SHARE,
+    DETECT_SMB_NTLMSSP_USER,
+    DETECT_SMB_NTLMSSP_DOMAIN,
 
     DETECT_ASN1,
 
@@ -219,6 +229,9 @@ enum DetectKeywordId {
     DETECT_AL_TLS_CERT_SUBJECT,
     DETECT_AL_TLS_CERT_SERIAL,
     DETECT_AL_TLS_CERT_FINGERPRINT,
+    DETECT_AL_TLS_RANDOM_TIME,
+    DETECT_AL_TLS_RANDOM_BYTES,
+    DETECT_AL_TLS_RANDOM,
 
     DETECT_AL_TLS_JA3_HASH,
     DETECT_AL_TLS_JA3_STRING,
@@ -242,6 +255,7 @@ enum DetectKeywordId {
     DETECT_AL_KRB5_MSGTYPE,
     DETECT_AL_KRB5_CNAME,
     DETECT_AL_KRB5_SNAME,
+    DETECT_AL_KRB5_TICKET_ENCRYPTION,
 
     DETECT_AL_SIP_METHOD,
     DETECT_AL_SIP_URI,
@@ -264,7 +278,11 @@ enum DetectKeywordId {
     DETECT_TCPMSS,
     DETECT_FTPDATA,
     DETECT_TARGET,
-    DETECT_AL_TEMPLATE_RUST_BUFFER,
+    DETECT_AL_TEMPLATE_BUFFER,
+    DETECT_AL_DHCP_LEASETIME,
+    DETECT_AL_DHCP_REBINDING_TIME,
+    DETECT_AL_DHCP_RENEWAL_TIME,
+    DETECT_AL_SNMP_USM,
     DETECT_AL_SNMP_VERSION,
     DETECT_AL_SNMP_COMMUNITY,
     DETECT_AL_SNMP_PDU_TYPE,
@@ -284,7 +302,11 @@ enum DetectKeywordId {
     DETECT_AL_MQTT_PUBLISH_MESSAGE,
     DETECT_AL_MQTT_SUBSCRIBE_TOPIC,
     DETECT_AL_MQTT_UNSUBSCRIBE_TOPIC,
-    DETECT_AL_TEMPLATE_BUFFER,
+    DETECT_AL_QUIC_VERSION,
+    DETECT_AL_QUIC_SNI,
+    DETECT_AL_QUIC_UA,
+    DETECT_AL_QUIC_CYU_HASH,
+    DETECT_AL_QUIC_CYU_STRING,
 
     DETECT_BYPASS,
 
@@ -298,12 +320,23 @@ enum DetectKeywordId {
     DETECT_TRANSFORM_DOTPREFIX,
     DETECT_TRANSFORM_PCREXFORM,
     DETECT_TRANSFORM_URL_DECODE,
+    DETECT_TRANSFORM_XOR,
+
+    DETECT_AL_IKE_EXCH_TYPE,
+    DETECT_AL_IKE_SPI_INITIATOR,
+    DETECT_AL_IKE_SPI_RESPONDER,
+    DETECT_AL_IKE_VENDOR,
+    DETECT_AL_IKE_CHOSEN_SA,
+    DETECT_AL_IKE_KEY_EXCHANGE_PAYLOAD_LENGTH,
+    DETECT_AL_IKE_NONCE_PAYLOAD_LENGTH,
+    DETECT_AL_IKE_NONCE,
+    DETECT_AL_IKE_KEY_EXCHANGE,
 
     /* make sure this stays last */
     DETECT_TBLSIZE,
 };
 
-void SigTableList(const char *keyword);
+int SigTableList(const char *keyword);
 void SigTableSetup(void);
 void SigTableRegisterTests(void);
 
