@@ -40,10 +40,15 @@
 #include "detect-engine.h"
 #include "detect-engine-mpm.h"
 #include "detect-engine-state.h"
+#include "detect-engine-build.h"
 
 #include "util-var-name.h"
 #include "util-unittest.h"
 #include "util-debug.h"
+#include "util-conf.h"
+
+#define PARSE_REGEX         "^([a-z]+)(?:,\\s*(.*))?"
+static DetectParseRegex parse_regex;
 
 #define MAX_TOKENS 100
 
@@ -69,6 +74,8 @@ void DetectFlowbitsRegister (void)
 #endif
     /* this is compatible to ip-only signatures */
     sigmatch_table[DETECT_FLOWBITS].flags |= SIGMATCH_IPONLY_COMPAT;
+
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 }
 
 static int FlowbitOrAddData(DetectEngineCtx *de_ctx, DetectFlowbitsData *cd, char *arrptr)
@@ -90,19 +97,24 @@ static int FlowbitOrAddData(DetectEngineCtx *de_ctx, DetectFlowbitsData *cd, cha
 
         // Check for spaces in between the flowbit names
         if (strchr(token, ' ') != NULL) {
-            SCLogError(SC_ERR_INVALID_SIGNATURE, "Spaces are not allowed in flowbit names.");
+            SCLogError("Spaces are not allowed in flowbit names.");
             return -1;
         }
 
         if (i == MAX_TOKENS) {
-            SCLogError(SC_ERR_INVALID_SIGNATURE, "Number of flowbits exceeds "
-                       "maximum allowed: %d.", MAX_TOKENS);
+            SCLogError("Number of flowbits exceeds "
+                       "maximum allowed: %d.",
+                    MAX_TOKENS);
             return -1;
         }
         strarr[i++] = token;
     next:
         arrptr = NULL;
     }
+    if (i == 0) {
+        SCLogError("No valid flowbits specified");
+        return -1;
+    }
 
     cd->or_list_size = i;
     cd->or_list = SCCalloc(cd->or_list_size, sizeof(uint32_t));
@@ -200,146 +212,131 @@ int DetectFlowbitMatch (DetectEngineThreadCtx *det_ctx, Packet *p,
         case DETECT_FLOWBITS_CMD_TOGGLE:
             return DetectFlowbitMatchToggle(p,fd);
         default:
-            SCLogError(SC_ERR_UNKNOWN_VALUE, "unknown cmd %" PRIu32 "", fd->cmd);
+            SCLogError("unknown cmd %" PRIu32 "", fd->cmd);
             return 0;
     }
 
     return 0;
 }
 
-static int DetectFlowbitParse(
-        DetectEngineCtx *de_ctx, const char *rawstr, DetectFlowbitsData **cdout)
+static int DetectFlowbitParse(const char *str, char *cmd, int cmd_len, char *name,
+    int name_len)
 {
-    bool cmd_set = false;
-    bool name_set = false;
-    uint8_t cmd = 0;
-    char name[256] = "";
-    char *val = NULL;
-    DetectFlowbitsData *cd = NULL;
-    char copy[strlen(rawstr) + 1];
-    strlcpy(copy, rawstr, sizeof(copy));
-    char *context = NULL;
-    char *token = strtok_r(copy, ",", &context);
-    while (token != NULL) {
-        while (*token != '\0' && isblank(*token)) {
-            token++;
-        }
-        if (strchr(token, '|') == NULL) {
-            val = strchr(token, ' ');
-            if (val != NULL) {
-                *val++ = '\0';
-                while (*val != '\0' && isblank(*val)) {
-                    val++;
-                }
-            } else {
-                SCLogDebug("val %s", token);
-            }
-        }
-        if (strlen(token) == 0) {
-            return -1;
-        }
-        if (strcmp(token, "noalert") == 0 && !cmd_set) {
-            if (strtok_r(NULL, ",", &context) != NULL) {
-                return -1;
-            }
-            if (val && strlen(val) != 0) {
-                return -1;
-            }
-            *cdout = NULL;
-            return 0;
-        }
-        if (!cmd_set) {
-            if (val && strlen(val) != 0) {
-                return -1;
-            }
-            if (strcmp(token, "set") == 0) {
-                cmd = DETECT_FLOWBITS_CMD_SET;
-            } else if (strcmp(token, "isset") == 0) {
-                cmd = DETECT_FLOWBITS_CMD_ISSET;
-            } else if (strcmp(token, "unset") == 0) {
-                cmd = DETECT_FLOWBITS_CMD_UNSET;
-            } else if (strcmp(token, "isnotset") == 0) {
-                cmd = DETECT_FLOWBITS_CMD_ISNOTSET;
-            } else if (strcmp(token, "toggle") == 0) {
-                cmd = DETECT_FLOWBITS_CMD_TOGGLE;
-            } else {
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "Invalid flowbits cmd: %s", token);
-                return -1;
-            }
-            cmd_set = true;
-        } else if (!name_set) {
-            if (val && strlen(val) != 0) {
-                return -1;
-            }
+    int rc;
+    size_t pcre2len;
+    pcre2_match_data *match = NULL;
 
-            if (strchr(token, '|') == NULL) {
-                /* Validate name, spaces are not allowed. */
-                for (size_t i = 0; i < strlen(token); i++) {
-                    if (isblank(token[i])) {
-                        SCLogError(SC_ERR_INVALID_SIGNATURE, "spaces not allowed in flowbit names");
-                        return 0;
-                    }
+    int count = DetectParsePcreExec(&parse_regex, &match, str, 0, 0);
+    if (count != 2 && count != 3) {
+        SCLogError("\"%s\" is not a valid setting for flowbits.", str);
+        goto error;
+    }
+
+    pcre2len = cmd_len;
+    rc = pcre2_substring_copy_bynumber(match, 1, (PCRE2_UCHAR8 *)cmd, &pcre2len);
+    if (rc < 0) {
+        SCLogError("pcre2_substring_copy_bynumber failed");
+        goto error;
+    }
+
+    if (count == 3) {
+        pcre2len = name_len;
+        rc = pcre2_substring_copy_bynumber(match, 2, (PCRE2_UCHAR8 *)name, &pcre2len);
+        if (rc < 0) {
+            SCLogError("pcre2_substring_copy_bynumber failed");
+            goto error;
+        }
+
+        /* Trim trailing whitespace. */
+        while (strlen(name) > 0 && isblank(name[strlen(name) - 1])) {
+            name[strlen(name) - 1] = '\0';
+        }
+
+        if (strchr(name, '|') == NULL) {
+            /* Validate name, spaces are not allowed. */
+            for (size_t i = 0; i < strlen(name); i++) {
+                if (isblank(name[i])) {
+                    SCLogError("spaces not allowed in flowbit names");
+                    goto error;
                 }
             }
-            strlcpy(name, token, sizeof(name));
-            name_set = true;
-        } else {
-            if (!SigMatchStrictEnabled(DETECT_FLOWBITS)) {
-                SCLogWarning(SC_ERR_INVALID_SIGNATURE,
-                        "Invalid flowbits keyword: %s. This will become an error in Suricata 7.0.",
-                        token);
-                return -4;
-            } else {
-                SCLogError(SC_ERR_INVALID_SIGNATURE, "Invalid flowbits keyword: %s", token);
-                return -1;
-            }
         }
-        token = strtok_r(NULL, ",", &context);
     }
-    if (cmd_set && !name_set) {
+
+    pcre2_match_data_free(match);
+    return 1;
+
+error:
+    if (match) {
+        pcre2_match_data_free(match);
+    }
+    return 0;
+}
+
+int DetectFlowbitSetup (DetectEngineCtx *de_ctx, Signature *s, const char *rawstr)
+{
+    DetectFlowbitsData *cd = NULL;
+    SigMatch *sm = NULL;
+    uint8_t fb_cmd = 0;
+    char fb_cmd_str[16] = "", fb_name[256] = "";
+
+    if (!DetectFlowbitParse(rawstr, fb_cmd_str, sizeof(fb_cmd_str), fb_name,
+            sizeof(fb_name))) {
         return -1;
     }
 
+    if (strcmp(fb_cmd_str,"noalert") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_NOALERT;
+    } else if (strcmp(fb_cmd_str,"isset") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_ISSET;
+    } else if (strcmp(fb_cmd_str,"isnotset") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_ISNOTSET;
+    } else if (strcmp(fb_cmd_str,"set") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_SET;
+    } else if (strcmp(fb_cmd_str,"unset") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_UNSET;
+    } else if (strcmp(fb_cmd_str,"toggle") == 0) {
+        fb_cmd = DETECT_FLOWBITS_CMD_TOGGLE;
+    } else {
+        SCLogError("ERROR: flowbits action \"%s\" is not supported.", fb_cmd_str);
+        goto error;
+    }
+
+    switch (fb_cmd) {
+        case DETECT_FLOWBITS_CMD_NOALERT:
+            if (strlen(fb_name) != 0)
+                goto error;
+            s->flags |= SIG_FLAG_NOALERT;
+            return 0;
+        case DETECT_FLOWBITS_CMD_ISNOTSET:
+        case DETECT_FLOWBITS_CMD_ISSET:
+        case DETECT_FLOWBITS_CMD_SET:
+        case DETECT_FLOWBITS_CMD_UNSET:
+        case DETECT_FLOWBITS_CMD_TOGGLE:
+        default:
+            if (strlen(fb_name) == 0)
+                goto error;
+            break;
+    }
+
     cd = SCCalloc(1, sizeof(DetectFlowbitsData));
     if (unlikely(cd == NULL))
         goto error;
-    if (strchr(name, '|') != NULL) {
-        int retval = FlowbitOrAddData(de_ctx, cd, name);
+    if (strchr(fb_name, '|') != NULL) {
+        int retval = FlowbitOrAddData(de_ctx, cd, fb_name);
         if (retval == -1) {
             goto error;
         }
-        cd->cmd = cmd;
+        cd->cmd = fb_cmd;
     } else {
-        cd->idx = VarNameStoreSetupAdd(name, VAR_TYPE_FLOW_BIT);
+        cd->idx = VarNameStoreSetupAdd(fb_name, VAR_TYPE_FLOW_BIT);
         de_ctx->max_fb_id = MAX(cd->idx, de_ctx->max_fb_id);
-        cd->cmd = cmd;
+        cd->cmd = fb_cmd;
         cd->or_list_size = 0;
         cd->or_list = NULL;
-        SCLogDebug(
-                "idx %" PRIu32 ", cmd %d, name %s", cd->idx, cmd, strlen(name) ? name : "(none)");
+        SCLogDebug("idx %" PRIu32 ", cmd %s, name %s",
+            cd->idx, fb_cmd_str, strlen(fb_name) ? fb_name : "(none)");
     }
-    *cdout = cd;
-
-    return 0;
-error:
-    if (cd != NULL)
-        DetectFlowbitFree(de_ctx, cd);
-    return -1;
-}
-
-int DetectFlowbitSetup(DetectEngineCtx *de_ctx, Signature *s, const char *rawstr)
-{
-    DetectFlowbitsData *cd = NULL;
-    SigMatch *sm = NULL;
-
-    int result = DetectFlowbitParse(de_ctx, rawstr, &cd);
-    if (result < 0) {
-        return result;
-    } else if (result == 0 && cd == NULL) {
-        s->flags |= SIG_FLAG_NOALERT;
-        return 0;
-    }
-
     /* Okay so far so good, lets get this into a SigMatch
      * and put it in the Signature. */
     sm = SigMatchAlloc();
@@ -349,8 +346,8 @@ int DetectFlowbitSetup(DetectEngineCtx *de_ctx, Signature *s, const char *rawstr
     sm->type = DETECT_FLOWBITS;
     sm->ctx = (SigMatchCtx *)cd;
 
-    switch (cd->cmd) {
-            /* case DETECT_FLOWBITS_CMD_NOALERT can't happen here */
+    switch (fb_cmd) {
+        /* case DETECT_FLOWBITS_CMD_NOALERT can't happen here */
 
         case DETECT_FLOWBITS_CMD_ISNOTSET:
         case DETECT_FLOWBITS_CMD_ISSET:
@@ -431,7 +428,7 @@ int DetectFlowbitsAnalyze(DetectEngineCtx *de_ctx)
     struct FBAnalyze *array = SCCalloc(array_size, sizeof(struct FBAnalyze));
 
     if (array == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Unable to allocate flowbit analyze array");
+        SCLogError("Unable to allocate flowbit analyze array");
         return -1;
     }
 
@@ -441,15 +438,9 @@ int DetectFlowbitsAnalyze(DetectEngineCtx *de_ctx)
     /* fill flowbit array, updating counters per sig */
     for (uint32_t i = 0; i < de_ctx->sig_array_len; i++) {
         const Signature *s = de_ctx->sig_array[i];
-        bool has_state = false;
 
         /* see if the signature uses stateful matching */
-        for (uint32_t x = DETECT_SM_LIST_DYNAMIC_START; x < s->init_data->smlists_array_size; x++) {
-            if (s->init_data->smlists[x] == NULL)
-                continue;
-            has_state = true;
-            break;
-        }
+        bool has_state = (s->init_data->buffer_index != 0);
 
         for (const SigMatch *sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
             switch (sm->type) {
@@ -601,8 +592,8 @@ int DetectFlowbitsAnalyze(DetectEngineCtx *de_ctx)
             array[i].cnts[DETECT_FLOWBITS_CMD_SET] == 0) {
 
             const Signature *s = de_ctx->sig_array[array[i].isset_sids[0]];
-            SCLogWarning(SC_WARN_FLOWBIT, "flowbit '%s' is checked but not "
-                    "set. Checked in %u and %u other sigs",
+            SCLogWarning("flowbit '%s' is checked but not "
+                         "set. Checked in %u and %u other sigs",
                     varname, s->id, array[i].isset_sids_idx - 1);
         }
         if (array[i].state_cnts[DETECT_FLOWBITS_CMD_ISSET] &&
@@ -760,47 +751,48 @@ static void DetectFlowbitsAnalyzeDump(const DetectEngineCtx *de_ctx,
 
 static int FlowBitsTestParse01(void)
 {
-    DetectEngineCtx *de_ctx = NULL;
-    de_ctx = DetectEngineCtxInit();
-    FAIL_IF_NULL(de_ctx);
-    de_ctx->flags |= DE_QUIET;
-    DetectFlowbitsData *cd = NULL;
+    char command[16] = "", name[16] = "";
 
-#define BAD_INPUT(str) FAIL_IF_NOT(DetectFlowbitParse(de_ctx, (str), &cd) == -1);
+    /* Single argument version. */
+    FAIL_IF(!DetectFlowbitParse("noalert", command, sizeof(command), name,
+            sizeof(name)));
+    FAIL_IF(strcmp(command, "noalert") != 0);
 
-    BAD_INPUT("alert");
-    BAD_INPUT("n0alert");
-    BAD_INPUT("nOalert");
-    BAD_INPUT("set,namewith space");
-    BAD_INPUT("issset,typo");
-    BAD_INPUT("isset,a b | c");
+    /* No leading or trailing spaces. */
+    FAIL_IF(!DetectFlowbitParse("set,flowbit", command, sizeof(command), name,
+            sizeof(name)));
+    FAIL_IF(strcmp(command, "set") != 0);
+    FAIL_IF(strcmp(name, "flowbit") != 0);
 
-#undef BAD_INPUT
+    /* Leading space. */
+    FAIL_IF(!DetectFlowbitParse("set, flowbit", command, sizeof(command), name,
+            sizeof(name)));
+    FAIL_IF(strcmp(command, "set") != 0);
+    FAIL_IF(strcmp(name, "flowbit") != 0);
 
-#define GOOD_INPUT(str, command)                                                                   \
-    FAIL_IF_NOT(DetectFlowbitParse(de_ctx, (str), &cd) == 0);                                      \
-    FAIL_IF_NULL(cd);                                                                              \
-    FAIL_IF_NOT(cd->cmd == (command));                                                             \
-    DetectFlowbitFree(NULL, cd);                                                                   \
-    cd = NULL;
+    /* Trailing space. */
+    FAIL_IF(!DetectFlowbitParse("set,flowbit ", command, sizeof(command), name,
+            sizeof(name)));
+    FAIL_IF(strcmp(command, "set") != 0);
+    FAIL_IF(strcmp(name, "flowbit") != 0);
 
-    GOOD_INPUT("unset, flowbit ", DETECT_FLOWBITS_CMD_UNSET);
-    GOOD_INPUT("set,flowbit", DETECT_FLOWBITS_CMD_SET);
-    GOOD_INPUT("isset, flowbit", DETECT_FLOWBITS_CMD_ISSET);
-    GOOD_INPUT("isnotset,flowbit ", DETECT_FLOWBITS_CMD_ISNOTSET);
-    GOOD_INPUT("toggle, flowbit ", DETECT_FLOWBITS_CMD_TOGGLE);
-    GOOD_INPUT("isset, fb1|fb2", DETECT_FLOWBITS_CMD_ISSET);
+    /* Leading and trailing space. */
+    FAIL_IF(!DetectFlowbitParse("set, flowbit ", command, sizeof(command), name,
+            sizeof(name)));
+    FAIL_IF(strcmp(command, "set") != 0);
+    FAIL_IF(strcmp(name, "flowbit") != 0);
 
-#undef GOOD_INPUT
+    /* Spaces are not allowed in the name. */
+    FAIL_IF(DetectFlowbitParse("set,namewith space", command, sizeof(command),
+            name, sizeof(name)));
 
-    DetectEngineCtxFree(de_ctx);
     PASS;
 }
 
 /**
  * \test FlowBitsTestSig01 is a test for a valid noalert flowbits option
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -825,7 +817,7 @@ static int FlowBitsTestSig01(void)
 /**
  * \test FlowBitsTestSig02 is a test for a valid isset,set,isnotset,unset,toggle flowbits options
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -869,7 +861,7 @@ static int FlowBitsTestSig02(void)
 /**
  * \test FlowBitsTestSig03 is a test for a invalid flowbits option
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -894,7 +886,7 @@ static int FlowBitsTestSig03(void)
 /**
  * \test FlowBitsTestSig04 is a test check idx value
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -922,7 +914,7 @@ static int FlowBitsTestSig04(void)
 /**
  * \test FlowBitsTestSig05 is a test check noalert flag
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -948,7 +940,7 @@ static int FlowBitsTestSig05(void)
 /**
  * \test FlowBitsTestSig06 is a test set flowbits option
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -1021,7 +1013,7 @@ static int FlowBitsTestSig06(void)
 /**
  * \test FlowBitsTestSig07 is a test unset flowbits option
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -1094,9 +1086,9 @@ static int FlowBitsTestSig07(void)
 }
 
 /**
- * \test FlowBitsTestSig08 is a test toogle flowbits option
+ * \test FlowBitsTestSig08 is a test toggle flowbits option
  *
- *  \retval 1 on succces
+ *  \retval 1 on success
  *  \retval 0 on failure
  */
 
@@ -1169,239 +1161,6 @@ static int FlowBitsTestSig08(void)
     PASS;
 }
 
-/**
- * \test FlowBitsTestSig09 is to test isset flowbits option with oring
- *
- *  \retval 1 on success
- *  \retval 0 on failure
- */
-
-static int FlowBitsTestSig09(void)
-{
-    uint8_t *buf = (uint8_t *)
-                    "GET /one/ HTTP/1.1\r\n"
-                    "Host: one.example.org\r\n"
-                    "\r\n";
-    uint16_t buflen = strlen((char *)buf);
-    Packet *p = PacketGetFromAlloc();
-    FAIL_IF_NULL(p);
-    Signature *s = NULL;
-    ThreadVars th_v;
-    DetectEngineThreadCtx *det_ctx = NULL;
-    DetectEngineCtx *de_ctx = NULL;
-    Flow f;
-
-    memset(&th_v, 0, sizeof(th_v));
-    memset(&f, 0, sizeof(Flow));
-
-    FLOW_INITIALIZE(&f);
-    p->flow = &f;
-
-    p->src.family = AF_INET;
-    p->dst.family = AF_INET;
-    p->payload = buf;
-    p->payload_len = buflen;
-    p->proto = IPPROTO_TCP;
-    p->flags |= PKT_HAS_FLOW;
-    p->flowflags |= FLOW_PKT_TOSERVER;
-
-    de_ctx = DetectEngineCtxInit();
-    FAIL_IF_NULL(de_ctx);
-
-    de_ctx->flags |= DE_QUIET;
-
-    s = de_ctx->sig_list = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb1; sid:1;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb2; sid:2;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit isset ored flowbits\"; flowbits:isset,fb3|fb4; sid:3;)");
-    FAIL_IF_NULL(s);
-
-    SigGroupBuild(de_ctx);
-    DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
-
-    SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
-
-    FAIL_IF_NOT(PacketAlertCheck(p, 1));
-    FAIL_IF_NOT(PacketAlertCheck(p, 2));
-    FAIL_IF(PacketAlertCheck(p, 3));
-
-    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-    DetectEngineCtxFree(de_ctx);
-
-    FLOW_DESTROY(&f);
-
-    SCFree(p);
-    PASS;
-}
-
-/**
- * \test FlowBitsTestSig10 is to test isset flowbits option with oring
- *
- *  \retval 1 on success
- *  \retval 0 on failure
- */
-
-static int FlowBitsTestSig10(void)
-{
-    uint8_t *buf = (uint8_t *)
-                    "GET /one/ HTTP/1.1\r\n"
-                    "Host: one.example.org\r\n"
-                    "\r\n";
-    uint16_t buflen = strlen((char *)buf);
-    Packet *p = PacketGetFromAlloc();
-    FAIL_IF_NULL(p);
-    Signature *s = NULL;
-    ThreadVars th_v;
-    DetectEngineThreadCtx *det_ctx = NULL;
-    DetectEngineCtx *de_ctx = NULL;
-    Flow f;
-
-    memset(&th_v, 0, sizeof(th_v));
-    memset(&f, 0, sizeof(Flow));
-
-    FLOW_INITIALIZE(&f);
-    p->flow = &f;
-
-    p->src.family = AF_INET;
-    p->dst.family = AF_INET;
-    p->payload = buf;
-    p->payload_len = buflen;
-    p->proto = IPPROTO_TCP;
-    p->flags |= PKT_HAS_FLOW;
-    p->flowflags |= FLOW_PKT_TOSERVER;
-
-    de_ctx = DetectEngineCtxInit();
-    FAIL_IF_NULL(de_ctx);
-
-    de_ctx->flags |= DE_QUIET;
-
-    s = de_ctx->sig_list = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb1; sid:1;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb2; sid:2;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb3; sid:3;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit isset ored flowbits\"; flowbits:isset,fb3|fb4; sid:4;)");
-    FAIL_IF_NULL(s);
-
-    SigGroupBuild(de_ctx);
-    DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
-
-    SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
-
-    FAIL_IF_NOT(PacketAlertCheck(p, 1));
-    FAIL_IF_NOT(PacketAlertCheck(p, 2));
-    FAIL_IF_NOT(PacketAlertCheck(p, 3));
-    FAIL_IF_NOT(PacketAlertCheck(p, 4));
-
-    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-    DetectEngineCtxFree(de_ctx);
-
-    FLOW_DESTROY(&f);
-
-    SCFree(p);
-    PASS;
-}
-
-/**
- * \test FlowBitsTestSig11 is to test isnotset flowbits option with oring
- *
- *  \retval 1 on success
- *  \retval 0 on failure
- */
-
-static int FlowBitsTestSig11(void)
-{
-    uint8_t *buf = (uint8_t *)
-                    "GET /one/ HTTP/1.1\r\n"
-                    "Host: one.example.org\r\n"
-                    "\r\n";
-    uint16_t buflen = strlen((char *)buf);
-    Packet *p = PacketGetFromAlloc();
-    FAIL_IF_NULL(p);
-    Signature *s = NULL;
-    ThreadVars th_v;
-    DetectEngineThreadCtx *det_ctx = NULL;
-    DetectEngineCtx *de_ctx = NULL;
-    Flow f;
-
-    memset(&th_v, 0, sizeof(th_v));
-    memset(&f, 0, sizeof(Flow));
-
-    FLOW_INITIALIZE(&f);
-    p->flow = &f;
-
-    p->src.family = AF_INET;
-    p->dst.family = AF_INET;
-    p->payload = buf;
-    p->payload_len = buflen;
-    p->proto = IPPROTO_TCP;
-    p->flags |= PKT_HAS_FLOW;
-    p->flowflags |= FLOW_PKT_TOSERVER;
-
-    de_ctx = DetectEngineCtxInit();
-    FAIL_IF_NULL(de_ctx);
-
-    de_ctx->flags |= DE_QUIET;
-
-    s = de_ctx->sig_list = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb1; sid:1;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb2; sid:2;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit set\"; flowbits:set,fb3; sid:3;)");
-    FAIL_IF_NULL(s);
-    s = s->next  = SigInit(de_ctx,"alert ip any any -> any any (msg:\"Flowbit isnotset ored flowbits\"; flowbits:isnotset, fb1 | fb2 ; sid:4;)");
-    FAIL_IF_NULL(s);
-
-    SigGroupBuild(de_ctx);
-    DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
-
-    SigMatchSignatures(&th_v, de_ctx, det_ctx, p);
-
-    FAIL_IF_NOT(PacketAlertCheck(p, 1));
-    FAIL_IF_NOT(PacketAlertCheck(p, 2));
-    FAIL_IF_NOT(PacketAlertCheck(p, 3));
-    FAIL_IF(PacketAlertCheck(p, 4));
-
-    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-    DetectEngineCtxFree(de_ctx);
-
-    FLOW_DESTROY(&f);
-
-    SCFree(p);
-    PASS;
-}
-
-/**
- * \test FlowBitsTestSig12 is a test to check random arguments to
- *  flowbits keyword are rejected
- *  See https://redmine.openinfosecfoundation.org/issues/5154
- *  \retval 1 on succces
- *  \retval 0 on failure
- */
-
-static int FlowBitsTestSig12(void)
-{
-    Signature *s = NULL;
-    DetectEngineCtx *de_ctx = NULL;
-
-    de_ctx = DetectEngineCtxInit();
-    FAIL_IF_NULL(de_ctx);
-
-    de_ctx->flags |= DE_QUIET;
-
-    s = DetectEngineAppendSig(de_ctx,
-            "alert http any any -> any any (msg:\"flowbits with noalert option\"; "
-            "flow:established,to_server; http.method; content:\"POST\"; "
-            "flowbits:set,ET.whatever,asdfasdf; sid:7;)");
-    FAIL_IF_NOT_NULL(s);
-
-    SigGroupBuild(de_ctx);
-    DetectEngineCtxFree(de_ctx);
-    PASS;
-}
-
 /**
  * \brief this function registers unit tests for FlowBits
  */
@@ -1416,9 +1175,5 @@ void FlowBitsRegisterTests(void)
     UtRegisterTest("FlowBitsTestSig06", FlowBitsTestSig06);
     UtRegisterTest("FlowBitsTestSig07", FlowBitsTestSig07);
     UtRegisterTest("FlowBitsTestSig08", FlowBitsTestSig08);
-    UtRegisterTest("FlowBitsTestSig09", FlowBitsTestSig09);
-    UtRegisterTest("FlowBitsTestSig10", FlowBitsTestSig10);
-    UtRegisterTest("FlowBitsTestSig11", FlowBitsTestSig11);
-    UtRegisterTest("FlowBitsTestSig12", FlowBitsTestSig12);
 }
 #endif /* UNITTESTS */
