@@ -29,7 +29,6 @@
 
 #include "suricata-common.h"
 #include "threads.h"
-#include "debug.h"
 #include "decode.h"
 
 #include "detect.h"
@@ -38,6 +37,7 @@
 #include "detect-engine.h"
 #include "detect-engine-mpm.h"
 #include "detect-engine-state.h"
+#include "detect-engine-build.h"
 
 #include "flow.h"
 #include "flow-var.h"
@@ -160,27 +160,28 @@ static int DetectSshVersionMatch (DetectEngineThreadCtx *det_ctx,
 static DetectSshVersionData *DetectSshVersionParse (DetectEngineCtx *de_ctx, const char *str)
 {
     DetectSshVersionData *ssh = NULL;
-    int ret = 0, res = 0;
-    int ov[MAX_SUBSTRINGS];
+    int res = 0;
+    size_t pcre2_len;
 
-    ret = DetectParsePcreExec(&parse_regex, str, 0, 0, ov, MAX_SUBSTRINGS);
+    pcre2_match_data *match = NULL;
+    int ret = DetectParsePcreExec(&parse_regex, &match, str, 0, 0);
     if (ret < 1 || ret > 3) {
-        SCLogError(SC_ERR_PCRE_MATCH, "invalid ssh.protoversion option");
+        SCLogError("invalid ssh.protoversion option");
         goto error;
     }
 
     if (ret > 1) {
         const char *str_ptr;
-        res = pcre_get_substring((char *)str, ov, MAX_SUBSTRINGS, 1, &str_ptr);
+        res = pcre2_substring_get_bynumber(match, 1, (PCRE2_UCHAR8 **)&str_ptr, &pcre2_len);
         if (res < 0) {
-            SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_get_substring failed");
+            SCLogError("pcre2_substring_get_bynumber failed");
             goto error;
         }
 
         /* We have a correct id option */
         ssh = SCMalloc(sizeof(DetectSshVersionData));
         if (unlikely(ssh == NULL)) {
-            pcre_free_substring(str_ptr);
+            pcre2_substring_free((PCRE2_UCHAR *)str_ptr);
             goto error;
         }
         memset(ssh, 0x00, sizeof(DetectSshVersionData));
@@ -190,24 +191,28 @@ static DetectSshVersionData *DetectSshVersionParse (DetectEngineCtx *de_ctx, con
         if (strcmp("2_compat", str_ptr) == 0) {
             ssh->flags |= SSH_FLAG_PROTOVERSION_2_COMPAT;
             SCLogDebug("will look for ssh protocol version 2 (2, 2.0, 1.99 that's considered as 2");
-            pcre_free_substring(str_ptr);
+            pcre2_substring_free((PCRE2_UCHAR *)str_ptr);
             return ssh;
         }
 
         ssh->ver = (uint8_t *)SCStrdup((char*)str_ptr);
         if (ssh->ver == NULL) {
-            pcre_free_substring(str_ptr);
+            pcre2_substring_free((PCRE2_UCHAR *)str_ptr);
             goto error;
         }
-        ssh->len = strlen((char *) ssh->ver);
-        pcre_free_substring(str_ptr);
+        ssh->len = (uint16_t)strlen((char *)ssh->ver);
+        pcre2_substring_free((PCRE2_UCHAR *)str_ptr);
 
         SCLogDebug("will look for ssh %s", ssh->ver);
     }
 
+    pcre2_match_data_free(match);
     return ssh;
 
 error:
+    if (match) {
+        pcre2_match_data_free(match);
+    }
     if (ssh != NULL)
         DetectSshVersionFree(de_ctx, ssh);
     return NULL;
@@ -271,6 +276,7 @@ void DetectSshVersionFree(DetectEngineCtx *de_ctx, void *ptr)
 }
 
 #ifdef UNITTESTS /* UNITTESTS */
+#include "detect-engine-alert.h"
 
 /**
  * \test DetectSshVersionTestParse01 is a test to make sure that we parse
@@ -585,4 +591,4 @@ static void DetectSshVersionRegisterTests(void)
     UtRegisterTest("DetectSshVersionTestDetect03",
                    DetectSshVersionTestDetect03);
 }
-#endif /* UNITTESTS */
\ No newline at end of file
+#endif /* UNITTESTS */
