@@ -34,6 +34,7 @@
 
 #include "util-var.h"
 #include "util-debug.h"
+#include "util-macset.h"
 #include "flow-storage.h"
 
 #include "detect.h"
@@ -149,8 +150,7 @@ void FlowInit(Flow *f, const Packet *p)
 
     f->proto = p->proto;
     f->recursion_level = p->recursion_level;
-    f->vlan_id[0] = p->vlan_id[0];
-    f->vlan_id[1] = p->vlan_id[1];
+    memcpy(&f->vlan_id[0], &p->vlan_id[0], sizeof(f->vlan_id));
     f->vlan_idx = p->vlan_idx;
     f->livedev = p->livedev;
 
@@ -186,15 +186,17 @@ void FlowInit(Flow *f, const Packet *p)
     } else if (p->sctph != NULL) { /* XXX MACRO */
         SET_SCTP_SRC_PORT(p,&f->sp);
         SET_SCTP_DST_PORT(p,&f->dp);
+    } else if (p->esph != NULL) {
+        f->esp.spi = ESP_GET_SPI(p);
     } else {
         /* nothing to do for this IP proto. */
         SCLogDebug("no special setup for IP proto %u", p->proto);
     }
-    COPY_TIMESTAMP(&p->ts, &f->startts);
+    f->startts = p->ts;
 
     f->protomap = FlowGetProtoMapping(f->proto);
     f->timeout_policy = FlowGetTimeoutPolicy(f);
-    const uint32_t timeout_at = (uint32_t)f->startts.tv_sec + f->timeout_policy;
+    const uint32_t timeout_at = (uint32_t)SCTIME_SECS(f->startts) + f->timeout_policy;
     f->timeout_at = timeout_at;
 
     if (MacSetFlowStorageEnabled()) {
@@ -235,3 +237,68 @@ void RegisterFlowBypassInfo(void)
     g_bypass_info_id = FlowStorageRegister("bypass_counters", sizeof(void *),
                                               NULL, FlowBypassFree);
 }
+
+void FlowEndCountersRegister(ThreadVars *t, FlowEndCounters *fec)
+{
+    for (int i = 0; i < FLOW_STATE_SIZE; i++) {
+        const char *name = NULL;
+        if (i == FLOW_STATE_NEW) {
+            name = "flow.end.state.new";
+        } else if (i == FLOW_STATE_ESTABLISHED) {
+            name = "flow.end.state.established";
+        } else if (i == FLOW_STATE_CLOSED) {
+            name = "flow.end.state.closed";
+        } else if (i == FLOW_STATE_LOCAL_BYPASSED) {
+            name = "flow.end.state.local_bypassed";
+#ifdef CAPTURE_OFFLOAD
+        } else if (i == FLOW_STATE_CAPTURE_BYPASSED) {
+            name = "flow.end.state.capture_bypassed";
+#endif
+        }
+        if (name) {
+            fec->flow_state[i] = StatsRegisterCounter(name, t);
+        }
+    }
+
+    for (enum TcpState i = TCP_NONE; i <= TCP_CLOSED; i++) {
+        const char *name = NULL;
+        switch (i) {
+            case TCP_NONE:
+                name = "flow.end.tcp_state.none";
+                break;
+            case TCP_SYN_SENT:
+                name = "flow.end.tcp_state.syn_sent";
+                break;
+            case TCP_SYN_RECV:
+                name = "flow.end.tcp_state.syn_recv";
+                break;
+            case TCP_ESTABLISHED:
+                name = "flow.end.tcp_state.established";
+                break;
+            case TCP_FIN_WAIT1:
+                name = "flow.end.tcp_state.fin_wait1";
+                break;
+            case TCP_FIN_WAIT2:
+                name = "flow.end.tcp_state.fin_wait2";
+                break;
+            case TCP_TIME_WAIT:
+                name = "flow.end.tcp_state.time_wait";
+                break;
+            case TCP_LAST_ACK:
+                name = "flow.end.tcp_state.last_ack";
+                break;
+            case TCP_CLOSE_WAIT:
+                name = "flow.end.tcp_state.close_wait";
+                break;
+            case TCP_CLOSING:
+                name = "flow.end.tcp_state.closing";
+                break;
+            case TCP_CLOSED:
+                name = "flow.end.tcp_state.closed";
+                break;
+        }
+
+        fec->flow_tcp_state[i] = StatsRegisterCounter(name, t);
+    }
+    fec->flow_tcp_liberal = StatsRegisterCounter("flow.end.tcp_liberal", t);
+}
