@@ -25,6 +25,9 @@
 
 #include "suricata-common.h"
 #include "suricata.h"
+
+#include "action-globals.h"
+#include "packet.h"
 #include "decode.h"
 #include "conf.h"
 #include "threadvars.h"
@@ -54,6 +57,7 @@
 #include "util-unittest-helper.h"
 #include "util-byte.h"
 #include "util-misc.h"
+#include "util-macset.h"
 
 #include "util-debug.h"
 #include "util-privs.h"
@@ -80,7 +84,7 @@ SC_ATOMIC_DECLARE(FlowProtoTimeoutPtr, flow_timeouts);
 /** atomic int that is used when freeing a flow from the hash. In this
  *  case we walk the hash to find a flow to free. This var records where
  *  we left off in the hash. Without this only the top rows of the hash
- *  are freed. This isn't just about fairness. Under severe presure, the
+ *  are freed. This isn't just about fairness. Under severe pressure, the
  *  hash rows on top would be all freed and the time to find a flow to
  *  free increased with every run. */
 SC_ATOMIC_DECLARE(unsigned int, flow_prune_idx);
@@ -186,6 +190,23 @@ int FlowHasAlerts(const Flow *f)
     return 0;
 }
 
+bool FlowHasGaps(const Flow *f, uint8_t way)
+{
+    if (f->proto == IPPROTO_TCP) {
+        TcpSession *ssn = (TcpSession *)f->protoctx;
+        if (ssn != NULL) {
+            if (way == STREAM_TOCLIENT) {
+                if (ssn->server.flags & STREAMTCP_STREAM_FLAG_HAS_GAP)
+                    return 1;
+            } else {
+                if (ssn->client.flags & STREAMTCP_STREAM_FLAG_HAS_GAP)
+                    return 1;
+            }
+        }
+    }
+    return 0;
+}
+
 /** \brief Set flag to indicate to change proto for the flow
  *
  * \param f flow
@@ -341,23 +362,24 @@ static inline int FlowUpdateSeenFlag(const Packet *p)
     return 1;
 }
 
-static inline void FlowUpdateTTL(Flow *f, Packet *p, uint8_t ttl)
+static inline void FlowUpdateTtlTS(Flow *f, Packet *p, uint8_t ttl)
 {
-    if (FlowGetPacketDirection(f, p) == TOSERVER) {
-        if (f->min_ttl_toserver == 0) {
-            f->min_ttl_toserver = ttl;
-        } else {
-            f->min_ttl_toserver = MIN(f->min_ttl_toserver, ttl);
-        }
-        f->max_ttl_toserver = MAX(f->max_ttl_toserver, ttl);
+    if (f->min_ttl_toserver == 0) {
+        f->min_ttl_toserver = ttl;
     } else {
-        if (f->min_ttl_toclient == 0) {
-            f->min_ttl_toclient = ttl;
-        } else {
-            f->min_ttl_toclient = MIN(f->min_ttl_toclient, ttl);
-        }
-        f->max_ttl_toclient = MAX(f->max_ttl_toclient, ttl);
+        f->min_ttl_toserver = MIN(f->min_ttl_toserver, ttl);
     }
+    f->max_ttl_toserver = MAX(f->max_ttl_toserver, ttl);
+}
+
+static inline void FlowUpdateTtlTC(Flow *f, Packet *p, uint8_t ttl)
+{
+    if (f->min_ttl_toclient == 0) {
+        f->min_ttl_toclient = ttl;
+    } else {
+        f->min_ttl_toclient = MIN(f->min_ttl_toclient, ttl);
+    }
+    f->max_ttl_toclient = MAX(f->max_ttl_toclient, ttl);
 }
 
 static inline void FlowUpdateEthernet(ThreadVars *tv, DecodeThreadVars *dtv,
@@ -392,15 +414,16 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
 {
     SCLogDebug("packet %"PRIu64" -- flow %p", p->pcap_cnt, f);
 
+    const int pkt_dir = FlowGetPacketDirection(f, p);
 #ifdef CAPTURE_OFFLOAD
     int state = f->flow_state;
 
     if (state != FLOW_STATE_CAPTURE_BYPASSED) {
 #endif
         /* update the last seen timestamp of this flow */
-        if (timercmp(&p->ts, &f->lastts, >)) {
-            COPY_TIMESTAMP(&p->ts, &f->lastts);
-            const uint32_t timeout_at = (uint32_t)f->lastts.tv_sec + f->timeout_policy;
+        if (SCTIME_CMP_GT(p->ts, f->lastts)) {
+            f->lastts = p->ts;
+            const uint32_t timeout_at = (uint32_t)SCTIME_SECS(f->lastts) + f->timeout_policy;
             if (timeout_at != f->timeout_at) {
                 f->timeout_at = timeout_at;
             }
@@ -408,9 +431,9 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
 #ifdef CAPTURE_OFFLOAD
     } else {
         /* still seeing packet, we downgrade to local bypass */
-        if (p->ts.tv_sec - f->lastts.tv_sec > FLOW_BYPASSED_TIMEOUT / 2) {
+        if (SCTIME_SECS(p->ts) - SCTIME_SECS(f->lastts) > FLOW_BYPASSED_TIMEOUT / 2) {
             SCLogDebug("Downgrading flow to local bypass");
-            COPY_TIMESTAMP(&p->ts, &f->lastts);
+            f->lastts = p->ts;
             FlowUpdateState(f, FLOW_STATE_LOCAL_BYPASSED);
         } else {
             /* In IPS mode the packet could come from the other interface so it would
@@ -422,7 +445,7 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
     }
 #endif
     /* update flags and counters */
-    if (FlowGetPacketDirection(f, p) == TOSERVER) {
+    if (pkt_dir == TOSERVER) {
         f->todstpktcnt++;
         f->todstbytecnt += GET_PKT_LEN(p);
         p->flowflags = FLOW_PKT_TOSERVER;
@@ -438,6 +461,12 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
             p->flags |= PKT_PROTO_DETECT_TS_DONE;
         }
         FlowUpdateEthernet(tv, dtv, f, p->ethh, true);
+        /* update flow's ttl fields if needed */
+        if (PKT_IS_IPV4(p)) {
+            FlowUpdateTtlTS(f, p, IPV4_GET_IPTTL(p));
+        } else if (PKT_IS_IPV6(p)) {
+            FlowUpdateTtlTS(f, p, IPV6_GET_HLIM(p));
+        }
     } else {
         f->tosrcpktcnt++;
         f->tosrcbytecnt += GET_PKT_LEN(p);
@@ -454,6 +483,12 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
             p->flags |= PKT_PROTO_DETECT_TC_DONE;
         }
         FlowUpdateEthernet(tv, dtv, f, p->ethh, false);
+        /* update flow's ttl fields if needed */
+        if (PKT_IS_IPV4(p)) {
+            FlowUpdateTtlTC(f, p, IPV4_GET_IPTTL(p));
+        } else if (PKT_IS_IPV6(p)) {
+            FlowUpdateTtlTC(f, p, IPV6_GET_HLIM(p));
+        }
     }
 
     if (f->flow_state == FLOW_STATE_ESTABLISHED) {
@@ -473,6 +508,9 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
         FlowUpdateState(f, FLOW_STATE_ESTABLISHED);
     }
 
+    if (f->flags & FLOW_ACTION_DROP) {
+        PacketDrop(p, ACTION_DROP, PKT_DROP_REASON_FLOW_DROP);
+    }
     /*set the detection bypass flags*/
     if (f->flags & FLOW_NOPACKET_INSPECTION) {
         SCLogDebug("setting FLOW_NOPACKET_INSPECTION flag on flow %p", f);
@@ -482,13 +520,6 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
         SCLogDebug("setting FLOW_NOPAYLOAD_INSPECTION flag on flow %p", f);
         DecodeSetNoPayloadInspectionFlag(p);
     }
-
-    /* update flow's ttl fields if needed */
-    if (PKT_IS_IPV4(p)) {
-        FlowUpdateTTL(f, p, IPV4_GET_IPTTL(p));
-    } else if (PKT_IS_IPV6(p)) {
-        FlowUpdateTTL(f, p, IPV6_GET_HLIM(p));
-    }
 }
 
 /** \brief Entry point for packet flow handling
@@ -502,7 +533,7 @@ void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars
 void FlowHandlePacket(ThreadVars *tv, FlowLookupStruct *fls, Packet *p)
 {
     /* Get this packet's flow from the hash. FlowHandlePacket() will setup
-     * a new flow if nescesary. If we get NULL, we're out of flow memory.
+     * a new flow if necessary. If we get NULL, we're out of flow memory.
      * The returned flow is locked. */
     Flow *f = FlowGetFlowFromHash(tv, fls, p, &p->flow);
     if (f == NULL)
@@ -515,7 +546,7 @@ void FlowHandlePacket(ThreadVars *tv, FlowLookupStruct *fls, Packet *p)
 
 /** \brief initialize the configuration
  *  \warning Not thread safe */
-void FlowInitConfig(char quiet)
+void FlowInitConfig(bool quiet)
 {
     SCLogDebug("initializing flow engine...");
 
@@ -539,8 +570,8 @@ void FlowInitConfig(char quiet)
         if (val <= 100 && val >= 1) {
             flow_config.emergency_recovery = (uint8_t)val;
         } else {
-            SCLogError(SC_ERR_INVALID_VALUE, "flow.emergency-recovery must be in the range of "
-                                             "1 and 100 (as percentage)");
+            SCLogError("flow.emergency-recovery must be in the range of "
+                       "1 and 100 (as percentage)");
             flow_config.emergency_recovery = FLOW_DEFAULT_EMERGENCY_RECOVERY;
         }
     } else {
@@ -557,13 +588,13 @@ void FlowInitConfig(char quiet)
     if ((ConfGet("flow.memcap", &conf_val)) == 1)
     {
         if (conf_val == NULL) {
-            FatalError(SC_ERR_FATAL, "Invalid value for flow.memcap: NULL");
+            FatalError("Invalid value for flow.memcap: NULL");
         }
 
         if (ParseSizeStringU64(conf_val, &flow_memcap_copy) < 0) {
-            SCLogError(SC_ERR_SIZE_PARSE, "Error parsing flow.memcap "
+            SCLogError("Error parsing flow.memcap "
                        "from conf file - %s.  Killing engine",
-                       conf_val);
+                    conf_val);
             exit(EXIT_FAILURE);
         } else {
             SC_ATOMIC_SET(flow_config.memcap, flow_memcap_copy);
@@ -572,7 +603,7 @@ void FlowInitConfig(char quiet)
     if ((ConfGet("flow.hash-size", &conf_val)) == 1)
     {
         if (conf_val == NULL) {
-            FatalError(SC_ERR_FATAL, "Invalid value for flow.hash-size: NULL");
+            FatalError("Invalid value for flow.hash-size: NULL");
         }
 
         if (StringParseUint32(&configval, 10, strlen(conf_val),
@@ -583,7 +614,7 @@ void FlowInitConfig(char quiet)
     if ((ConfGet("flow.prealloc", &conf_val)) == 1)
     {
         if (conf_val == NULL) {
-            FatalError(SC_ERR_FATAL, "Invalid value for flow.prealloc: NULL");
+            FatalError("Invalid value for flow.prealloc: NULL");
         }
 
         if (StringParseUint32(&configval, 10, strlen(conf_val),
@@ -601,18 +632,17 @@ void FlowInitConfig(char quiet)
     /* alloc hash memory */
     uint64_t hash_size = flow_config.hash_size * sizeof(FlowBucket);
     if (!(FLOW_CHECK_MEMCAP(hash_size))) {
-        SCLogError(SC_ERR_FLOW_INIT, "allocating flow hash failed: "
-                "max flow memcap is smaller than projected hash size. "
-                "Memcap: %"PRIu64", Hash table size %"PRIu64". Calculate "
-                "total hash size by multiplying \"flow.hash-size\" with %"PRIuMAX", "
-                "which is the hash bucket size.", SC_ATOMIC_GET(flow_config.memcap), hash_size,
-                (uintmax_t)sizeof(FlowBucket));
+        SCLogError("allocating flow hash failed: "
+                   "max flow memcap is smaller than projected hash size. "
+                   "Memcap: %" PRIu64 ", Hash table size %" PRIu64 ". Calculate "
+                   "total hash size by multiplying \"flow.hash-size\" with %" PRIuMAX ", "
+                   "which is the hash bucket size.",
+                SC_ATOMIC_GET(flow_config.memcap), hash_size, (uintmax_t)sizeof(FlowBucket));
         exit(EXIT_FAILURE);
     }
     flow_hash = SCMallocAligned(flow_config.hash_size * sizeof(FlowBucket), CLS);
     if (unlikely(flow_hash == NULL)) {
-        FatalError(SC_ERR_FATAL,
-                   "Fatal error encountered in FlowInitConfig. Exiting...");
+        FatalError("Fatal error encountered in FlowInitConfig. Exiting...");
     }
     memset(flow_hash, 0, flow_config.hash_size * sizeof(FlowBucket));
 
@@ -623,14 +653,14 @@ void FlowInitConfig(char quiet)
     }
     (void) SC_ATOMIC_ADD(flow_memuse, (flow_config.hash_size * sizeof(FlowBucket)));
 
-    if (quiet == FALSE) {
+    if (!quiet) {
         SCLogConfig("allocated %"PRIu64" bytes of memory for the flow hash... "
                   "%" PRIu32 " buckets of size %" PRIuMAX "",
                   SC_ATOMIC_GET(flow_memuse), flow_config.hash_size,
                   (uintmax_t)sizeof(FlowBucket));
     }
     FlowSparePoolInit();
-    if (quiet == FALSE) {
+    if (!quiet) {
         SCLogConfig("flow memory usage: %"PRIu64" bytes, maximum: %"PRIu64,
                 SC_ATOMIC_GET(flow_memuse), SC_ATOMIC_GET(flow_config.memcap));
     }
@@ -644,6 +674,22 @@ void FlowInitConfig(char quiet)
     return;
 }
 
+void FlowReset(void)
+{
+    // resets the flows (for reuse by fuzzing)
+    for (uint32_t u = 0; u < flow_config.hash_size; u++) {
+        Flow *f = flow_hash[u].head;
+        while (f) {
+            Flow *n = f->next;
+            uint8_t proto_map = FlowGetProtoMapping(f->proto);
+            FlowClearMemory(f, proto_map);
+            FlowFree(f);
+            f = n;
+        }
+        flow_hash[u].head = NULL;
+    }
+}
+
 /** \brief shutdown the flow engine
  *  \warning Not thread safe */
 void FlowShutdown(void)
@@ -659,7 +705,6 @@ void FlowShutdown(void)
         for (uint32_t u = 0; u < flow_config.hash_size; u++) {
             f = flow_hash[u].head;
             while (f) {
-                DEBUG_VALIDATE_BUG_ON(f->use_cnt != 0);
                 Flow *n = f->next;
                 uint8_t proto_map = FlowGetProtoMapping(f->proto);
                 FlowClearMemory(f, proto_map);
@@ -668,7 +713,6 @@ void FlowShutdown(void)
             }
             f = flow_hash[u].evicted;
             while (f) {
-                DEBUG_VALIDATE_BUG_ON(f->use_cnt != 0);
                 Flow *n = f->next;
                 uint8_t proto_map = FlowGetProtoMapping(f->proto);
                 FlowClearMemory(f, proto_map);
@@ -994,26 +1038,30 @@ void FlowInitFlowProto(void)
         FlowProtoTimeout *e = &flow_timeouts_emerg[i];
 
         if (e->est_timeout > n->est_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value %u for \'established\' "
-                    "must be below regular value %u", e->est_timeout, n->est_timeout);
+            SCLogWarning("emergency timeout value %u for \'established\' "
+                         "must be below regular value %u",
+                    e->est_timeout, n->est_timeout);
             e->est_timeout = n->est_timeout / 10;
         }
 
         if (e->new_timeout > n->new_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value %u for \'new\' must be "
-                    "below regular value %u", e->new_timeout, n->new_timeout);
+            SCLogWarning("emergency timeout value %u for \'new\' must be "
+                         "below regular value %u",
+                    e->new_timeout, n->new_timeout);
             e->new_timeout = n->new_timeout / 10;
         }
 
         if (e->closed_timeout > n->closed_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value %u for \'closed\' must "
-                    "be below regular value %u", e->closed_timeout, n->closed_timeout);
+            SCLogWarning("emergency timeout value %u for \'closed\' must "
+                         "be below regular value %u",
+                    e->closed_timeout, n->closed_timeout);
             e->closed_timeout = n->closed_timeout / 10;
         }
 
         if (e->bypassed_timeout > n->bypassed_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value %u for \'bypassed\' "
-                    "must be below regular value %u", e->bypassed_timeout, n->bypassed_timeout);
+            SCLogWarning("emergency timeout value %u for \'bypassed\' "
+                         "must be below regular value %u",
+                    e->bypassed_timeout, n->bypassed_timeout);
             e->bypassed_timeout = n->bypassed_timeout / 10;
         }
     }
@@ -1024,25 +1072,25 @@ void FlowInitFlowProto(void)
         FlowProtoTimeout *d = &flow_timeouts_delta[i];
 
         if (e->est_timeout > n->est_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value for \'established\' must be below normal value");
+            SCLogWarning("emergency timeout value for \'established\' must be below normal value");
             e->est_timeout = n->est_timeout / 10;
         }
         d->est_timeout = n->est_timeout - e->est_timeout;
 
         if (e->new_timeout > n->new_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value for \'new\' must be below normal value");
+            SCLogWarning("emergency timeout value for \'new\' must be below normal value");
             e->new_timeout = n->new_timeout / 10;
         }
         d->new_timeout = n->new_timeout - e->new_timeout;
 
         if (e->closed_timeout > n->closed_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value for \'closed\' must be below normal value");
+            SCLogWarning("emergency timeout value for \'closed\' must be below normal value");
             e->closed_timeout = n->closed_timeout / 10;
         }
         d->closed_timeout = n->closed_timeout - e->closed_timeout;
 
         if (e->bypassed_timeout > n->bypassed_timeout) {
-            SCLogWarning(SC_WARN_FLOW_EMERGENCY, "emergency timeout value for \'bypassed\' must be below normal value");
+            SCLogWarning("emergency timeout value for \'bypassed\' must be below normal value");
             e->bypassed_timeout = n->bypassed_timeout / 10;
         }
         d->bypassed_timeout = n->bypassed_timeout - e->bypassed_timeout;
@@ -1099,20 +1147,10 @@ int FlowSetProtoFreeFunc (uint8_t proto, void (*Free)(void *))
     return 1;
 }
 
-AppProto FlowGetAppProtocol(const Flow *f)
-{
-    return f->alproto;
-}
-
-void *FlowGetAppState(const Flow *f)
-{
-    return f->alstate;
-}
-
 /**
  *  \brief get 'disruption' flags: GAP/DEPTH/PASS
  *  \param f locked flow
- *  \param flags existing flags to be ammended
+ *  \param flags existing flags to be amended
  *  \retval flags original flags + disrupt flags (if any)
  *  \TODO handle UDP
  */
@@ -1141,13 +1179,14 @@ void FlowUpdateState(Flow *f, const enum FlowState s)
 {
     if (s != f->flow_state) {
         /* set the state */
-        f->flow_state = s;
+        // Explicit cast from the enum type to the compact version
+        f->flow_state = (FlowStateType)s;
 
         /* update timeout policy and value */
         const uint32_t timeout_policy = FlowGetTimeoutPolicy(f);
         if (timeout_policy != f->timeout_policy) {
             f->timeout_policy = timeout_policy;
-            const uint32_t timeout_at = (uint32_t)f->lastts.tv_sec + timeout_policy;
+            const uint32_t timeout_at = (uint32_t)SCTIME_SECS(f->lastts) + timeout_policy;
             if (timeout_at != f->timeout_at)
                 f->timeout_at = timeout_at;
         }
@@ -1155,7 +1194,7 @@ void FlowUpdateState(Flow *f, const enum FlowState s)
 #ifdef UNITTESTS
     if (f->fb != NULL) {
 #endif
-        /* and reset the flow buckup next_ts value so that the flow manager
+        /* and reset the flow bucket next_ts value so that the flow manager
          * has to revisit this row */
         SC_ATOMIC_SET(f->fb->next_ts, 0);
 #ifdef UNITTESTS
@@ -1172,10 +1211,43 @@ void FlowUpdateState(Flow *f, const enum FlowState s)
  */
 void FlowGetLastTimeAsParts(Flow *flow, uint64_t *secs, uint64_t *usecs)
 {
-    *secs = (uint64_t)flow->lastts.tv_sec;
-    *usecs = (uint64_t)flow->lastts.tv_usec;
+    *secs = (uint64_t)SCTIME_SECS(flow->lastts);
+    *usecs = (uint64_t)SCTIME_USECS(flow->lastts);
 }
 
+/**
+ * \brief Get flow source port.
+ *
+ * A function to get the flow sport useful when the caller only has an
+ * opaque pointer to the flow structure.
+ */
+uint16_t FlowGetSourcePort(Flow *flow)
+{
+    return flow->sp;
+}
+
+/**
+ * \brief Get flow destination port.
+ *
+ * A function to get the flow dport useful when the caller only has an
+ * opaque pointer to the flow structure.
+ */
+
+uint16_t FlowGetDestinationPort(Flow *flow)
+{
+    return flow->dp;
+}
+/**
+ * \brief Get flow flags.
+ *
+ * A function to get the flow flags useful when the caller only has an
+ * opaque pointer to the flow structure.
+ */
+
+uint32_t FlowGetFlags(Flow *flow)
+{
+    return flow->flags;
+}
 /************************************Unittests*******************************/
 
 #ifdef UNITTESTS
