@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2020 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -24,7 +24,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -119,6 +118,7 @@
 
 #define LOG_FORMAT_GROUPED     BIT_U64(60)
 #define LOG_FORMAT_DETAILED    BIT_U64(61)
+#define LOG_HTTPS              BIT_U64(62)
 
 #define LOG_FORMAT_ALL (LOG_FORMAT_GROUPED|LOG_FORMAT_DETAILED)
 #define LOG_ALL_RRTYPES (~(uint64_t)(LOG_QUERIES|LOG_ANSWERS|LOG_FORMAT_DETAILED|LOG_FORMAT_GROUPED))
@@ -176,6 +176,7 @@ typedef enum {
     DNS_RRTYPE_HIP,
     DNS_RRTYPE_CDS,
     DNS_RRTYPE_CDNSKEY,
+    DNS_RRTYPE_HTTPS,
     DNS_RRTYPE_SPF,
     DNS_RRTYPE_TKEY,
     DNS_RRTYPE_TSIG,
@@ -185,17 +186,11 @@ typedef enum {
     DNS_RRTYPE_MAX,
 } DnsRRTypes;
 
-typedef enum {
-    DNS_VERSION_1 = 1,
-    DNS_VERSION_2
-} DnsVersion;
-
-#define DNS_VERSION_DEFAULT DNS_VERSION_2
-
 static struct {
     const char *config_rrtype;
     uint64_t flags;
 } dns_rrtype_fields[] = {
+    // clang-format off
    { "a", LOG_A },
    { "ns", LOG_NS },
    { "md", LOG_MD },
@@ -248,36 +243,33 @@ static struct {
    { "hip", LOG_HIP },
    { "cds", LOG_CDS },
    { "cdnskey", LOG_CDNSKEY },
+   { "https", LOG_HTTPS },
    { "spf", LOG_SPF },
    { "tkey", LOG_TKEY },
    { "tsig", LOG_TSIG },
    { "maila", LOG_MAILA },
    { "any", LOG_ANY },
    { "uri", LOG_URI }
+    // clang-format on
 };
 
 typedef struct LogDnsFileCtx_ {
-    LogFileCtx *file_ctx;
     uint64_t flags; /** Store mode */
-    DnsVersion version;
-    OutputJsonCommonSettings cfg;
+    OutputJsonCtx *eve_ctx;
 } LogDnsFileCtx;
 
 typedef struct LogDnsLogThread_ {
     LogDnsFileCtx *dnslog_ctx;
-    /** LogFileCtx has the pointer to the file and a mutex to allow multithreading */
-    LogFileCtx *file_ctx;
-    MemBuffer *buffer;
+    OutputJsonThreadCtx *ctx;
 } LogDnsLogThread;
 
-static bool v1_deprecation_warned = false;
-
-JsonBuilder *JsonDNSLogQuery(void *txptr, uint64_t tx_id)
+JsonBuilder *JsonDNSLogQuery(void *txptr)
 {
     JsonBuilder *queryjb = jb_new_array();
     if (queryjb == NULL) {
         return NULL;
     }
+    bool has_query = false;
 
     for (uint16_t i = 0; i < UINT16_MAX; i++) {
         JsonBuilder *js = jb_new_object();
@@ -286,15 +278,21 @@ JsonBuilder *JsonDNSLogQuery(void *txptr, uint64_t tx_id)
             break;
         }
         jb_close(js);
+        has_query = true;
         jb_append_object(queryjb, js);
         jb_free(js);
     }
 
+    if (!has_query) {
+        jb_free(queryjb);
+        return NULL;
+    }
+
     jb_close(queryjb);
     return queryjb;
 }
 
-JsonBuilder *JsonDNSLogAnswer(void *txptr, uint64_t tx_id)
+JsonBuilder *JsonDNSLogAnswer(void *txptr)
 {
     if (!rs_dns_do_log_answer(txptr, LOG_ALL_RRTYPES)) {
         return NULL;
@@ -319,11 +317,10 @@ static int JsonDnsLoggerToServer(ThreadVars *tv, void *thread_data,
     }
 
     for (uint16_t i = 0; i < 0xffff; i++) {
-        JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL);
+        JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL, dnslog_ctx->eve_ctx);
         if (unlikely(jb == NULL)) {
             return TM_ECODE_OK;
         }
-        EveAddCommonOptions(&dnslog_ctx->cfg, p, f, jb);
 
         jb_open_object(jb, "dns");
         if (!rs_dns_log_json_query(txptr, i, td->dnslog_ctx->flags, jb)) {
@@ -332,8 +329,7 @@ static int JsonDnsLoggerToServer(ThreadVars *tv, void *thread_data,
         }
         jb_close(jb);
 
-        MemBufferReset(td->buffer);
-        OutputJsonBuilderBuffer(jb, td->file_ctx, &td->buffer);
+        OutputJsonBuilderBuffer(jb, td->ctx);
         jb_free(jb);
     }
 
@@ -352,69 +348,33 @@ static int JsonDnsLoggerToClient(ThreadVars *tv, void *thread_data,
         return TM_ECODE_OK;
     }
 
-    if (td->dnslog_ctx->version == DNS_VERSION_2) {
-        if (rs_dns_do_log_answer(txptr, td->dnslog_ctx->flags)) {
-            JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL);
-            if (unlikely(jb == NULL)) {
-                return TM_ECODE_OK;
-            }
-            EveAddCommonOptions(&dnslog_ctx->cfg, p, f, jb);
-
-            jb_open_object(jb, "dns");
-            rs_dns_log_json_answer(txptr, td->dnslog_ctx->flags, jb);
-            jb_close(jb);
-            MemBufferReset(td->buffer);
-            OutputJsonBuilderBuffer(jb, td->file_ctx, &td->buffer);
-            jb_free(jb);
+    if (rs_dns_do_log_answer(txptr, td->dnslog_ctx->flags)) {
+        JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL, dnslog_ctx->eve_ctx);
+        if (unlikely(jb == NULL)) {
+            return TM_ECODE_OK;
         }
-    } else {
-        /* Log answers. */
-        for (uint16_t i = 0; i < UINT16_MAX; i++) {
-            JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL);
-            if (unlikely(jb == NULL)) {
-                return TM_ECODE_OK;
-            }
-            EveAddCommonOptions(&dnslog_ctx->cfg, p, f, jb);
 
-            JsonBuilder *answer = rs_dns_log_json_answer_v1(txptr, i,
-                    td->dnslog_ctx->flags);
-            if (answer == NULL) {
-                jb_free(jb);
-                break;
-            }
-            jb_set_object(jb, "dns", answer);
-            jb_free(answer);
-
-            MemBufferReset(td->buffer);
-            OutputJsonBuilderBuffer(jb, td->file_ctx, &td->buffer);
-            jb_free(jb);
-        }
-        /* Log authorities. */
-        for (uint16_t i = 0; i < UINT16_MAX; i++) {
-            JsonBuilder *jb = CreateEveHeader(p, LOG_DIR_FLOW, "dns", NULL);
-            if (unlikely(jb == NULL)) {
-                return TM_ECODE_OK;
-            }
-            EveAddCommonOptions(&dnslog_ctx->cfg, p, f, jb);
-
-            JsonBuilder *answer = rs_dns_log_json_authority_v1(txptr, i,
-                    td->dnslog_ctx->flags);
-            if (answer == NULL) {
-                jb_free(jb);
-                break;
-            }
-            jb_set_object(jb, "dns", answer);
-            jb_free(answer);
-
-            MemBufferReset(td->buffer);
-            OutputJsonBuilderBuffer(jb, td->file_ctx, &td->buffer);
-            jb_free(jb);
-        }
+        jb_open_object(jb, "dns");
+        rs_dns_log_json_answer(txptr, td->dnslog_ctx->flags, jb);
+        jb_close(jb);
+        OutputJsonBuilderBuffer(jb, td->ctx);
+        jb_free(jb);
     }
 
     SCReturnInt(TM_ECODE_OK);
 }
 
+static int JsonDnsLogger(ThreadVars *tv, void *thread_data, const Packet *p, Flow *f, void *alstate,
+        void *txptr, uint64_t tx_id)
+{
+    if (rs_dns_tx_is_request(txptr)) {
+        return JsonDnsLoggerToServer(tv, thread_data, p, f, alstate, txptr, tx_id);
+    } else if (rs_dns_tx_is_response(txptr)) {
+        return JsonDnsLoggerToClient(tv, thread_data, p, f, alstate, txptr, tx_id);
+    }
+    return TM_ECODE_OK;
+}
+
 static TmEcode LogDnsLogThreadInit(ThreadVars *t, const void *initdata, void **data)
 {
     LogDnsLogThread *aft = SCCalloc(1, sizeof(LogDnsLogThread));
@@ -427,15 +387,10 @@ static TmEcode LogDnsLogThreadInit(ThreadVars *t, const void *initdata, void **d
         goto error_exit;
     }
 
-    aft->buffer = MemBufferCreateNew(JSON_OUTPUT_BUFFER_SIZE);
-    if (aft->buffer == NULL) {
-        goto error_exit;
-    }
-
-    /* Use the Ouptut Context (file pointer and mutex) */
-    aft->dnslog_ctx= ((OutputCtx *)initdata)->data;
-    aft->file_ctx = LogFileEnsureExists(aft->dnslog_ctx->file_ctx, t->id);
-    if (!aft->file_ctx) {
+    /* Use the Output Context (file pointer and mutex) */
+    aft->dnslog_ctx = ((OutputCtx *)initdata)->data;
+    aft->ctx = CreateEveThreadCtx(t, aft->dnslog_ctx->eve_ctx);
+    if (!aft->ctx) {
         goto error_exit;
     }
 
@@ -443,9 +398,6 @@ static TmEcode LogDnsLogThreadInit(ThreadVars *t, const void *initdata, void **d
     return TM_ECODE_OK;
 
 error_exit:
-    if (aft->buffer != NULL) {
-        MemBufferFree(aft->buffer);
-    }
     SCFree(aft);
     return TM_ECODE_FAILED;
 }
@@ -456,8 +408,8 @@ static TmEcode LogDnsLogThreadDeinit(ThreadVars *t, void *data)
     if (aft == NULL) {
         return TM_ECODE_OK;
     }
+    FreeEveThreadCtx(aft->ctx);
 
-    MemBufferFree(aft->buffer);
     /* clear memory */
     memset(aft, 0, sizeof(LogDnsLogThread));
 
@@ -485,9 +437,7 @@ static void JsonDnsLogParseConfig(LogDnsFileCtx *dnslog_ctx, ConfNode *conf,
             dnslog_ctx->flags &= ~LOG_QUERIES;
         }
     } else {
-        if (dnslog_ctx->version == DNS_VERSION_2) {
-            dnslog_ctx->flags |= LOG_QUERIES;
-        }
+        dnslog_ctx->flags |= LOG_QUERIES;
     }
 
     const char *response = ConfNodeLookupChildValue(conf, answer_key);
@@ -498,57 +448,47 @@ static void JsonDnsLogParseConfig(LogDnsFileCtx *dnslog_ctx, ConfNode *conf,
             dnslog_ctx->flags &= ~LOG_ANSWERS;
         }
     } else {
-        if (dnslog_ctx->version == DNS_VERSION_2) {
-            dnslog_ctx->flags |= LOG_ANSWERS;
-        }
+        dnslog_ctx->flags |= LOG_ANSWERS;
     }
 
     ConfNode *custom;
     if ((custom = ConfNodeLookupChild(conf, answer_types_key)) != NULL) {
         dnslog_ctx->flags &= ~LOG_ALL_RRTYPES;
         ConfNode *field;
-        TAILQ_FOREACH(field, &custom->head, next)
-        {
-            if (field != NULL)
-            {
-                DnsRRTypes f;
-                for (f = DNS_RRTYPE_A; f < DNS_RRTYPE_MAX; f++)
-                {
-                    if (strcasecmp(dns_rrtype_fields[f].config_rrtype,
-                                   field->val) == 0)
-                    {
-                        dnslog_ctx->flags |= dns_rrtype_fields[f].flags;
-                        break;
-                    }
+        TAILQ_FOREACH (field, &custom->head, next) {
+            DnsRRTypes f;
+            for (f = DNS_RRTYPE_A; f < DNS_RRTYPE_MAX; f++) {
+                if (strcasecmp(dns_rrtype_fields[f].config_rrtype, field->val) == 0) {
+                    dnslog_ctx->flags |= dns_rrtype_fields[f].flags;
+                    break;
                 }
             }
         }
     } else {
-        if (dnslog_ctx->version == DNS_VERSION_2) {
-            dnslog_ctx->flags |= LOG_ALL_RRTYPES;
-        }
+        dnslog_ctx->flags |= LOG_ALL_RRTYPES;
     }
 }
 
-static DnsVersion JsonDnsParseVersion(ConfNode *conf)
+static void JsonDnsCheckVersion(ConfNode *conf)
 {
     if (conf == NULL) {
-        return DNS_VERSION_DEFAULT;
+        return;
     }
 
-    DnsVersion version = DNS_VERSION_DEFAULT;
-    intmax_t config_version;
+    static bool v1_deprecation_warned = false;
     const ConfNode *has_version = ConfNodeLookupChild(conf, "version");
-
     if (has_version != NULL) {
         bool invalid = false;
+        intmax_t config_version;
         if (ConfGetChildValueInt(conf, "version", &config_version)) {
             switch(config_version) {
-                case 1:
-                    version = DNS_VERSION_1;
-                    break;
                 case 2:
-                    version = DNS_VERSION_2;
+                    break;
+                case 1:
+                    if (!v1_deprecation_warned) {
+                        SCLogError("DNS EVE v1 logging has been removed, will use v2");
+                        v1_deprecation_warned = true;
+                    }
                     break;
                 default:
                     invalid = true;
@@ -558,23 +498,9 @@ static DnsVersion JsonDnsParseVersion(ConfNode *conf)
             invalid = true;
         }
         if (invalid) {
-            SCLogWarning(SC_ERR_INVALID_ARGUMENT,
-                    "invalid eve-log dns version option: %s, "
-                    "defaulting to version %u",
-                    has_version->val, version);
+            SCLogWarning("Invalid EVE DNS version \"%s\", will use v2", has_version->val);
         }
-    } else {
-        SCLogConfig("eve-log dns version not set, defaulting to version %u",
-                version);
     }
-
-    if (!v1_deprecation_warned && version == DNS_VERSION_1) {
-        SCLogWarning(SC_WARN_DEPRECATED, "DNS EVE v1 style logs have been "
-                                         "deprecated and will be removed by May 2022");
-        v1_deprecation_warned = true;
-    }
-
-    return version;
 }
 
 static void JsonDnsLogInitFilters(LogDnsFileCtx *dnslog_ctx, ConfNode *conf)
@@ -582,26 +508,21 @@ static void JsonDnsLogInitFilters(LogDnsFileCtx *dnslog_ctx, ConfNode *conf)
     dnslog_ctx->flags = ~0ULL;
 
     if (conf) {
-        if (dnslog_ctx->version == DNS_VERSION_1) {
-            JsonDnsLogParseConfig(dnslog_ctx, conf, "query", "answer", "custom");
-        } else {
-            JsonDnsLogParseConfig(dnslog_ctx, conf, "requests", "responses", "types");
-
-            if (dnslog_ctx->flags & LOG_ANSWERS) {
-                ConfNode *format;
-                if ((format = ConfNodeLookupChild(conf, "formats")) != NULL) {
-                    dnslog_ctx->flags &= ~LOG_FORMAT_ALL;
-                    ConfNode *field;
-                    TAILQ_FOREACH(field, &format->head, next) {
-                        if (strcasecmp(field->val, "detailed") == 0) {
-                            dnslog_ctx->flags |= LOG_FORMAT_DETAILED;
-                        } else if (strcasecmp(field->val, "grouped") == 0) {
-                            dnslog_ctx->flags |= LOG_FORMAT_GROUPED;
-                        }
+        JsonDnsLogParseConfig(dnslog_ctx, conf, "requests", "responses", "types");
+        if (dnslog_ctx->flags & LOG_ANSWERS) {
+            ConfNode *format;
+            if ((format = ConfNodeLookupChild(conf, "formats")) != NULL) {
+                dnslog_ctx->flags &= ~LOG_FORMAT_ALL;
+                ConfNode *field;
+                TAILQ_FOREACH (field, &format->head, next) {
+                    if (strcasecmp(field->val, "detailed") == 0) {
+                        dnslog_ctx->flags |= LOG_FORMAT_DETAILED;
+                    } else if (strcasecmp(field->val, "grouped") == 0) {
+                        dnslog_ctx->flags |= LOG_FORMAT_GROUPED;
                     }
-                } else {
-                    dnslog_ctx->flags |= LOG_FORMAT_ALL;
                 }
+            } else {
+                dnslog_ctx->flags |= LOG_FORMAT_ALL;
             }
         }
     }
@@ -616,7 +537,9 @@ static OutputInitResult JsonDnsLogInitCtxSub(ConfNode *conf, OutputCtx *parent_c
         return result;
     }
 
-    DnsVersion version = JsonDnsParseVersion(conf);
+    /* As only a single version of logging is supported, this exists to warn about
+     * unsupported versions. */
+    JsonDnsCheckVersion(conf);
 
     OutputJsonCtx *ojc = parent_ctx->data;
 
@@ -626,8 +549,7 @@ static OutputInitResult JsonDnsLogInitCtxSub(ConfNode *conf, OutputCtx *parent_c
     }
     memset(dnslog_ctx, 0x00, sizeof(LogDnsFileCtx));
 
-    dnslog_ctx->file_ctx = ojc->file_ctx;
-    dnslog_ctx->cfg = ojc->cfg;
+    dnslog_ctx->eve_ctx = ojc;
 
     OutputCtx *output_ctx = SCCalloc(1, sizeof(OutputCtx));
     if (unlikely(output_ctx == NULL)) {
@@ -638,7 +560,6 @@ static OutputInitResult JsonDnsLogInitCtxSub(ConfNode *conf, OutputCtx *parent_c
     output_ctx->data = dnslog_ctx;
     output_ctx->DeInit = LogDnsLogDeInitCtxSub;
 
-    dnslog_ctx->version = version;
     JsonDnsLogInitFilters(dnslog_ctx, conf);
 
     SCLogDebug("DNS log sub-module initialized");
@@ -655,15 +576,7 @@ static OutputInitResult JsonDnsLogInitCtxSub(ConfNode *conf, OutputCtx *parent_c
 #define MODULE_NAME "JsonDnsLog"
 void JsonDnsLogRegister (void)
 {
-    /* Sub-logger for requests. */
-    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_DNS_TS, "eve-log",
-        MODULE_NAME, "eve-log.dns", JsonDnsLogInitCtxSub, ALPROTO_DNS,
-        JsonDnsLoggerToServer, 0, 1, LogDnsLogThreadInit,
-        LogDnsLogThreadDeinit, NULL);
-
-    /* Sub-logger for replies. */
-    OutputRegisterTxSubModuleWithProgress(LOGGER_JSON_DNS_TC, "eve-log",
-        MODULE_NAME, "eve-log.dns", JsonDnsLogInitCtxSub, ALPROTO_DNS,
-        JsonDnsLoggerToClient, 1, 1, LogDnsLogThreadInit, LogDnsLogThreadDeinit,
-        NULL);
+    OutputRegisterTxSubModule(LOGGER_JSON_TX, "eve-log", MODULE_NAME, "eve-log.dns",
+            JsonDnsLogInitCtxSub, ALPROTO_DNS, JsonDnsLogger, LogDnsLogThreadInit,
+            LogDnsLogThreadDeinit, NULL);
 }
