@@ -26,18 +26,18 @@
  */
 
 #include "suricata-common.h"
-#include "util-error.h"
-#include "util-debug.h"
-#include "util-byte.h"
-#include "util-ip.h"
-#include "util-radix-tree.h"
-#include "util-unittest.h"
-#include "threads.h"
-#include "util-print.h"
-#include "host.h"
-#include "conf.h"
 #include "detect.h"
 #include "reputation.h"
+#include "threads.h"
+#include "conf.h"
+
+#include "util-byte.h"
+#include "util-debug.h"
+#include "util-error.h"
+#include "util-ip.h"
+#include "util-path.h"
+#include "util-print.h"
+#include "util-unittest.h"
 #include "util-validate.h"
 
 /** effective reputation version, atomic as the host
@@ -77,11 +77,11 @@ static void SRepCIDRFreeUserData(void *data)
     return;
 }
 
-static void SRepCIDRAddNetblock(SRepCIDRTree *cidr_ctx, char *ip, int cat, int value)
+static void SRepCIDRAddNetblock(SRepCIDRTree *cidr_ctx, char *ip, int cat, uint8_t value)
 {
     SReputation *user_data = NULL;
     if ((user_data = SCMalloc(sizeof(SReputation))) == NULL) {
-        FatalError(SC_ERR_FATAL, "Error allocating memory. Exiting");
+        FatalError("Error allocating memory. Exiting");
     }
     memset(user_data, 0x00, sizeof(SReputation));
 
@@ -100,8 +100,7 @@ static void SRepCIDRAddNetblock(SRepCIDRTree *cidr_ctx, char *ip, int cat, int v
 
         SCLogDebug("adding ipv6 host %s", ip);
         if (SCRadixAddKeyIPV6String(ip, cidr_ctx->srepIPV6_tree[cat], (void *)user_data) == NULL) {
-            SCLogWarning(SC_ERR_INVALID_VALUE,
-                        "failed to add ipv6 host %s", ip);
+            SCLogWarning("failed to add ipv6 host %s", ip);
         }
 
     } else {
@@ -116,8 +115,7 @@ static void SRepCIDRAddNetblock(SRepCIDRTree *cidr_ctx, char *ip, int cat, int v
 
         SCLogDebug("adding ipv4 host %s", ip);
         if (SCRadixAddKeyIPV4String(ip, cidr_ctx->srepIPV4_tree[cat], (void *)user_data) == NULL) {
-            SCLogWarning(SC_ERR_INVALID_VALUE,
-                        "failed to add ipv4 host %s", ip);
+            SCLogWarning("failed to add ipv4 host %s", ip);
         }
     }
 }
@@ -169,7 +167,7 @@ uint8_t SRepCIDRGetIPRepDst(SRepCIDRTree *cidr_ctx, Packet *p, uint8_t cat, uint
 }
 
 /** \brief Increment effective reputation version after
- *         a rule/reputatio reload is complete. */
+ *         a rule/reputation reload is complete. */
 void SRepReloadComplete(void)
 {
     (void) SC_ATOMIC_ADD(srep_eversion, 1);
@@ -309,11 +307,11 @@ static int SRepSplitLine(SRepCIDRTree *cidr_ctx, char *line, Address *ip, uint8_
     if (strcmp(ptrs[0], "ip") == 0)
         return 1;
 
-    int c, v;
-    if (StringParseI32RangeCheck(&c, 10, 0, (const char *)ptrs[1], 0, SREP_MAX_CATS - 1) < 0)
+    uint8_t c, v;
+    if (StringParseU8RangeCheck(&c, 10, 0, (const char *)ptrs[1], 0, SREP_MAX_CATS - 1) < 0)
         return -1;
 
-    if (StringParseI32RangeCheck(&v, 10, 0, (const char *)ptrs[2], 0, SREP_MAX_VAL) < 0)
+    if (StringParseU8RangeCheck(&v, 10, 0, (const char *)ptrs[2], 0, SREP_MAX_VAL) < 0)
         return -1;
 
     if (strchr(ptrs[0], '/') != NULL) {
@@ -355,7 +353,7 @@ static int SRepLoadCatFile(const char *filename)
     FILE *fp = fopen(filename, "r");
 
     if (fp == NULL) {
-        SCLogError(SC_ERR_OPENING_RULE_FILE, "opening ip rep file %s: %s", filename, strerror(errno));
+        SCLogError("opening ip rep file %s: %s", filename, strerror(errno));
         return -1;
     }
 
@@ -401,7 +399,7 @@ int SRepLoadCatFileFromFD(FILE *fp)
         if (SRepCatSplitLine(line, &cat, shortname, sizeof(shortname)) == 0) {
             strlcpy(srep_cat_table[cat], shortname, SREP_SHORTNAME_LEN);
         } else {
-            SCLogError(SC_ERR_NO_REPUTATION, "bad line \"%s\"", line);
+            SCLogError("bad line \"%s\"", line);
         }
     }
 
@@ -421,7 +419,7 @@ static int SRepLoadFile(SRepCIDRTree *cidr_ctx, char *filename)
     FILE *fp = fopen(filename, "r");
 
     if (fp == NULL) {
-        SCLogError(SC_ERR_OPENING_RULE_FILE, "opening ip rep file %s: %s", filename, strerror(errno));
+        SCLogError("opening ip rep file %s: %s", filename, strerror(errno));
         return -1;
     }
 
@@ -464,7 +462,7 @@ int SRepLoadFileFromFD(SRepCIDRTree *cidr_ctx, FILE *fp)
         uint8_t cat = 0, value = 0;
         int r = SRepSplitLine(cidr_ctx, line, &a, &cat, &value);
         if (r < 0) {
-            SCLogError(SC_ERR_NO_REPUTATION, "bad line \"%s\"", line);
+            SCLogError("bad line \"%s\"", line);
         } else if (r == 0) {
             if (a.family == AF_INET) {
                 char ipstr[16];
@@ -478,7 +476,7 @@ int SRepLoadFileFromFD(SRepCIDRTree *cidr_ctx, FILE *fp)
 
             Host *h = HostGetHostFromHash(&a);
             if (h == NULL) {
-                SCLogError(SC_ERR_NO_REPUTATION, "failed to get a host, increase host.memcap");
+                SCLogError("failed to get a host, increase host.memcap");
                 break;
             } else {
                 //SCLogInfo("host %p", h);
@@ -611,21 +609,22 @@ int SRepInit(DetectEngineCtx *de_ctx)
     }
 
     if (files == NULL) {
-        SCLogError(SC_ERR_NO_REPUTATION, "\"reputation-files\" not set");
+        SCLogError("\"reputation-files\" not set");
         return -1;
     }
 
     if (init) {
         if (filename == NULL) {
-            SCLogError(SC_ERR_NO_REPUTATION, "\"reputation-categories-file\" not set");
+            SCLogError("\"reputation-categories-file\" not set");
             return -1;
         }
 
         /* init even if we have reputation files, so that when we
          * have a live reload, we have inited the cats */
         if (SRepLoadCatFile(filename) < 0) {
-            SCLogError(SC_ERR_NO_REPUTATION, "failed to load reputation "
-                    "categories file %s", filename);
+            SCLogError("failed to load reputation "
+                       "categories file %s",
+                    filename);
             return -1;
         }
     }
@@ -642,7 +641,7 @@ int SRepInit(DetectEngineCtx *de_ctx)
 
                 int r = SRepLoadFile(cidr_ctx, sfile);
                 if (r < 0){
-                    if (de_ctx->failure_fatal == 1) {
+                    if (de_ctx->failure_fatal) {
                         exit(EXIT_FAILURE);
                     }
                 }
