@@ -31,6 +31,7 @@
  */
 
 #include "suricata-common.h"
+#include "suricata.h"
 #include "tm-threads.h"
 #include "conf.h"
 #include "runmodes.h"
@@ -40,11 +41,11 @@
 #include "detect-engine-mpm.h"
 
 #include "alert-fastlog.h"
-#include "alert-prelude.h"
 #include "alert-debuglog.h"
 
 #include "flow-bypass.h"
 
+#include "util-conf.h"
 #include "util-debug.h"
 #include "util-time.h"
 #include "util-cpu.h"
@@ -54,31 +55,117 @@
 #include "util-ioctl.h"
 #include "util-ebpf.h"
 #include "util-byte.h"
-#include "util-bpf.h"
 
 #include "source-af-packet.h"
+#include "util-bpf.h"
 
-extern intmax_t max_pending_packets;
+extern uint16_t max_pending_packets;
 
 const char *RunModeAFPGetDefaultMode(void)
 {
     return "workers";
 }
 
+static int AFPRunModeIsIPS(void)
+{
+    int nlive = LiveGetDeviceCount();
+    int ldev;
+    ConfNode *if_root;
+    ConfNode *if_default = NULL;
+    ConfNode *af_packet_node;
+    int has_ips = 0;
+    int has_ids = 0;
+
+    /* Find initial node */
+    af_packet_node = ConfGetNode("af-packet");
+    if (af_packet_node == NULL) {
+        return 0;
+    }
+
+    if_default = ConfNodeLookupKeyValue(af_packet_node, "interface", "default");
+
+    for (ldev = 0; ldev < nlive; ldev++) {
+        const char *live_dev = LiveGetDeviceName(ldev);
+        if (live_dev == NULL) {
+            SCLogError("Problem with config file");
+            return 0;
+        }
+        const char *copymodestr = NULL;
+        if_root = ConfFindDeviceConfig(af_packet_node, live_dev);
+
+        if (if_root == NULL) {
+            if (if_default == NULL) {
+                SCLogError("Problem with config file");
+                return 0;
+            }
+            if_root = if_default;
+        }
+
+        if (ConfGetChildValueWithDefault(if_root, if_default, "copy-mode", &copymodestr) == 1) {
+            if (strcmp(copymodestr, "ips") == 0) {
+                has_ips = 1;
+            } else {
+                has_ids = 1;
+            }
+        } else {
+            has_ids = 1;
+        }
+    }
+
+    if (has_ids && has_ips) {
+        SCLogWarning("AF_PACKET using both IPS and TAP/IDS mode, this will not "
+                     "be allowed in Suricata 8 due to undefined behavior. See ticket #5588.");
+        for (ldev = 0; ldev < nlive; ldev++) {
+            const char *live_dev = LiveGetDeviceName(ldev);
+            if (live_dev == NULL) {
+                SCLogError("Problem with config file");
+                return 0;
+            }
+            if_root = ConfNodeLookupKeyValue(af_packet_node, "interface", live_dev);
+            const char *copymodestr = NULL;
+
+            if (if_root == NULL) {
+                if (if_default == NULL) {
+                    SCLogError("Problem with config file");
+                    return 0;
+                }
+                if_root = if_default;
+            }
+
+            if (!((ConfGetChildValueWithDefault(if_root, if_default, "copy-mode", &copymodestr) ==
+                          1) &&
+                        (strcmp(copymodestr, "ips") == 0))) {
+                SCLogError("AF_PACKET IPS mode used and interface '%s' is in IDS or TAP mode. "
+                           "Sniffing '%s' but expect bad result as stream-inline is activated.",
+                        live_dev, live_dev);
+            }
+        }
+    }
+
+    return has_ips;
+}
+
+static void AFPRunModeEnableIPS(void)
+{
+    if (AFPRunModeIsIPS()) {
+        SCLogInfo("Setting IPS mode");
+        EngineModeSetIPS();
+    }
+}
+
 void RunModeIdsAFPRegister(void)
 {
-    RunModeRegisterNewRunMode(RUNMODE_AFP_DEV, "single",
-                              "Single threaded af-packet mode",
-                              RunModeIdsAFPSingle);
+    RunModeRegisterNewRunMode(RUNMODE_AFP_DEV, "single", "Single threaded af-packet mode",
+            RunModeIdsAFPSingle, AFPRunModeEnableIPS);
     RunModeRegisterNewRunMode(RUNMODE_AFP_DEV, "workers",
-                              "Workers af-packet mode, each thread does all"
-                              " tasks from acquisition to logging",
-                              RunModeIdsAFPWorkers);
+            "Workers af-packet mode, each thread does all"
+            " tasks from acquisition to logging",
+            RunModeIdsAFPWorkers, AFPRunModeEnableIPS);
     RunModeRegisterNewRunMode(RUNMODE_AFP_DEV, "autofp",
-                              "Multi socket AF_PACKET mode.  Packets from "
-                              "each flow are assigned to a single detect "
-                              "thread.",
-                              RunModeIdsAFPAutoFp);
+            "Multi socket AF_PACKET mode.  Packets from "
+            "each flow are assigned to a single detect "
+            "thread.",
+            RunModeIdsAFPAutoFp, AFPRunModeEnableIPS);
     return;
 }
 
@@ -143,7 +230,7 @@ static void *ParseAFPConfig(const char *iface)
     aconf->promisc = 1;
     aconf->checksum_mode = CHECKSUM_VALIDATION_KERNEL;
     aconf->DerefFunc = AFPDerefConfig;
-    aconf->flags = AFP_RING_MODE;
+    aconf->flags = 0;
     aconf->bpf_filter = NULL;
     aconf->ebpf_lb_file = NULL;
     aconf->ebpf_lb_fd = -1;
@@ -160,7 +247,7 @@ static void *ParseAFPConfig(const char *iface)
     /* Find initial node */
     af_packet_node = ConfGetNode("af-packet");
     if (af_packet_node == NULL) {
-        SCLogInfo("unable to find af-packet config using default values");
+        SCLogInfo("%s: unable to find af-packet config using default values", iface);
         goto finalize;
     }
 
@@ -168,9 +255,9 @@ static void *ParseAFPConfig(const char *iface)
     if_default = ConfFindDeviceConfig(af_packet_node, "default");
 
     if (if_root == NULL && if_default == NULL) {
-        SCLogInfo("unable to find af-packet config for "
+        SCLogInfo("%s: unable to find af-packet config for "
                   "interface \"%s\" or \"default\", using default values",
-                  iface);
+                iface, iface);
         goto finalize;
     }
 
@@ -190,8 +277,9 @@ static void *ParseAFPConfig(const char *iface)
                 aconf->threads = 0;
             } else {
                 if (StringParseInt32(&aconf->threads, 10, 0, (const char *)threadsstr) < 0) {
-                    SCLogWarning(SC_ERR_INVALID_VALUE, "Invalid number of "
-                                 "threads, resetting to default");
+                    SCLogWarning("%s: invalid number of "
+                                 "threads, resetting to default",
+                            iface);
                     aconf->threads = 0;
                 }
             }
@@ -206,82 +294,66 @@ static void *ParseAFPConfig(const char *iface)
 
     if (ConfGetChildValueBoolWithDefault(if_root, if_default, "use-mmap", (int *)&boolval) == 1) {
         if (!boolval) {
-            SCLogConfig("Disabling mmaped capture on iface %s",
-                    aconf->iface);
-            aconf->flags &= ~(AFP_RING_MODE|AFP_TPACKET_V3);
+            SCLogWarning(
+                    "%s: \"use-mmap\" option is obsolete: mmap is always enabled", aconf->iface);
         }
     }
 
-    if (aconf->flags & AFP_RING_MODE) {
-        (void)ConfGetChildValueBoolWithDefault(if_root, if_default,
-                                               "mmap-locked", (int *)&boolval);
-        if (boolval) {
-            SCLogConfig("Enabling locked memory for mmap on iface %s",
-                    aconf->iface);
-            aconf->flags |= AFP_MMAP_LOCKED;
-        }
+    (void)ConfGetChildValueBoolWithDefault(if_root, if_default, "mmap-locked", (int *)&boolval);
+    if (boolval) {
+        SCLogConfig("%s: enabling locked memory for mmap", aconf->iface);
+        aconf->flags |= AFP_MMAP_LOCKED;
+    }
 
-        if (ConfGetChildValueBoolWithDefault(if_root, if_default,
-                                             "tpacket-v3", (int *)&boolval) == 1)
-        {
-            if (boolval) {
-                if (strcasecmp(RunmodeGetActive(), "workers") == 0) {
+    if (ConfGetChildValueBoolWithDefault(if_root, if_default, "tpacket-v3", (int *)&boolval) == 1) {
+        if (boolval) {
+            if (strcasecmp(RunmodeGetActive(), "workers") == 0) {
 #ifdef HAVE_TPACKET_V3
-                    SCLogConfig("Enabling tpacket v3 capture on iface %s",
-                            aconf->iface);
-                    aconf->flags |= AFP_TPACKET_V3;
+                SCLogConfig("%s: enabling tpacket v3", aconf->iface);
+                aconf->flags |= AFP_TPACKET_V3;
 #else
-                    SCLogNotice("System too old for tpacket v3 switching to v2");
-                    aconf->flags &= ~AFP_TPACKET_V3;
+                SCLogWarning("%s: system too old for tpacket v3 switching to v2", iface);
+                aconf->flags &= ~AFP_TPACKET_V3;
 #endif
-                } else {
-                    SCLogWarning(SC_ERR_RUNMODE,
-                            "tpacket v3 is only implemented for 'workers' runmode."
-                            " Switching to tpacket v2.");
-                    aconf->flags &= ~AFP_TPACKET_V3;
-                }
             } else {
+                SCLogWarning("%s: tpacket v3 is only implemented for 'workers' runmode."
+                             " Switching to tpacket v2.",
+                        iface);
                 aconf->flags &= ~AFP_TPACKET_V3;
             }
-        }
-
-        (void)ConfGetChildValueBoolWithDefault(if_root, if_default,
-                                               "use-emergency-flush", (int *)&boolval);
-        if (boolval) {
-            SCLogConfig("Enabling ring emergency flush on iface %s",
-                    aconf->iface);
-            aconf->flags |= AFP_EMERGENCY_MODE;
+        } else {
+            aconf->flags &= ~AFP_TPACKET_V3;
         }
     }
 
-    aconf->copy_mode = AFP_COPY_MODE_NONE;
+    (void)ConfGetChildValueBoolWithDefault(
+            if_root, if_default, "use-emergency-flush", (int *)&boolval);
+    if (boolval) {
+        SCLogConfig("%s: using emergency ring flush", aconf->iface);
+        aconf->flags |= AFP_EMERGENCY_MODE;
+    }
+
     if (ConfGetChildValueWithDefault(if_root, if_default, "copy-mode", &copymodestr) == 1) {
         if (aconf->out_iface == NULL) {
-            SCLogInfo("Copy mode activated but no destination"
-                      " iface. Disabling feature");
-        } else if (!(aconf->flags & AFP_RING_MODE)) {
-            SCLogInfo("Copy mode activated but use-mmap "
-                      "set to no. Disabling feature");
+            SCLogWarning("%s: copy mode activated but no destination"
+                         " iface. Disabling feature",
+                    iface);
         } else if (strlen(copymodestr) <= 0) {
             aconf->out_iface = NULL;
         } else if (strcmp(copymodestr, "ips") == 0) {
-            SCLogInfo("AF_PACKET IPS mode activated %s->%s",
-                    iface,
-                    aconf->out_iface);
+            SCLogInfo("%s: AF_PACKET IPS mode activated %s->%s", iface, iface, aconf->out_iface);
             aconf->copy_mode = AFP_COPY_MODE_IPS;
             if (aconf->flags & AFP_TPACKET_V3) {
-                SCLogWarning(SC_ERR_RUNMODE, "Using tpacket_v3 in IPS mode will result in high latency");
+                SCLogWarning("%s: using tpacket_v3 in IPS mode will result in high latency", iface);
             }
         } else if (strcmp(copymodestr, "tap") == 0) {
-            SCLogInfo("AF_PACKET TAP mode activated %s->%s",
-                    iface,
-                    aconf->out_iface);
+            SCLogInfo("%s: AF_PACKET TAP mode activated %s->%s", iface, iface, aconf->out_iface);
             aconf->copy_mode = AFP_COPY_MODE_TAP;
             if (aconf->flags & AFP_TPACKET_V3) {
-                SCLogWarning(SC_ERR_RUNMODE, "Using tpacket_v3 in TAP mode will result in high latency");
+                SCLogWarning("%s: using tpacket_v3 in TAP mode will result in high latency", iface);
             }
         } else {
-            SCLogInfo("Invalid mode (not in tap, ips)");
+            SCLogWarning("Invalid 'copy-mode' (not in tap, ips)");
         }
     }
 
@@ -289,7 +361,7 @@ static void *ParseAFPConfig(const char *iface)
         aconf->cluster_id = (uint16_t)(cluster_id_auto++);
     } else {
         if (StringParseUint16(&aconf->cluster_id, 10, 0, (const char *)tmpclusterid) < 0) {
-            SCLogWarning(SC_ERR_INVALID_VALUE, "Invalid cluster_id, resetting to 0");
+            SCLogWarning("%s: invalid cluster_id, resetting to 0", iface);
             aconf->cluster_id = 0;
         }
         SCLogDebug("Going to use cluster-id %" PRIu16, aconf->cluster_id);
@@ -300,66 +372,58 @@ static void *ParseAFPConfig(const char *iface)
         aconf->cluster_type = PACKET_FANOUT_HASH | PACKET_FANOUT_FLAG_DEFRAG;
         cluster_type = PACKET_FANOUT_HASH;
     } else if (strcmp(tmpctype, "cluster_round_robin") == 0) {
-        SCLogConfig("Using round-robin cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogConfig("%s: using round-robin cluster mode for AF_PACKET", aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_LB;
         cluster_type = PACKET_FANOUT_LB;
-    } else if (strcmp(tmpctype, "cluster_flow") == 0) {
+    } else if (strcmp(tmpctype, "cluster_flow") == 0 || strcmp(tmpctype, "cluster_rollover") == 0) {
+        if (strcmp(tmpctype, "cluster_rollover") == 0) {
+            SCLogWarning("%s: cluster_rollover deprecated; using \"cluster_flow\" instead. See "
+                         "ticket #6128",
+                    aconf->iface);
+        }
         /* In hash mode, we also ask for defragmentation needed to
          * compute the hash */
         uint16_t defrag = 0;
         int conf_val = 0;
-        SCLogConfig("Using flow cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogConfig("%s: using flow cluster mode for AF_PACKET", aconf->iface);
         ConfGetChildValueBoolWithDefault(if_root, if_default, "defrag", &conf_val);
         if (conf_val) {
-            SCLogConfig("Using defrag kernel functionality for AF_PACKET (iface %s)",
-                    aconf->iface);
+            SCLogConfig("%s: using defrag kernel functionality for AF_PACKET", aconf->iface);
             defrag = PACKET_FANOUT_FLAG_DEFRAG;
         }
         aconf->cluster_type = PACKET_FANOUT_HASH | defrag;
         cluster_type = PACKET_FANOUT_HASH;
     } else if (strcmp(tmpctype, "cluster_cpu") == 0) {
-        SCLogConfig("Using cpu cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogConfig("%s: using cpu cluster mode for AF_PACKET", aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_CPU;
         cluster_type = PACKET_FANOUT_CPU;
     } else if (strcmp(tmpctype, "cluster_qm") == 0) {
-        SCLogConfig("Using queue based cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogConfig("%s: using queue based cluster mode for AF_PACKET", aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_QM;
         cluster_type = PACKET_FANOUT_QM;
     } else if (strcmp(tmpctype, "cluster_random") == 0) {
-        SCLogConfig("Using random based cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogConfig("%s: using random based cluster mode for AF_PACKET", aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_RND;
         cluster_type = PACKET_FANOUT_RND;
-    } else if (strcmp(tmpctype, "cluster_rollover") == 0) {
-        SCLogConfig("Using rollover based cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
-        SCLogWarning(SC_WARN_UNCOMMON, "Rollover mode is causing severe flow "
-                                       "tracking issues, use it at your own risk.");
-        aconf->cluster_type = PACKET_FANOUT_ROLLOVER;
-        cluster_type = PACKET_FANOUT_ROLLOVER;
 #ifdef HAVE_PACKET_EBPF
     } else if (strcmp(tmpctype, "cluster_ebpf") == 0) {
-        SCLogInfo("Using ebpf based cluster mode for AF_PACKET (iface %s)",
-                aconf->iface);
+        SCLogInfo("%s: using ebpf based cluster mode for AF_PACKET", aconf->iface);
         aconf->cluster_type = PACKET_FANOUT_EBPF;
         cluster_type = PACKET_FANOUT_EBPF;
 #endif
     } else {
-        SCLogWarning(SC_ERR_INVALID_CLUSTER_TYPE,"invalid cluster-type %s",tmpctype);
+        SCLogWarning("invalid cluster-type %s", tmpctype);
     }
 
     int conf_val = 0;
     ConfGetChildValueBoolWithDefault(if_root, if_default, "rollover", &conf_val);
     if (conf_val) {
-        SCLogConfig("Using rollover kernel functionality for AF_PACKET (iface %s)",
+        SCLogConfig("%s: Rollover requested for AF_PACKET but ignored -- see ticket #6128.",
                 aconf->iface);
-        aconf->cluster_type |= PACKET_FANOUT_FLAG_ROLLOVER;
-        SCLogWarning(SC_WARN_UNCOMMON, "Rollover option is causing severe flow "
-                                       "tracking issues, use it at your own risk.");
+        SCLogWarning("%s: rollover option has been deprecated and will be ignored as it can cause "
+                     "severe flow "
+                     "tracking issues; see ticket #6128.",
+                iface);
     }
 
     ConfSetBPFFilter(if_root, if_default, iface, &aconf->bpf_filter);
@@ -368,8 +432,7 @@ static void *ParseAFPConfig(const char *iface)
         aconf->ebpf_lb_file = NULL;
     } else {
 #ifdef HAVE_PACKET_EBPF
-        SCLogConfig("af-packet will use '%s' as eBPF load balancing file",
-                  ebpf_file);
+        SCLogConfig("%s: af-packet will use '%s' as eBPF load balancing file", iface, ebpf_file);
         aconf->ebpf_lb_file = ebpf_file;
         aconf->ebpf_t_config.flags |= EBPF_SOCKET_FILTER;
 #endif
@@ -379,8 +442,7 @@ static void *ParseAFPConfig(const char *iface)
     boolval = false;
     if (ConfGetChildValueBoolWithDefault(if_root, if_default, "pinned-maps", (int *)&boolval) == 1) {
         if (boolval) {
-            SCLogConfig("Using pinned maps on iface %s",
-                        aconf->iface);
+            SCLogConfig("%s: using pinned maps", aconf->iface);
             aconf->ebpf_t_config.flags |= EBPF_PINNED_MAPS;
         }
         const char *pinned_maps_name = NULL;
@@ -403,12 +465,12 @@ static void *ParseAFPConfig(const char *iface)
                                &aconf->ebpf_lb_fd,
                                &aconf->ebpf_t_config);
         if (ret != 0) {
-            SCLogWarning(SC_ERR_INVALID_VALUE, "Error when loading eBPF lb file");
+            SCLogWarning("%s: failed to load eBPF lb file", iface);
         }
     }
 #else
     if (aconf->ebpf_lb_file) {
-        SCLogError(SC_ERR_UNIMPLEMENTED, "eBPF support is not build-in");
+        SCLogError("%s: eBPF support is not built-in", iface);
     }
 #endif
 
@@ -416,8 +478,7 @@ static void *ParseAFPConfig(const char *iface)
         aconf->ebpf_filter_file = NULL;
     } else {
 #ifdef HAVE_PACKET_EBPF
-        SCLogConfig("af-packet will use '%s' as eBPF filter file",
-                  ebpf_file);
+        SCLogConfig("%s: af-packet will use '%s' as eBPF filter file", iface, ebpf_file);
         aconf->ebpf_filter_file = ebpf_file;
         aconf->ebpf_t_config.mode = AFP_MODE_EBPF_BYPASS;
         aconf->ebpf_t_config.flags |= EBPF_SOCKET_FILTER;
@@ -425,12 +486,11 @@ static void *ParseAFPConfig(const char *iface)
         ConfGetChildValueBoolWithDefault(if_root, if_default, "bypass", &conf_val);
         if (conf_val) {
 #ifdef HAVE_PACKET_EBPF
-            SCLogConfig("Using bypass kernel functionality for AF_PACKET (iface %s)",
-                    aconf->iface);
+            SCLogConfig("%s: using bypass kernel functionality for AF_PACKET", aconf->iface);
             aconf->flags |= AFP_BYPASS;
             BypassedFlowManagerRegisterUpdateFunc(EBPFUpdateFlow, NULL);
 #else
-            SCLogError(SC_ERR_UNIMPLEMENTED, "Bypass set but eBPF support is not built-in");
+            SCLogError("%s: bypass set but eBPF support is not built-in", iface);
 #endif
         }
     }
@@ -442,11 +502,10 @@ static void *ParseAFPConfig(const char *iface)
                                &aconf->ebpf_filter_fd,
                                &aconf->ebpf_t_config);
         if (ret != 0) {
-            SCLogWarning(SC_ERR_INVALID_VALUE,
-                         "Error when loading eBPF filter file");
+            SCLogWarning("%s: failed to load eBPF filter file", iface);
         }
 #else
-        SCLogError(SC_ERR_UNIMPLEMENTED, "eBPF support is not build-in");
+        SCLogError("%s: eBPF support is not built-in", iface);
 #endif
     }
 
@@ -459,15 +518,14 @@ static void *ParseAFPConfig(const char *iface)
         aconf->xdp_filter_file = ebpf_file;
         ConfGetChildValueBoolWithDefault(if_root, if_default, "bypass", &conf_val);
         if (conf_val) {
-            SCLogConfig("Using bypass kernel functionality for AF_PACKET (iface %s)",
-                    aconf->iface);
+            SCLogConfig("%s: using bypass kernel functionality for AF_PACKET", aconf->iface);
             aconf->flags |= AFP_XDPBYPASS;
             /* if maps are pinned we need to read them at start */
             if (aconf->ebpf_t_config.flags & EBPF_PINNED_MAPS) {
                 RunModeEnablesBypassManager();
                 struct ebpf_timeout_config *ebt = SCCalloc(1, sizeof(struct ebpf_timeout_config));
                 if (ebt == NULL) {
-                    SCLogError(SC_ERR_MEM_ALLOC, "Flow bypass alloc error");
+                    SCLogError("%s: flow bypass alloc error", iface);
                 } else {
                     memcpy(ebt, &(aconf->ebpf_t_config), sizeof(struct ebpf_timeout_config));
                     BypassedFlowManagerRegisterCheckFunc(NULL,
@@ -478,7 +536,7 @@ static void *ParseAFPConfig(const char *iface)
             BypassedFlowManagerRegisterUpdateFunc(EBPFUpdateFlow, NULL);
         }
 #else
-        SCLogWarning(SC_ERR_UNIMPLEMENTED, "XDP filter set but XDP support is not built-in");
+        SCLogWarning("%s: XDP filter set but XDP support is not built-in", iface);
 #endif
 #ifdef HAVE_PACKET_XDP
         const char *xdp_mode;
@@ -493,16 +551,14 @@ static void *ParseAFPConfig(const char *iface)
                 aconf->xdp_mode = XDP_FLAGS_HW_MODE;
                 aconf->ebpf_t_config.flags |= EBPF_XDP_HW_MODE;
             } else {
-                SCLogWarning(SC_ERR_INVALID_VALUE,
-                             "Invalid xdp-mode value: '%s'", xdp_mode);
+                SCLogWarning("Invalid xdp-mode value: '%s'", xdp_mode);
             }
         }
 
         boolval = true;
         if (ConfGetChildValueBoolWithDefault(if_root, if_default, "use-percpu-hash", (int *)&boolval) == 1) {
             if (boolval == false) {
-                SCLogConfig("Not using percpu hash on iface %s",
-                        aconf->iface);
+                SCLogConfig("%s: not using percpu hash", aconf->iface);
                 aconf->ebpf_t_config.cpus_count = 1;
             }
         }
@@ -517,27 +573,24 @@ static void *ParseAFPConfig(const char *iface)
                                &aconf->ebpf_t_config);
         switch (ret) {
             case 1:
-                SCLogInfo("Loaded pinned maps from sysfs");
+                SCLogInfo("%s: loaded pinned maps from sysfs", iface);
                 break;
             case -1:
-                SCLogWarning(SC_ERR_INVALID_VALUE,
-                             "Error when loading XDP filter file");
+                SCLogWarning("%s: failed to load XDP filter file", iface);
                 break;
             case 0:
                 ret = EBPFSetupXDP(aconf->iface, aconf->xdp_filter_fd, aconf->xdp_mode);
                 if (ret != 0) {
-                    SCLogWarning(SC_ERR_INVALID_VALUE,
-                            "Error when setting up XDP");
+                    SCLogWarning("%s: failed to set up XDP", iface);
                 } else {
                     /* Try to get the xdp-cpu-redirect key */
                     const char *cpuset;
                     if (ConfGetChildValueWithDefault(if_root, if_default,
                                 "xdp-cpu-redirect", &cpuset) == 1) {
-                        SCLogConfig("Setting up CPU map XDP");
+                        SCLogConfig("%s: Setting up CPU map XDP", iface);
                         ConfNode *node = ConfGetChildWithDefault(if_root, if_default, "xdp-cpu-redirect");
                         if (node == NULL) {
-                            SCLogError(SC_ERR_INVALID_VALUE,
-                                       "Previously found node has disappeared");
+                            SCLogError("Previously found node has disappeared");
                         } else {
                             EBPFBuildCPUSet(node, aconf->iface);
                         }
@@ -552,7 +605,7 @@ static void *ParseAFPConfig(const char *iface)
                 }
         }
 #else
-        SCLogError(SC_ERR_UNIMPLEMENTED, "XDP support is not built-in");
+        SCLogError("%s: XDP support is not built-in", iface);
 #endif
     }
 
@@ -567,7 +620,8 @@ static void *ParseAFPConfig(const char *iface)
 
     if ((ConfGetChildValueIntWithDefault(if_root, if_default, "block-size", &value)) == 1) {
         if (value % getpagesize()) {
-            SCLogError(SC_ERR_INVALID_VALUE, "Block-size must be a multiple of pagesize.");
+            SCLogWarning("%s: block-size %" PRIuMAX " must be a multiple of pagesize (%u).", iface,
+                    value, getpagesize());
         } else {
             aconf->block_size = value;
         }
@@ -581,8 +635,7 @@ static void *ParseAFPConfig(const char *iface)
 
     (void)ConfGetChildValueBoolWithDefault(if_root, if_default, "disable-promisc", (int *)&boolval);
     if (boolval) {
-        SCLogConfig("Disabling promiscuous mode on iface %s",
-                aconf->iface);
+        SCLogConfig("%s: disabling promiscuous mode", aconf->iface);
         aconf->promisc = 0;
     }
 
@@ -596,7 +649,7 @@ static void *ParseAFPConfig(const char *iface)
         } else if (strcmp(tmpctype, "kernel") == 0) {
             aconf->checksum_mode = CHECKSUM_VALIDATION_KERNEL;
         } else {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid value for checksum-checks for %s", aconf->iface);
+            SCLogWarning("%s: invalid value for checksum-checks", aconf->iface);
         }
     }
 
@@ -607,8 +660,9 @@ finalize:
     if (aconf->threads != 1) {
         if (AFPIsFanoutSupported(aconf->cluster_id) == 0) {
             if (aconf->threads != 0) {
-                SCLogNotice("fanout not supported on this system, falling "
-                        "back to 1 capture thread");
+                SCLogNotice("%s: fanout not supported on this system, falling "
+                            "back to 1 capture thread",
+                        iface);
             }
             aconf->threads = 1;
         }
@@ -619,20 +673,21 @@ finalize:
         /* for cluster_flow use core count */
         if (cluster_type == PACKET_FANOUT_HASH) {
             aconf->threads = (int)UtilCpuGetNumProcessorsOnline();
-            SCLogPerf("%u cores, so using %u threads", aconf->threads, aconf->threads);
+            SCLogPerf("%s: cluster_flow: %u cores, using %u threads", iface, aconf->threads,
+                    aconf->threads);
 
-        /* for cluster_qm use RSS queue count */
+            /* for cluster_qm use RSS queue count */
         } else if (cluster_type == PACKET_FANOUT_QM) {
             int rss_queues = GetIfaceRSSQueuesNum(iface);
             if (rss_queues > 0) {
                 aconf->threads = rss_queues;
-                SCLogPerf("%d RSS queues, so using %u threads", rss_queues, aconf->threads);
+                SCLogPerf("%s: cluster_qm: %d RSS queues, using %u threads", iface, rss_queues,
+                        aconf->threads);
             }
         }
 
         if (aconf->threads) {
-            SCLogPerf("Using %d AF_PACKET threads for interface %s",
-                    aconf->threads, iface);
+            SCLogDebug("using %d threads for interface %s", aconf->threads, iface);
         }
     }
     if (aconf->threads <= 0) {
@@ -644,8 +699,9 @@ finalize:
     if (aconf->ring_size != 0) {
         if (aconf->ring_size * aconf->threads < max_pending_packets) {
             aconf->ring_size = max_pending_packets / aconf->threads + 1;
-            SCLogWarning(SC_ERR_AFP_CREATE, "Inefficient setup: ring-size < max_pending_packets. "
-                         "Resetting to decent value %d.", aconf->ring_size);
+            SCLogWarning("%s: inefficient setup: ring-size < max_pending_packets. "
+                         "Resetting to decent value %d.",
+                    iface, aconf->ring_size);
             /* We want at least that max_pending_packets packets can be handled by the
              * interface. This is generous if we have multiple interfaces listening. */
         }
@@ -662,8 +718,9 @@ finalize:
             /* af-packet can handle csum offloading */
             if (LiveGetOffload() == 0) {
                 if (GetIfaceOffloading(iface, 0, 1) == 1) {
-                    SCLogWarning(SC_ERR_AFP_CREATE,
-                            "Using AF_PACKET with offloading activated leads to capture problems");
+                    SCLogWarning(
+                            "%s: using AF_PACKET with offloads enabled leads to capture problems",
+                            iface);
                 }
             } else {
                 DisableIfaceOffloading(LiveGetDevice(iface), 0, 1);
@@ -674,23 +731,11 @@ finalize:
             break;
     }
 
-    if (active_runmode && !strcmp("workers", active_runmode)) {
-        aconf->flags |= AFP_ZERO_COPY;
-    } else {
+    if (active_runmode == NULL || strcmp("workers", active_runmode) != 0) {
         /* If we are using copy mode we need a lock */
         aconf->flags |= AFP_SOCK_PROTECT;
+        aconf->flags |= AFP_NEED_PEER;
     }
-
-    /* If we are in RING mode, then we can use ZERO copy
-     * by using the data release mechanism */
-    if (aconf->flags & AFP_RING_MODE) {
-        aconf->flags |= AFP_ZERO_COPY;
-    }
-
-    if (aconf->flags & AFP_ZERO_COPY) {
-        SCLogConfig("%s: enabling zero copy mode by using data release call", iface);
-    }
-
     return aconf;
 }
 
@@ -700,88 +745,7 @@ static int AFPConfigGeThreadsCount(void *conf)
     return afp->threads;
 }
 
-int AFPRunModeIsIPS(void)
-{
-    int nlive = LiveGetDeviceNameCount();
-    int ldev;
-    ConfNode *if_root;
-    ConfNode *if_default = NULL;
-    ConfNode *af_packet_node;
-    int has_ips = 0;
-    int has_ids = 0;
-
-    /* Find initial node */
-    af_packet_node = ConfGetNode("af-packet");
-    if (af_packet_node == NULL) {
-        return 0;
-    }
-
-    if_default = ConfNodeLookupKeyValue(af_packet_node, "interface", "default");
-
-    for (ldev = 0; ldev < nlive; ldev++) {
-        const char *live_dev = LiveGetDeviceNameName(ldev);
-        if (live_dev == NULL) {
-            SCLogError(SC_ERR_INVALID_VALUE, "Problem with config file");
-            return 0;
-        }
-        const char *copymodestr = NULL;
-        if_root = ConfFindDeviceConfig(af_packet_node, live_dev);
-
-        if (if_root == NULL) {
-            if (if_default == NULL) {
-                SCLogError(SC_ERR_INVALID_VALUE, "Problem with config file");
-                return 0;
-            }
-            if_root = if_default;
-        }
-
-        if (ConfGetChildValueWithDefault(if_root, if_default, "copy-mode", &copymodestr) == 1) {
-            if (strcmp(copymodestr, "ips") == 0) {
-                has_ips = 1;
-            } else {
-                has_ids = 1;
-            }
-        } else {
-            has_ids = 1;
-        }
-    }
-
-    if (has_ids && has_ips) {
-        SCLogWarning(SC_ERR_INVALID_ARGUMENT,
-                "AF_PACKET using both IPS and TAP/IDS mode, this will not "
-                "be allowed in Suricata 8 due to undefined behavior. See ticket #5588.");
-        for (ldev = 0; ldev < nlive; ldev++) {
-            const char *live_dev = LiveGetDeviceNameName(ldev);
-            if (live_dev == NULL) {
-                SCLogError(SC_ERR_INVALID_VALUE, "Problem with config file");
-                return 0;
-            }
-            if_root = ConfNodeLookupKeyValue(af_packet_node, "interface", live_dev);
-            const char *copymodestr = NULL;
-
-            if (if_root == NULL) {
-                if (if_default == NULL) {
-                    SCLogError(SC_ERR_INVALID_VALUE, "Problem with config file");
-                    return 0;
-                }
-                if_root = if_default;
-            }
-
-            if (! ((ConfGetChildValueWithDefault(if_root, if_default, "copy-mode", &copymodestr) == 1) &&
-                        (strcmp(copymodestr, "ips") == 0))) {
-                SCLogError(SC_ERR_INVALID_ARGUMENT,
-                        "AF_PACKET IPS mode used and interface '%s' is in IDS or TAP mode. "
-                        "Sniffing '%s' but expect bad result as stream-inline is activated.",
-                        live_dev, live_dev);
-            }
-        }
-    }
-
-    return has_ips;
-}
-
-#endif
-
+#endif /* HAVE_AF_PACKET */
 
 int RunModeIdsAFPAutoFp(void)
 {
@@ -801,21 +765,18 @@ int RunModeIdsAFPAutoFp(void)
     SCLogDebug("live_dev %s", live_dev);
 
     if (AFPPeersListInit() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Unable to init peers list.");
+        FatalError("Unable to init peers list.");
     }
 
-    ret = RunModeSetLiveCaptureAutoFp(ParseAFPConfig,
-                              AFPConfigGeThreadsCount,
-                              "ReceiveAFP",
-                              "DecodeAFP", thread_name_autofp,
-                              live_dev);
+    ret = RunModeSetLiveCaptureAutoFp(ParseAFPConfig, AFPConfigGeThreadsCount, "ReceiveAFP",
+            "DecodeAFP", thread_name_autofp, live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     /* In IPS mode each threads must have a peer */
     if (AFPPeersListCheck() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Some IPS capture threads did not peer.");
+        FatalError("Some IPS capture threads did not peer.");
     }
 
     SCLogDebug("RunModeIdsAFPAutoFp initialised");
@@ -840,7 +801,7 @@ int RunModeIdsAFPSingle(void)
     (void)ConfGet("af-packet.live-interface", &live_dev);
 
     if (AFPPeersListInit() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Unable to init peers list.");
+        FatalError("Unable to init peers list.");
     }
 
     ret = RunModeSetLiveCaptureSingle(ParseAFPConfig,
@@ -849,12 +810,12 @@ int RunModeIdsAFPSingle(void)
                                     "DecodeAFP", thread_name_single,
                                     live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     /* In IPS mode each threads must have a peer */
     if (AFPPeersListCheck() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Some IPS capture threads did not peer.");
+        FatalError("Some IPS capture threads did not peer.");
     }
 
     SCLogDebug("RunModeIdsAFPSingle initialised");
@@ -882,21 +843,18 @@ int RunModeIdsAFPWorkers(void)
     (void)ConfGet("af-packet.live-interface", &live_dev);
 
     if (AFPPeersListInit() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Unable to init peers list.");
+        FatalError("Unable to init peers list.");
     }
 
-    ret = RunModeSetLiveCaptureWorkers(ParseAFPConfig,
-                                    AFPConfigGeThreadsCount,
-                                    "ReceiveAFP",
-                                    "DecodeAFP", thread_name_workers,
-                                    live_dev);
+    ret = RunModeSetLiveCaptureWorkers(ParseAFPConfig, AFPConfigGeThreadsCount, "ReceiveAFP",
+            "DecodeAFP", thread_name_workers, live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     /* In IPS mode each threads must have a peer */
     if (AFPPeersListCheck() != TM_ECODE_OK) {
-        FatalError(SC_ERR_FATAL, "Some IPS capture threads did not peer.");
+        FatalError("Some IPS capture threads did not peer.");
     }
 
     SCLogDebug("RunModeIdsAFPWorkers initialised");
