@@ -90,7 +90,7 @@ void RunModeNapatechRegister(void)
     RunModeRegisterNewRunMode(RUNMODE_NAPATECH, "workers",
             "Workers Napatech mode, each thread does all"
             " tasks from acquisition to logging",
-            RunModeNapatechWorkers);
+            RunModeNapatechWorkers, NULL);
     return;
 #endif
 }
@@ -120,8 +120,8 @@ static int NapatechRegisterDeviceStreams(void)
      * automatically creates streams.  Therefore, these two options are mutually exclusive.
      */
     if (use_all_streams && auto_config) {
-        FatalError(SC_ERR_FATAL,
-                   "napatech.auto-config cannot be used in configuration file at the same time as napatech.use-all-streams.");
+        FatalError("napatech.auto-config cannot be used in configuration file at the same time as "
+                   "napatech.use-all-streams.");
     }
 
     /* to use hardware_bypass we need to configure the streams to be consistent.
@@ -129,8 +129,7 @@ static int NapatechRegisterDeviceStreams(void)
      * option.
      */
     if (use_hw_bypass && auto_config == 0) {
-        FatalError(SC_ERR_FATAL,
-                   "napatech auto-config must be enabled when using napatech.use_hw_bypass.");
+        FatalError("napatech auto-config must be enabled when using napatech.use_hw_bypass.");
     }
 
     /* Get the stream ID's either from the conf or by querying Napatech */
@@ -143,20 +142,16 @@ static int NapatechRegisterDeviceStreams(void)
     for (uint16_t inst = 0; inst < stream_cnt; ++inst) {
         char *plive_dev_buf = SCCalloc(1, 9);
         if (unlikely(plive_dev_buf == NULL)) {
-                    FatalError(SC_ERR_FATAL,
-                               "Failed to allocate memory for NAPATECH stream counter.");
+            FatalError("Failed to allocate memory for NAPATECH stream counter.");
         }
         snprintf(plive_dev_buf, 9, "nt%d", stream_config[inst].stream_id);
 
         if (auto_config) {
             if (stream_config[inst].is_active) {
-                SCLogError(SC_ERR_NAPATECH_STREAMS_REGISTER_FAILED,
-                        "Registering Napatech device: %s - active stream found.",
-                        plive_dev_buf);
-                SCLogError(SC_ERR_NAPATECH_STREAMS_REGISTER_FAILED,
+                SCLogError("Registering Napatech device: %s - active stream found.", plive_dev_buf);
+                SCLogError(
                         "run /opt/napatech3/bin/ntpl -e \"delete=all\" to delete existing stream");
-                        FatalError(SC_ERR_FATAL,
-                                   "or disable auto-config in the conf file before running.");
+                FatalError("or disable auto-config in the conf file before running.");
             }
         } else {
             SCLogInfo("Registering Napatech device: %s - active stream%sfound.",
@@ -170,7 +165,7 @@ static int NapatechRegisterDeviceStreams(void)
         last_stream = stream_config[inst].stream_id;
     }
 
-    /* Napatech stats come from a separate thread.  This will surpress
+    /* Napatech stats come from a separate thread.  This will suppress
      * the counters when suricata exits.
      */
     LiveDeviceHasNoStats();
@@ -182,21 +177,19 @@ static void *NapatechConfigParser(const char *device)
     /* Expect device to be of the form nt%d where %d is the stream id to use */
     int dev_len = strlen(device);
     if (dev_len < 3 || dev_len > 5) {
-        SCLogError(SC_ERR_NAPATECH_PARSE_CONFIG,
-                "Could not parse config for device: %s - invalid length", device);
+        SCLogError("Could not parse config for device: %s - invalid length", device);
         return NULL;
     }
 
     struct NapatechStreamDevConf *conf = SCCalloc(1, sizeof (struct NapatechStreamDevConf));
     if (unlikely(conf == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC,
-                "Failed to allocate memory for NAPATECH device name.");
+        SCLogError("Failed to allocate memory for NAPATECH device name.");
         return NULL;
     }
 
     /* device+2 is a pointer to the beginning of the stream id after the constant nt portion */
     if (StringParseUint16(&conf->stream_id, 10, 0, device + 2) < 0) {
-        SCLogError(SC_ERR_INVALID_VALUE, "Invalid value for stream_id: %s", device + 2);
+        SCLogError("Invalid value for stream_id: %s", device + 2);
         SCFree(conf);
         return NULL;
     }
@@ -207,8 +200,7 @@ static void *NapatechConfigParser(const char *device)
     if (ConfGetInt("napatech.hba", &conf->hba) == 0) {
         conf->hba = -1;
     } else {
-        SCLogWarning(SC_WARN_COMPATIBILITY,
-                "Napatech Host Buffer Allocation (hba) will be deprecated in Suricata v7.0.");
+        SCLogWarning("Napatech Host Buffer Allocation (hba) will be deprecated in Suricata v7.0.");
     }
     return (void *) conf;
 }
@@ -229,21 +221,19 @@ static int NapatechInit(int runmode)
 
     /* Initialize the API and check version compatibility */
     if ((status = NT_Init(NTAPI_VERSION)) != NT_SUCCESS) {
-        NAPATECH_ERROR(SC_ERR_NAPATECH_INIT_FAILED, status);
+        NAPATECH_ERROR(status);
         exit(EXIT_FAILURE);
     }
 
     status = NapatechRegisterDeviceStreams();
     if (status < 0 || num_configured_streams <= 0) {
-                    FatalError(SC_ERR_FATAL,
-                               "Unable to find existing Napatech Streams");
+        FatalError("Unable to find existing Napatech Streams");
     }
 
     struct NapatechStreamDevConf *conf =
                             SCCalloc(1, sizeof (struct NapatechStreamDevConf));
     if (unlikely(conf == NULL)) {
-        FatalError(SC_ERR_FATAL,
-                   "Failed to allocate memory for NAPATECH device.");
+        FatalError("Failed to allocate memory for NAPATECH device.");
     }
 
     if ((ConfGetInt("napatech.hba", &conf->hba) != 0) && (conf->hba > 0)) {
@@ -255,12 +245,12 @@ static int NapatechInit(int runmode)
         if (NapatechVerifyBypassSupport()) {
             SCLogInfo("Napatech Hardware Bypass is supported and enabled.");
         } else {
-                    FatalError(SC_ERR_FATAL,
-                               "Napatech Hardware Bypass requested in conf but is not supported by the hardware.");
+            FatalError("Napatech Hardware Bypass requested in conf but is not supported by the "
+                       "hardware.");
         }
 #else
-                FatalError(SC_ERR_FATAL,
-                           "Napatech Hardware Bypass requested in conf but is not enabled by the software.");
+        FatalError(
+                "Napatech Hardware Bypass requested in conf but is not enabled by the software.");
 #endif
     } else {
         SCLogInfo("Hardware Bypass is disabled in the conf file.");
@@ -271,17 +261,15 @@ static int NapatechInit(int runmode)
 
     switch (runmode) {
         case NT_RUNMODE_WORKERS:
-            status = RunModeSetLiveCaptureWorkers(NapatechConfigParser,
-                    NapatechGetThreadsCount,
-                    "NapatechStream", "NapatechDecode",
-                    thread_name_workers, NULL);
+            status = RunModeSetLiveCaptureWorkers(NapatechConfigParser, NapatechGetThreadsCount,
+                    "NapatechStream", "NapatechDecode", thread_name_workers, NULL);
             break;
         default:
             status = -1;
     }
 
     if (status != 0) {
-        FatalError(SC_ERR_FATAL, "Runmode start failed");
+        FatalError("Runmode start failed");
     }
     return 0;
 }
