@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 Open Information Security Foundation
+/* Copyright (C) 2014-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -33,26 +33,9 @@
 
 #include "source-nflog.h"
 
-const char *RunModeIdsNflogGetDefaultMode(void)
-{
-    return "autofp";
-}
-
-void RunModeIdsNflogRegister(void)
-{
-    RunModeRegisterNewRunMode(RUNMODE_NFLOG, "autofp",
-                              "Multi threaded nflog mode",
-                              RunModeIdsNflogAutoFp);
-    RunModeRegisterNewRunMode(RUNMODE_NFLOG, "single",
-                              "Single threaded nflog mode",
-                              RunModeIdsNflogSingle);
-    RunModeRegisterNewRunMode(RUNMODE_NFLOG, "workers",
-                              "Workers nflog mode",
-                              RunModeIdsNflogWorkers);
-    return;
-}
-
 #ifdef HAVE_NFLOG
+#include "util-time.h"
+
 static void NflogDerefConfig(void *data)
 {
     NflogGroupConfig *nflogconf = (NflogGroupConfig *)data;
@@ -102,7 +85,7 @@ static void *ParseNflogConfig(const char *group)
     strlcpy(nflogconf->numgroup, group, sizeof(nflogconf->numgroup));
 
     if (ParseSizeStringU16(group, &nflogconf->group) < 0) {
-        FatalError(SC_ERR_FATAL, "NFLOG's group number invalid.");
+        FatalError("NFLOG's group number invalid.");
     }
 
     boolval = ConfGetChildValueIntWithDefault(group_root, group_default,
@@ -111,7 +94,7 @@ static void *ParseNflogConfig(const char *group)
     if (boolval)
         nflogconf->nlbufsiz = bufsize;
     else {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid buffer-size value");
+        SCLogError("Invalid buffer-size value");
         SCFree(nflogconf);
         return NULL;
     }
@@ -122,14 +105,14 @@ static void *ParseNflogConfig(const char *group)
     if (boolval)
         nflogconf->nlbufsiz_max = bufsize_max;
     else {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid max-size value");
+        SCLogError("Invalid max-size value");
         SCFree(nflogconf);
         return NULL;
     }
 
     if (nflogconf->nlbufsiz > nflogconf->nlbufsiz_max) {
-        SCLogWarning(SC_ERR_INVALID_ARGUMENT, "buffer-size value larger "
-                "than max-size value, adjusting buffer-size");
+        SCLogWarning("buffer-size value larger "
+                     "than max-size value, adjusting buffer-size");
         nflogconf->nlbufsiz = nflogconf->nlbufsiz_max;
     }
 
@@ -139,7 +122,7 @@ static void *ParseNflogConfig(const char *group)
     if (boolval)
         nflogconf->qthreshold = qthreshold;
     else {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid qthreshold value");
+        SCLogError("Invalid qthreshold value");
         SCFree(nflogconf);
         return NULL;
     }
@@ -150,7 +133,7 @@ static void *ParseNflogConfig(const char *group)
     if (boolval)
         nflogconf->qtimeout = qtimeout;
     else {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid qtimeout value");
+        SCLogError("Invalid qtimeout value");
         SCFree(nflogconf);
         return NULL;
     }
@@ -165,25 +148,18 @@ static int NflogConfigGeThreadsCount(void *conf)
 }
 #endif
 
-int RunModeIdsNflogAutoFp(void)
+static int RunModeIdsNflogAutoFp(void)
 {
     SCEnter();
 
 #ifdef HAVE_NFLOG
-    int ret = 0;
-    char *live_dev = NULL;
-
     RunModeInitialize();
     TimeModeSetLive();
 
-    ret = RunModeSetLiveCaptureAutoFp(ParseNflogConfig,
-                                      NflogConfigGeThreadsCount,
-                                      "ReceiveNFLOG",
-                                      "DecodeNFLOG",
-                                      thread_name_autofp,
-                                      live_dev);
+    int ret = RunModeSetLiveCaptureAutoFp(ParseNflogConfig, NflogConfigGeThreadsCount,
+            "ReceiveNFLOG", "DecodeNFLOG", thread_name_autofp, NULL);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     SCLogInfo("RunModeIdsNflogAutoFp initialised");
@@ -192,25 +168,18 @@ int RunModeIdsNflogAutoFp(void)
     SCReturnInt(0);
 }
 
-int RunModeIdsNflogSingle(void)
+static int RunModeIdsNflogSingle(void)
 {
     SCEnter();
 
 #ifdef HAVE_NFLOG
-    int ret = 0;
-    char *live_dev = NULL;
-
     RunModeInitialize();
     TimeModeSetLive();
 
-    ret = RunModeSetLiveCaptureSingle(ParseNflogConfig,
-                                      NflogConfigGeThreadsCount,
-                                      "ReceiveNFLOG",
-                                      "DecodeNFLOG",
-                                      thread_name_single,
-                                      live_dev);
+    int ret = RunModeSetLiveCaptureSingle(ParseNflogConfig, NflogConfigGeThreadsCount,
+            "ReceiveNFLOG", "DecodeNFLOG", thread_name_single, NULL);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     SCLogInfo("RunModeIdsNflogSingle initialised");
@@ -219,25 +188,18 @@ int RunModeIdsNflogSingle(void)
     SCReturnInt(0);
 }
 
-int RunModeIdsNflogWorkers(void)
+static int RunModeIdsNflogWorkers(void)
 {
     SCEnter();
 
 #ifdef HAVE_NFLOG
-    int ret = 0;
-    char *live_dev = NULL;
-
     RunModeInitialize();
     TimeModeSetLive();
 
-    ret = RunModeSetLiveCaptureWorkers(ParseNflogConfig,
-                                       NflogConfigGeThreadsCount,
-                                       "ReceiveNFLOG",
-                                       "DecodeNFLOG",
-                                       thread_name_workers,
-                                       live_dev);
+    int ret = RunModeSetLiveCaptureWorkers(ParseNflogConfig, NflogConfigGeThreadsCount,
+            "ReceiveNFLOG", "DecodeNFLOG", thread_name_workers, NULL);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
     SCLogInfo("RunModeIdsNflogWorkers initialised");
@@ -245,3 +207,19 @@ int RunModeIdsNflogWorkers(void)
 
     SCReturnInt(0);
 }
+
+const char *RunModeIdsNflogGetDefaultMode(void)
+{
+    return "autofp";
+}
+
+void RunModeIdsNflogRegister(void)
+{
+    RunModeRegisterNewRunMode(
+            RUNMODE_NFLOG, "autofp", "Multi threaded nflog mode", RunModeIdsNflogAutoFp, NULL);
+    RunModeRegisterNewRunMode(
+            RUNMODE_NFLOG, "single", "Single threaded nflog mode", RunModeIdsNflogSingle, NULL);
+    RunModeRegisterNewRunMode(
+            RUNMODE_NFLOG, "workers", "Workers nflog mode", RunModeIdsNflogWorkers, NULL);
+    return;
+}
