@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -16,20 +16,16 @@
  */
 
 #include "suricata-common.h"
-#include "tm-threads.h"
-#include "conf.h"
-#include "runmodes.h"
 #include "runmode-pcap.h"
+#include "runmodes.h"
 #include "output.h"
-#include "log-httplog.h"
 
+#include "util-conf.h"
 #include "util-debug.h"
 #include "util-time.h"
 #include "util-cpu.h"
-#include "util-affinity.h"
 #include "util-device.h"
 #include "util-runmodes.h"
-#include "util-atomic.h"
 #include "util-misc.h"
 #include "util-byte.h"
 
@@ -42,20 +38,19 @@ int RunModeIdsPcapWorkers(void);
 
 void RunModeIdsPcapRegister(void)
 {
-    RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "single",
-                              "Single threaded pcap live mode",
-                              RunModeIdsPcapSingle);
+    RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "single", "Single threaded pcap live mode",
+            RunModeIdsPcapSingle, NULL);
     RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "autofp",
-                              "Multi threaded pcap live mode.  Packets from "
-                              "each flow are assigned to a single detect thread, "
-                              "unlike \"pcap_live_auto\" where packets from "
-                              "the same flow can be processed by any detect "
-                              "thread",
-                              RunModeIdsPcapAutoFp);
+            "Multi threaded pcap live mode.  Packets from "
+            "each flow are assigned to a single detect thread, "
+            "unlike \"pcap_live_auto\" where packets from "
+            "the same flow can be processed by any detect "
+            "thread",
+            RunModeIdsPcapAutoFp, NULL);
     RunModeRegisterNewRunMode(RUNMODE_PCAP_DEV, "workers",
-                              "Workers pcap live mode, each thread does all"
-                              " tasks from acquisition to logging",
-                              RunModeIdsPcapWorkers);
+            "Workers pcap live mode, each thread does all"
+            " tasks from acquisition to logging",
+            RunModeIdsPcapWorkers, NULL);
 
     return;
 }
@@ -101,9 +96,10 @@ static void *ParsePcapConfig(const char *iface)
             SCLogInfo("Pcap will use %d buffer size", (int)value);
             aconf->buffer_size = value;
         } else {
-            SCLogWarning(SC_ERR_INVALID_ARGUMENT, "pcap.buffer-size "
-                    "value of %"PRIiMAX" is invalid. Valid range is "
-                    "0-2147483647", value);
+            SCLogWarning("pcap.buffer-size "
+                         "value of %" PRIiMAX " is invalid. Valid range is "
+                         "0-2147483647",
+                    value);
         }
     }
 
@@ -146,8 +142,9 @@ static void *ParsePcapConfig(const char *iface)
     } else {
         if (threadsstr != NULL) {
             if (StringParseInt32(&aconf->threads, 10, 0, (const char *)threadsstr) < 0) {
-                SCLogWarning(SC_ERR_INVALID_VALUE, "Invalid value for "
-                             "pcap.threads: %s, resetting to 1", threadsstr);
+                SCLogWarning("Invalid value for "
+                             "pcap.threads: %s, resetting to 1",
+                        threadsstr);
                 aconf->threads = 1;
             }
         }
@@ -165,9 +162,7 @@ static void *ParsePcapConfig(const char *iface)
             uint64_t bsize = 0;
 
             if (ParseSizeStringU64(s_limit, &bsize) < 0) {
-                SCLogError(SC_ERR_INVALID_ARGUMENT,
-                    "Failed to parse pcap buffer size: %s",
-                    s_limit);
+                SCLogError("Failed to parse pcap buffer size: %s", s_limit);
             } else {
                 /* the string 2gb returns 2147483648 which is 1 to high
                  * for a int. */
@@ -175,8 +170,8 @@ static void *ParsePcapConfig(const char *iface)
                     bsize = (uint64_t)INT_MAX;
 
                 if (bsize > INT_MAX) {
-                    SCLogError(SC_ERR_INVALID_ARGUMENT,
-                            "Failed to set pcap buffer size: 2gb max. %"PRIu64" > %d", bsize, INT_MAX);
+                    SCLogError("Failed to set pcap buffer size: 2gb max. %" PRIu64 " > %d", bsize,
+                            INT_MAX);
                 } else {
                     aconf->buffer_size = (int)bsize;
                 }
@@ -203,7 +198,7 @@ static void *ParsePcapConfig(const char *iface)
         } else if (ConfValIsFalse(tmpctype)) {
             aconf->checksum_mode = CHECKSUM_VALIDATION_DISABLE;
         } else {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Invalid value for checksum-checks for %s", aconf->iface);
+            SCLogError("Invalid value for checksum-checks for %s", aconf->iface);
         }
     }
 
@@ -251,10 +246,10 @@ int RunModeIdsPcapSingle(void)
                                     "DecodePcap", thread_name_single,
                                     live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Runmode start failed");
+        FatalError("Runmode start failed");
     }
 
-    SCLogInfo("RunModeIdsPcapSingle initialised");
+    SCLogDebug("RunModeIdsPcapSingle initialised");
 
     SCReturnInt(0);
 }
@@ -285,16 +280,13 @@ int RunModeIdsPcapAutoFp(void)
 
     (void) ConfGet("pcap.single-pcap-dev", &live_dev);
 
-    ret = RunModeSetLiveCaptureAutoFp(ParsePcapConfig,
-                              PcapConfigGeThreadsCount,
-                              "ReceivePcap",
-                              "DecodePcap", thread_name_autofp,
-                              live_dev);
+    ret = RunModeSetLiveCaptureAutoFp(ParsePcapConfig, PcapConfigGeThreadsCount, "ReceivePcap",
+            "DecodePcap", thread_name_autofp, live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Runmode start failed");
+        FatalError("Runmode start failed");
     }
 
-    SCLogInfo("RunModeIdsPcapAutoFp initialised");
+    SCLogDebug("RunModeIdsPcapAutoFp initialised");
 
     SCReturnInt(0);
 }
@@ -316,16 +308,13 @@ int RunModeIdsPcapWorkers(void)
 
     (void) ConfGet("pcap.single-pcap-dev", &live_dev);
 
-    ret = RunModeSetLiveCaptureWorkers(ParsePcapConfig,
-                                    PcapConfigGeThreadsCount,
-                                    "ReceivePcap",
-                                    "DecodePcap", thread_name_workers,
-                                    live_dev);
+    ret = RunModeSetLiveCaptureWorkers(ParsePcapConfig, PcapConfigGeThreadsCount, "ReceivePcap",
+            "DecodePcap", thread_name_workers, live_dev);
     if (ret != 0) {
-        FatalError(SC_ERR_FATAL, "Unable to start runmode");
+        FatalError("Unable to start runmode");
     }
 
-    SCLogInfo("RunModeIdsPcapWorkers initialised");
+    SCLogDebug("RunModeIdsPcapWorkers initialised");
 
     SCReturnInt(0);
 }
