@@ -36,6 +36,21 @@
 #include "conf.h"
 #include "queue.h"
 #include "runmodes.h"
+#include "runmode-af-packet.h"
+#include "runmode-af-xdp.h"
+#include "runmode-dpdk.h"
+#include "runmode-erf-dag.h"
+#include "runmode-erf-file.h"
+#include "runmode-ipfw.h"
+#include "runmode-napatech.h"
+#include "runmode-netmap.h"
+#include "runmode-nflog.h"
+#include "runmode-nfq.h"
+#include "runmode-pcap.h"
+#include "runmode-pcap-file.h"
+#include "runmode-pfring.h"
+#include "runmode-unix-socket.h"
+#include "runmode-windivert.h"
 #include "util-unittest.h"
 #include "util-misc.h"
 #include "util-plugin.h"
@@ -43,7 +58,6 @@
 #include "output.h"
 
 #include "alert-fastlog.h"
-#include "alert-prelude.h"
 #include "alert-debuglog.h"
 
 #include "log-httplog.h"
@@ -84,6 +98,7 @@ typedef struct RunMode_ {
     const char *description;
     /* runmode function */
     int (*RunModeFunc)(void);
+    void (*RunModeIsIPSEnabled)(void);
 } RunMode;
 
 typedef struct RunModes_ {
@@ -107,7 +122,7 @@ static TAILQ_HEAD(, OutputFreeList_) output_free_list =
 
 /**
  * \internal
- * \brief Translate a runmode mode to a printale string.
+ * \brief Translate a runmode mode to a printable string.
  *
  * \param runmode Runmode to be converted into a printable string.
  *
@@ -144,6 +159,8 @@ static const char *RunModeTranslateModeToName(int runmode)
             return "UNITTEST";
         case RUNMODE_AFP_DEV:
             return "AF_PACKET_DEV";
+        case RUNMODE_AFXDP_DEV:
+            return "AF_XDP_DEV";
         case RUNMODE_NETMAP:
 #ifdef HAVE_NETMAP
             return "NETMAP";
@@ -158,8 +175,15 @@ static const char *RunModeTranslateModeToName(int runmode)
 #else
             return "WINDIVERT(DISABLED)";
 #endif
+        case RUNMODE_DPDK:
+#ifdef HAVE_DPDK
+            return "DPDK";
+#else
+            return "DPDK(DISABLED)";
+#endif
+
         default:
-            FatalError(SC_ERR_UNKNOWN_RUN_MODE, "Unknown runtime mode. Aborting");
+            FatalError("Unknown runtime mode. Aborting");
     }
 }
 
@@ -168,8 +192,8 @@ static const char *RunModeTranslateModeToName(int runmode)
  * \brief Dispatcher function for runmodes.  Calls the required runmode function
  *        based on runmode + runmode_custom_id.
  *
- * \param runmode            The runmode type.
- * \param runmode_customd_id The runmode custom id.
+ * \param runmode           The runmode type.
+ * \param runmode_custom_id The runmode custom id.
  */
 static RunMode *RunModeGetCustomMode(enum RunModes runmode, const char *custom_mode)
 {
@@ -225,10 +249,12 @@ void RunModeRegisterRunModes(void)
     RunModeErfDagRegister();
     RunModeNapatechRegister();
     RunModeIdsAFPRegister();
+    RunModeIdsAFXDPRegister();
     RunModeIdsNetmapRegister();
     RunModeIdsNflogRegister();
     RunModeUnixSocketRegister();
     RunModeIpsWinDivertRegister();
+    RunModeDpdkRegister();
 #ifdef UNITTESTS
     UtRunModeRegister();
 #endif
@@ -279,24 +305,18 @@ void RunModeListRunmodes(void)
     return;
 }
 
-/**
- */
-void RunModeDispatch(int runmode, const char *custom_mode,
-    const char *capture_plugin_name, const char *capture_plugin_args)
+static const char *RunModeGetConfOrDefault(int capture_mode, const char *capture_plugin_name)
 {
-    char *local_custom_mode = NULL;
-
-    if (custom_mode == NULL) {
-        const char *val = NULL;
-        if (ConfGet("runmode", &val) != 1) {
-            custom_mode = NULL;
-        } else {
-            custom_mode = val;
-        }
+    const char *custom_mode = NULL;
+    const char *val = NULL;
+    if (ConfGet("runmode", &val) != 1) {
+        custom_mode = NULL;
+    } else {
+        custom_mode = val;
     }
 
-    if (custom_mode == NULL || strcmp(custom_mode, "auto") == 0) {
-        switch (runmode) {
+    if ((custom_mode == NULL) || (strcmp(custom_mode, "auto") == 0)) {
+        switch (capture_mode) {
             case RUNMODE_PCAP_DEV:
                 custom_mode = RunModeIdsGetDefaultMode();
                 break;
@@ -312,8 +332,7 @@ void RunModeDispatch(int runmode, const char *custom_mode,
 #ifdef HAVE_PLUGINS
                 SCCapturePlugin *plugin = SCPluginFindCaptureByName(capture_plugin_name);
                 if (plugin == NULL) {
-                    FatalError(SC_ERR_PLUGIN, "No capture plugin found with name %s",
-                            capture_plugin_name);
+                    FatalError("No capture plugin found with name %s", capture_plugin_name);
                 }
                 custom_mode = (const char *)plugin->GetDefaultMode();
 #endif
@@ -337,6 +356,9 @@ void RunModeDispatch(int runmode, const char *custom_mode,
             case RUNMODE_AFP_DEV:
                 custom_mode = RunModeAFPGetDefaultMode();
                 break;
+            case RUNMODE_AFXDP_DEV:
+                custom_mode = RunModeAFXDPGetDefaultMode();
+                break;
             case RUNMODE_NETMAP:
                 custom_mode = RunModeNetmapGetDefaultMode();
                 break;
@@ -350,29 +372,64 @@ void RunModeDispatch(int runmode, const char *custom_mode,
             case RUNMODE_WINDIVERT:
                 custom_mode = RunModeIpsWinDivertGetDefaultMode();
                 break;
+#endif
+#ifdef HAVE_DPDK
+            case RUNMODE_DPDK:
+                custom_mode = RunModeDpdkGetDefaultMode();
+                break;
 #endif
             default:
-                FatalError(SC_ERR_FATAL, "Unknown runtime mode. Aborting");
+                return NULL;
         }
-    } else { /* if (custom_mode == NULL) */
-        /* Add compability with old 'worker' name */
+    } else {
+        /* Add compatibility with old 'worker' name */
         if (!strcmp("worker", custom_mode)) {
-            SCLogWarning(SC_ERR_RUNMODE, "'worker' mode have been renamed "
+            SCLogWarning("'worker' mode have been renamed "
                          "to 'workers', please modify your setup.");
-            local_custom_mode = SCStrdup("workers");
-            if (unlikely(local_custom_mode == NULL)) {
-                FatalError(SC_ERR_FATAL, "Unable to dup custom mode");
-            }
-            custom_mode = local_custom_mode;
+            custom_mode = "workers";
         }
     }
 
+    return custom_mode;
+}
+
+void RunModeEngineIsIPS(int capture_mode, const char *runmode, const char *capture_plugin_name)
+{
+    if (runmode == NULL) {
+        runmode = RunModeGetConfOrDefault(capture_mode, capture_plugin_name);
+        if (runmode == NULL) // non-standard runmode
+            return;
+    }
+
+    RunMode *mode = RunModeGetCustomMode(capture_mode, runmode);
+    if (mode == NULL) {
+        return;
+    }
+
+    if (mode->RunModeIsIPSEnabled != NULL) {
+        mode->RunModeIsIPSEnabled();
+    }
+}
+
+/**
+ */
+void RunModeDispatch(int runmode, const char *custom_mode, const char *capture_plugin_name,
+        const char *capture_plugin_args)
+{
+    char *local_custom_mode = NULL;
+
+    if (custom_mode == NULL) {
+        custom_mode = RunModeGetConfOrDefault(runmode, capture_plugin_name);
+        if (custom_mode == NULL)
+            FatalError("Unknown runtime mode. Aborting");
+    }
+
     RunMode *mode = RunModeGetCustomMode(runmode, custom_mode);
     if (mode == NULL) {
-        SCLogError(SC_ERR_RUNMODE, "The custom type \"%s\" doesn't exist "
+        SCLogError("The custom type \"%s\" doesn't exist "
                    "for this runmode type \"%s\".  Please use --list-runmodes to "
                    "see available custom types for this runmode",
-                   custom_mode, RunModeTranslateModeToName(runmode));
+                custom_mode, RunModeTranslateModeToName(runmode));
         exit(EXIT_FAILURE);
     }
 
@@ -382,7 +439,7 @@ void RunModeDispatch(int runmode, const char *custom_mode,
     }
     active_runmode = SCStrdup(custom_mode);
     if (unlikely(active_runmode == NULL)) {
-        FatalError(SC_ERR_FATAL, "Unable to dup active mode");
+        FatalError("Unable to dup active mode");
     }
 
     if (strcasecmp(active_runmode, "autofp") == 0) {
@@ -431,14 +488,13 @@ int RunModeNeedsBypassManager(void)
  * \param description Description for this runmode.
  * \param RunModeFunc The function to be run for this runmode.
  */
-void RunModeRegisterNewRunMode(enum RunModes runmode,
-                               const char *name,
-                               const char *description,
-                               int (*RunModeFunc)(void))
+void RunModeRegisterNewRunMode(enum RunModes runmode, const char *name, const char *description,
+        int (*RunModeFunc)(void), void (*RunModeIsIPSEnabled)(void))
 {
     if (RunModeGetCustomMode(runmode, name) != NULL) {
-        FatalError(SC_ERR_RUNMODE, "runmode '%s' has already "
-                   "been registered. Please use an unique name.", name);
+        FatalError("runmode '%s' has already "
+                   "been registered. Please use an unique name.",
+                name);
     }
 
     void *ptmp = SCRealloc(runmodes[runmode].runmodes,
@@ -457,13 +513,14 @@ void RunModeRegisterNewRunMode(enum RunModes runmode,
     mode->runmode = runmode;
     mode->name = SCStrdup(name);
     if (unlikely(mode->name == NULL)) {
-        FatalError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
+        FatalError("Failed to allocate string");
     }
     mode->description = SCStrdup(description);
     if (unlikely(mode->description == NULL)) {
-        FatalError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
+        FatalError("Failed to allocate string");
     }
     mode->RunModeFunc = RunModeFunc;
+    mode->RunModeIsIPSEnabled = RunModeIsIPSEnabled;
 
     return;
 }
@@ -478,13 +535,11 @@ static void RunOutputFreeList(void)
 {
     OutputFreeList *output;
     while ((output = TAILQ_FIRST(&output_free_list))) {
-        SCLogDebug("output %s %p %p", output->output_module->name, output,
-            output->output_ctx);
+        TAILQ_REMOVE(&output_free_list, output, entries);
 
+        SCLogDebug("output %s %p %p", output->output_module->name, output, output->output_ctx);
         if (output->output_ctx != NULL && output->output_ctx->DeInit != NULL)
             output->output_ctx->DeInit(output->output_ctx);
-
-        TAILQ_REMOVE(&output_free_list, output, entries);
         SCFree(output);
     }
 }
@@ -603,7 +658,7 @@ static void SetupOutput(const char *name, OutputModule *module, OutputCtx *outpu
                 module->ThreadExitPrintStats);
         /* Not used with wild card loggers */
         if (module->alproto != ALPROTO_UNKNOWN) {
-            logger_bits[module->alproto] |= (1<<module->logger_id);
+            logger_bits[module->alproto] |= BIT_U32(module->logger_id);
         }
     } else if (module->FiledataLogFunc) {
         SCLogDebug("%s is a filedata logger", module->name);
@@ -624,8 +679,7 @@ static void SetupOutput(const char *name, OutputModule *module, OutputCtx *outpu
             module->ThreadInit, module->ThreadDeinit,
             module->ThreadExitPrintStats);
     } else {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Unknown logger type: name=%s",
-            module->name);
+        SCLogError("Unknown logger type: name=%s", module->name);
     }
 }
 
@@ -639,11 +693,17 @@ static void RunModeInitializeEveOutput(ConfNode *conf, OutputCtx *parent_ctx)
 
     ConfNode *type = NULL;
     TAILQ_FOREACH(type, &types->head, next) {
-        SCLogConfig("enabling 'eve-log' module '%s'", type->val);
-
         int sub_count = 0;
         char subname[256];
-        snprintf(subname, sizeof(subname), "eve-log.%s", type->val);
+
+        if (strcmp(type->val, "ikev2") == 0) {
+            SCLogWarning("eve module 'ikev2' has been replaced by 'ike'");
+            strlcpy(subname, "eve-log.ike", sizeof(subname));
+        } else {
+            snprintf(subname, sizeof(subname), "eve-log.%s", type->val);
+        }
+
+        SCLogConfig("enabling 'eve-log' module '%s'", type->val);
 
         ConfNode *sub_output_config = ConfNodeLookupChild(type, type->val);
         if (sub_output_config != NULL) {
@@ -662,19 +722,17 @@ static void RunModeInitializeEveOutput(ConfNode *conf, OutputCtx *parent_ctx)
 
                 if (sub_module->parent_name == NULL ||
                         strcmp(sub_module->parent_name, "eve-log") != 0) {
-                    FatalError(SC_ERR_INVALID_ARGUMENT,
-                            "bad parent for %s", subname);
+                    FatalError("bad parent for %s", subname);
                 }
                 if (sub_module->InitSubFunc == NULL) {
-                    FatalError(SC_ERR_INVALID_ARGUMENT,
-                            "bad sub-module for %s", subname);
+                    FatalError("bad sub-module for %s", subname);
                 }
 
                 /* pass on parent output_ctx */
                 OutputInitResult result =
                     sub_module->InitSubFunc(sub_output_config, parent_ctx);
                 if (!result.ok || result.ctx == NULL) {
-                    continue;
+                    FatalError("unable to initialize sub-module %s", subname);
                 }
 
                 AddOutputToFreeList(sub_module, result.ctx);
@@ -686,8 +744,7 @@ static void RunModeInitializeEveOutput(ConfNode *conf, OutputCtx *parent_ctx)
         /* Error is no registered loggers with this name
          * were found .*/
         if (!sub_count) {
-            FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
-                    "No output module named %s", subname);
+            FatalErrorOnInit("No output module named %s", subname);
             continue;
         }
     }
@@ -725,6 +782,9 @@ static void RunModeInitializeLuaOutput(ConfNode *conf, OutputCtx *parent_ctx)
     }
 }
 
+extern bool g_file_logger_enabled;
+extern bool g_filedata_logger_enabled;
+
 /**
  * Initialize the output modules.
  */
@@ -748,8 +808,7 @@ void RunModeInitializeOutputs(void)
         output_config = ConfNodeLookupChild(output, output->val);
         if (output_config == NULL) {
             /* Shouldn't happen. */
-            FatalError(SC_ERR_INVALID_ARGUMENT,
-                "Failed to lookup configuration child node: %s", output->val);
+            FatalError("Failed to lookup configuration child node: %s", output->val);
         }
 
         if (strcmp(output->val, "tls-store") == 0) {
@@ -762,42 +821,29 @@ void RunModeInitializeOutputs(void)
         }
 
         if (strcmp(output->val, "file-log") == 0) {
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "file-log is no longer supported,"
-                    " use eve.files instead "
-                    "(see https://redmine.openinfosecfoundation.org/issues/2376"
-                    " for an explanation)");
+            SCLogWarning("file-log is no longer supported,"
+                         " use eve.files instead "
+                         "(see ticket #2376"
+                         " for an explanation)");
             continue;
         } else if (strncmp(output->val, "unified-", sizeof("unified-") - 1) == 0) {
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "Unified1 is no longer supported,"
-                    " use Unified2 instead "
-                    "(see https://redmine.openinfosecfoundation.org/issues/353"
-                    " for an explanation)");
+            SCLogWarning("Unified1 is no longer supported,"
+                         " use Unified2 instead "
+                         "(see ticket #353"
+                         " for an explanation)");
             continue;
         } else if (strncmp(output->val, "unified2-", sizeof("unified2-") - 1) == 0) {
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "Unified2 is no longer supported.");
+            SCLogWarning("Unified2 is no longer supported.");
             continue;
-        } else if (strcmp(output->val, "alert-prelude") == 0) {
-#ifndef PRELUDE
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "Prelude support not compiled in. Reconfigure/"
-                    "recompile with --enable-prelude to add Prelude "
-                    "support.");
-            continue;
-#endif
         } else if (strcmp(output->val, "lua") == 0) {
 #ifndef HAVE_LUA
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "lua support not compiled in. Reconfigure/"
-                    "recompile with lua(jit) and its development "
-                    "files installed to add lua support.");
+            SCLogWarning("lua support not compiled in. Reconfigure/"
+                         "recompile with lua(jit) and its development "
+                         "files installed to add lua support.");
             continue;
 #endif
         } else if (strcmp(output->val, "dns-log") == 0) {
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "dns-log is not longer available as of Suricata 5.0");
+            SCLogWarning("dns-log is not longer available as of Suricata 5.0");
             continue;
         } else if (strcmp(output->val, "tls-log") == 0) {
             tls_log_enabled = 1;
@@ -816,8 +862,7 @@ void RunModeInitializeOutputs(void)
             if (module->InitFunc != NULL) {
                 OutputInitResult r = module->InitFunc(output_config);
                 if (!r.ok) {
-                    FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
-                        "output module \"%s\": setup failed", output->val);
+                    FatalErrorOnInit("output module \"%s\": setup failed", output->val);
                     continue;
                 } else if (r.ctx == NULL) {
                     continue;
@@ -848,8 +893,7 @@ void RunModeInitializeOutputs(void)
             }
         }
         if (count == 0) {
-            FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
-                "No output module named %s", output->val);
+            FatalErrorOnInit("No output module named %s", output->val);
             continue;
         }
     }
@@ -858,8 +902,7 @@ void RunModeInitializeOutputs(void)
     if (!tls_store_present && tls_log_enabled) {
         /* old YAML with no "tls-store" in outputs. "tls-log" value needs
          * to be started using 'tls-log' config as own config */
-        SCLogWarning(SC_ERR_CONF_YAML_ERROR,
-                     "Please use 'tls-store' in YAML to configure TLS storage");
+        SCLogWarning("Please use 'tls-store' in YAML to configure TLS storage");
 
         TAILQ_FOREACH(output, &outputs->head, next) {
             output_config = ConfNodeLookupChild(output, output->val);
@@ -868,8 +911,7 @@ void RunModeInitializeOutputs(void)
 
                 OutputModule *module = OutputGetModuleByConfName("tls-store");
                 if (module == NULL) {
-                    SCLogWarning(SC_ERR_INVALID_ARGUMENT,
-                            "No output module named %s, ignoring", "tls-store");
+                    SCLogWarning("No output module named %s, ignoring", "tls-store");
                     continue;
                 }
 
@@ -877,8 +919,7 @@ void RunModeInitializeOutputs(void)
                 if (module->InitFunc != NULL) {
                     OutputInitResult r = module->InitFunc(output_config);
                     if (!r.ok) {
-                        FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
-                                "output module setup failed");
+                        FatalErrorOnInit("output module setup failed");
                         continue;
                     } else if (r.ctx == NULL) {
                         continue;
@@ -893,13 +934,33 @@ void RunModeInitializeOutputs(void)
     }
 
     /* register the logger bits to the app-layer */
-    int a;
+    AppProto a;
     for (a = 0; a < ALPROTO_MAX; a++) {
+        if (AppLayerParserSupportsFiles(IPPROTO_TCP, a)) {
+            if (g_file_logger_enabled)
+                logger_bits[a] |= BIT_U32(LOGGER_FILE);
+            if (g_filedata_logger_enabled)
+                logger_bits[a] |= BIT_U32(LOGGER_FILEDATA);
+            SCLogDebug("IPPROTO_TCP::%s: g_file_logger_enabled %d g_filedata_logger_enabled %d -> "
+                       "%08x",
+                    AppProtoToString(a), g_file_logger_enabled, g_filedata_logger_enabled,
+                    logger_bits[a]);
+        }
+        if (AppLayerParserSupportsFiles(IPPROTO_UDP, a)) {
+            if (g_file_logger_enabled)
+                logger_bits[a] |= BIT_U32(LOGGER_FILE);
+            if (g_filedata_logger_enabled)
+                logger_bits[a] |= BIT_U32(LOGGER_FILEDATA);
+        }
+
         if (logger_bits[a] == 0)
             continue;
 
-        const int tcp = AppLayerParserProtocolHasLogger(IPPROTO_TCP, a);
-        const int udp = AppLayerParserProtocolHasLogger(IPPROTO_UDP, a);
+        const int tcp = AppLayerParserProtocolHasLogger(IPPROTO_TCP, a) | (g_file_logger_enabled) |
+                        (g_filedata_logger_enabled);
+        const int udp = AppLayerParserProtocolHasLogger(IPPROTO_UDP, a) | (g_file_logger_enabled) |
+                        (g_filedata_logger_enabled);
+        SCLogDebug("tcp %d udp %d", tcp, udp);
 
         SCLogDebug("logger for %s: %s %s", AppProtoToString(a),
                 tcp ? "true" : "false", udp ? "true" : "false");
@@ -942,11 +1003,10 @@ void RunModeInitialize(void)
      * in case the default per-thread stack size is to be adjusted
      */
     const char *ss = NULL;
-    if ((ConfGetValue("threading.stack-size", &ss)) == 1) {
+    if ((ConfGet("threading.stack-size", &ss)) == 1) {
         if (ss != NULL) {
             if (ParseSizeStringU64(ss, &threading_set_stack_size) < 0) {
-                FatalError(SC_ERR_INVALID_ARGUMENT,
-                        "Failed to initialize thread_stack_size output, invalid limit: %s", ss);
+                FatalError("Failed to initialize thread_stack_size output, invalid limit: %s", ss);
             }
         }
     }
