@@ -26,8 +26,11 @@
  *
  */
 #include "suricata-common.h"
+#include "decode.h"
+#include "packet.h"
 #include "suricata.h"
 #include "threadvars.h"
+#include "util-datalink.h"
 #include "util-optimize.h"
 #include "tm-queuehandlers.h"
 #include "tm-threads.h"
@@ -64,9 +67,8 @@ void TmModuleNapatechDecodeRegister(void)
 
 TmEcode NoNapatechSupportExit(ThreadVars *tv, const void *initdata, void **data)
 {
-    SCLogError(SC_ERR_NAPATECH_NOSUPPORT,
-            "Error creating thread %s: you do not have support for Napatech adapter "
-            "enabled please recompile with --enable-napatech",
+    SCLogError("Error creating thread %s: you do not have support for Napatech adapter "
+               "enabled please recompile with --enable-napatech",
             tv->name);
     exit(EXIT_FAILURE);
 }
@@ -77,7 +79,7 @@ TmEcode NoNapatechSupportExit(ThreadVars *tv, const void *initdata, void **data)
 #include <numa.h>
 #include <nt.h>
 
-extern intmax_t max_pending_packets;
+extern uint16_t max_pending_packets;
 
 typedef struct NapatechThreadVars_
 {
@@ -265,8 +267,7 @@ int NapatechSetPortmap(int port, int peer)
         inline_port_map[port] = peer;
         inline_port_map[peer] = port;
     } else {
-        SCLogError(SC_ERR_NAPATECH_PARSE_CONFIG,
-                "Port pairing is already configured.");
+        SCLogError("Port pairing is already configured.");
         return 0;
     }
     return 1;
@@ -290,7 +291,7 @@ int NapatechGetAdapter(uint8_t port)
 
     if (unlikely(port_adapter_map[port] == -1)) {
         if ((status = NT_InfoOpen(&h_info_stream, "ExampleInfo")) != NT_SUCCESS) {
-            NAPATECH_ERROR(SC_ERR_NAPATECH_OPEN_FAILED, status);
+            NAPATECH_ERROR(status);
             return -1;
         }
         /* Read the system info */
@@ -298,7 +299,7 @@ int NapatechGetAdapter(uint8_t port)
         h_info.u.port_v9.portNo = (uint8_t) port;
         if ((status = NT_InfoRead(h_info_stream, &h_info)) != NT_SUCCESS) {
             /* Get the status code as text */
-            NAPATECH_ERROR(SC_ERR_NAPATECH_OPEN_FAILED, status);
+            NAPATECH_ERROR(status);
             NT_InfoClose(h_info_stream);
             return -1;
         }
@@ -329,13 +330,12 @@ struct IPv6Tuple4
     uint16_t dp;    /*!< Destination port */
 };
 
-
 /**
  * \brief Compares the byte order value of two IPv6 addresses.
  *
  *
  * \param addr_a The first address to compare
- * \param addr_b The second adress to compare
+ * \param addr_b The second address to compare
  *
  * \return -1 if addr_a < addr_b
  *          1 if addr_a > addr_b
@@ -378,8 +378,8 @@ static NtFlowStream_t InitFlowStream(int adapter, int stream_id)
     snprintf(flow_name, sizeof(flow_name), "Flow_stream_%d", stream_id );
     SCLogDebug("Opening flow programming stream:  %s", flow_name);
     if ((status = NT_FlowOpen_Attr(&hFlowStream, flow_name, &attr)) != NT_SUCCESS) {
-        SCLogWarning(SC_WARN_COMPATIBILITY,
-                "Napatech bypass functionality not supported by the FPGA version on adapter %d - disabling support.",
+        SCLogWarning("Napatech bypass functionality not supported by the FPGA version on adapter "
+                     "%d - disabling support.",
                 adapter);
         return NULL;
     }
@@ -581,7 +581,7 @@ static int ProgramFlow(Packet *p, int is_inline)
     flow_match.gfi = 1; /* Generate FlowInfo records */
     flow_match.tau = 1; /* tcp automatic unlearn */
 
-    if (PACKET_TEST_ACTION(p, ACTION_DROP)) {
+    if (PacketCheckAction(p, ACTION_DROP)) {
         flow_match.keySetId = NAPATECH_FLOWTYPE_DROP;
     } else {
         if (is_inline) {
@@ -593,7 +593,7 @@ static int ProgramFlow(Packet *p, int is_inline)
 
     if (NT_FlowWrite(ntpv->flow_stream, &flow_match, -1) != NT_SUCCESS) {
         if (!(suricata_ctl_flags & SURICATA_STOP)) {
-            SCLogError(SC_ERR_NAPATECH_OPEN_FAILED,"NT_FlowWrite failed!.");
+            SCLogError("NT_FlowWrite failed!.");
             exit(EXIT_FAILURE);
         }
     }
@@ -648,14 +648,16 @@ TmEcode NapatechStreamThreadInit(ThreadVars *tv, const void *initdata, void **da
 
     NapatechThreadVars *ntv = SCCalloc(1, sizeof (NapatechThreadVars));
     if (unlikely(ntv == NULL)) {
-        FatalError(SC_ERR_FATAL,
-                   "Failed to allocate memory for NAPATECH  thread vars.");
+        FatalError("Failed to allocate memory for NAPATECH  thread vars.");
     }
 
     memset(ntv, 0, sizeof (NapatechThreadVars));
     ntv->stream_id = stream_id;
     ntv->tv = tv;
     ntv->hba = conf->hba;
+
+    DatalinkSetGlobalType(LINKTYPE_ETHERNET);
+
     SCLogDebug("Started processing packets from NAPATECH  Stream: %lu", ntv->stream_id);
 
     *data = (void *) ntv;
@@ -680,7 +682,7 @@ static void NapatechReleasePacket(struct Packet_ *p)
      * before releasing the Napatech buffer back to NTService.
      */
 #ifdef NAPATECH_ENABLE_BYPASS
-    if (is_inline && PACKET_TEST_ACTION(p, ACTION_DROP)) {
+    if (is_inline && PacketCheckAction(p, ACTION_DROP)) {
         p->ntpv.dyn3->wireLength = 0;
     }
 
@@ -713,8 +715,7 @@ static int GetNumaNode(void)
     cpu = sched_getcpu();
     node = numa_node_of_cpu(cpu);
 #else
-    SCLogWarning(SC_ERR_NAPATECH_NOSUPPORT,
-            "Auto configuration of NUMA node is not supported on this OS.");
+    SCLogWarning("Auto configuration of NUMA node is not supported on this OS.");
 #endif
 
     return node;
@@ -769,8 +770,8 @@ static void RecommendNUMAConfig(SCLogLevel log_level)
                 "E.g.: HostBuffersRx=%s%s%s%s", string0, string1, string2,
                 string3);
     } else if (log_level == SC_LOG_ERROR) {
-        SCLogError(SC_ERR_NAPATECH_STREAMS_REGISTER_FAILED,
-                "Or, try running /opt/napatech3/bin/ntpl -e \"delete=all\" to clean-up stream NUMA config.");
+        SCLogError("Or, try running /opt/napatech3/bin/ntpl -e \"delete=all\" to clean-up stream "
+                   "NUMA config.");
     }
 }
 
@@ -864,14 +865,12 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
             closer = 1;
 
             if (status == 0x20002061) {
-                SCLogError(SC_ERR_NAPATECH_STREAMS_REGISTER_FAILED,
-                        "Check host buffer configuration in ntservice.ini.");
+                SCLogError("Check host buffer configuration in ntservice.ini.");
                 RecommendNUMAConfig(SC_LOG_ERROR);
                 exit(EXIT_FAILURE);
 
             } else if (status == 0x20000008) {
-                        FatalError(SC_ERR_FATAL,
-                                   "Check napatech.ports in the suricata config file.");
+                FatalError("Check napatech.ports in the suricata config file.");
             }
             RecommendNUMAConfig(SC_LOG_PERF);
             SCLogNotice("Napatech packet input engine started.");
@@ -885,8 +884,7 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
     if (ntv->hba > 0) {
         char *s_hbad_pkt = SCCalloc(1, 32);
         if (unlikely(s_hbad_pkt == NULL)) {
-                    FatalError(SC_ERR_FATAL,
-                               "Failed to allocate memory for NAPATECH stream counter.");
+            FatalError("Failed to allocate memory for NAPATECH stream counter.");
         }
         snprintf(s_hbad_pkt, 32, "nt%d.hba_drop", ntv->stream_id);
         hba_pkt = StatsRegisterCounter(s_hbad_pkt, tv);
@@ -898,13 +896,17 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
     if ((status = NT_NetRxOpen(&(ntv->rx_stream), "SuricataStream",
             NT_NET_INTERFACE_PACKET, ntv->stream_id, ntv->hba)) != NT_SUCCESS) {
 
-        NAPATECH_ERROR(SC_ERR_NAPATECH_OPEN_FAILED, status);
+        NAPATECH_ERROR(status);
         SCFree(ntv);
         SCReturnInt(TM_ECODE_FAILED);
     }
     TmSlot *s = (TmSlot *) slot;
     ntv->slot = s->slot_next;
 
+    // Indicate that the thread is actually running its application level code (i.e., it can poll
+    // packets)
+    TmThreadsSetFlag(tv, THV_RUNNING);
+
     while (!(suricata_ctl_flags & SURICATA_STOP)) {
         /* make sure we have at least one packet in the packet pool, to prevent
          * us from alloc'ing packets at line rate */
@@ -919,7 +921,7 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
             }
             continue;
         } else if (unlikely(status != NT_SUCCESS)) {
-            NAPATECH_ERROR(SC_ERR_NAPATECH_OPEN_FAILED, status);
+            NAPATECH_ERROR(status);
             SCLogInfo("Failed to read from Napatech Stream %d: %s",
                     ntv->stream_id, error_buffer);
             break;
@@ -946,25 +948,28 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
          */
         switch (NT_NET_GET_PKT_TIMESTAMP_TYPE(packet_buffer)) {
             case NT_TIMESTAMP_TYPE_NATIVE_UNIX:
-                p->ts.tv_sec = pkt_ts / 100000000;
-                p->ts.tv_usec = ((pkt_ts % 100000000) / 100) + ((pkt_ts % 100) > 50 ? 1 : 0);
+                p->ts = SCTIME_FROM_SECS(pkt_ts / 100000000);
+                p->ts += SCTIME_FROM_USECS(
+                        ((pkt_ts % 100000000) / 100) + ((pkt_ts % 100) > 50 ? 1 : 0));
                 break;
             case NT_TIMESTAMP_TYPE_PCAP:
-                p->ts.tv_sec = pkt_ts >> 32;
-                p->ts.tv_usec = pkt_ts & 0xFFFFFFFF;
+                p->ts = SCTIME_FROM_SECS(pkt_ts >> 32);
+                p->ts += SCTIME_FROM_USECS(pkt_ts & 0xFFFFFFFF);
                 break;
             case NT_TIMESTAMP_TYPE_PCAP_NANOTIME:
-                p->ts.tv_sec = pkt_ts >> 32;
-                p->ts.tv_usec = ((pkt_ts & 0xFFFFFFFF) / 1000) + ((pkt_ts % 1000) > 500 ? 1 : 0);
+                p->ts = SCTIME_FROM_SECS(pkt_ts >> 32);
+                p->ts += SCTIME_FROM_USECS(
+                        ((pkt_ts & 0xFFFFFFFF) / 1000) + ((pkt_ts % 1000) > 500 ? 1 : 0));
                 break;
             case NT_TIMESTAMP_TYPE_NATIVE_NDIS:
                 /* number of seconds between 1/1/1601 and 1/1/1970 */
-                p->ts.tv_sec = (pkt_ts / 100000000) - 11644473600;
-                p->ts.tv_usec = ((pkt_ts % 100000000) / 100) + ((pkt_ts % 100) > 50 ? 1 : 0);
+                p->ts = SCTIME_FROM_SECS((pkt_ts / 100000000) - 11644473600);
+                p->ts += SCTIME_FROM_USECS(
+                        ((pkt_ts % 100000000) / 100) + ((pkt_ts % 100) > 50 ? 1 : 0));
                 break;
             default:
-                SCLogError(SC_ERR_NAPATECH_TIMESTAMP_TYPE_NOT_SUPPORTED,
-                        "Packet from Napatech Stream: %u does not have a supported timestamp format",
+                SCLogError("Packet from Napatech Stream: %u does not have a supported timestamp "
+                           "format",
                         ntv->stream_id);
                 NT_NetRxRelease(ntv->rx_stream, packet_buffer);
                 SCReturnInt(TM_ECODE_FAILED);
@@ -975,7 +980,7 @@ TmEcode NapatechPacketLoop(ThreadVars *tv, void *data, void *slot)
             stat_cmd.cmd = NT_NETRX_READ_CMD_STREAM_DROP;
             /* Update drop counter */
             if (unlikely((status = NT_NetRxRead(ntv->rx_stream, &stat_cmd)) != NT_SUCCESS)) {
-                NAPATECH_ERROR(SC_ERR_NAPATECH_OPEN_FAILED, status);
+                NAPATECH_ERROR(status);
                 SCLogInfo("Couldn't retrieve drop statistics from the RX stream: %u",
                         ntv->stream_id);
             } else {
@@ -1112,8 +1117,7 @@ TmEcode NapatechDecode(ThreadVars *tv, Packet *p, void *data)
             DecodeEthernet(tv, dtv, p, GET_PKT_DATA(p), GET_PKT_LEN(p));
             break;
         default:
-            SCLogError(SC_ERR_DATALINK_UNIMPLEMENTED,
-                    "Datalink type %" PRId32 " not yet supported in module NapatechDecode",
+            SCLogError("Datalink type %" PRId32 " not yet supported in module NapatechDecode",
                     p->datalink);
             break;
     }
