@@ -24,8 +24,10 @@
  */
 
 #include "source-pcap-file-directory-helper.h"
+#include "suricata.h"
 #include "runmode-unix-socket.h"
 #include "util-mem.h"
+#include "util-time.h"
 #include "source-pcap-file.h"
 
 static void GetTime(struct timespec *tm);
@@ -131,11 +133,11 @@ TmEcode PcapDirectoryFailure(PcapFileDirectoryVars *ptv)
     TmEcode status = TM_ECODE_FAILED;
 
     if (unlikely(ptv == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Directory vars was null");
+        SCLogError("Directory vars was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
     if (unlikely(ptv->shared == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Directory shared vars was null");
+        SCLogError("Directory shared vars was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -151,11 +153,11 @@ TmEcode PcapDirectoryDone(PcapFileDirectoryVars *ptv)
     TmEcode status = TM_ECODE_DONE;
 
     if (unlikely(ptv == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Directory vars was null");
+        SCLogError("Directory vars was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
     if (unlikely(ptv->shared == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Directory shared vars was null");
+        SCLogError("Directory shared vars was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -176,36 +178,26 @@ TmEcode PcapDetermineDirectoryOrFile(char *filename, DIR **directory)
     if (temp_dir == NULL) {//if null, our filename may just be a normal file
         switch (errno) {
             case EACCES:
-                SCLogError(SC_ERR_FOPEN, "%s: Permission denied", filename);
+                SCLogError("%s: Permission denied", filename);
                 break;
 
             case EBADF:
-                SCLogError(SC_ERR_FOPEN,
-                           "%s: Not a valid file descriptor opened for reading",
-                           filename);
+                SCLogError("%s: Not a valid file descriptor opened for reading", filename);
                 break;
 
             case EMFILE:
-                SCLogError(SC_ERR_FOPEN,
-                           "%s: Per process open file descriptor limit reached",
-                           filename);
+                SCLogError("%s: Per process open file descriptor limit reached", filename);
                 break;
 
             case ENFILE:
-                SCLogError(SC_ERR_FOPEN,
-                           "%s: System wide open file descriptor limit reached",
-                           filename);
+                SCLogError("%s: System wide open file descriptor limit reached", filename);
                 break;
 
             case ENOENT:
-                SCLogError(SC_ERR_FOPEN,
-                           "%s: Does not exist, or name is an empty string",
-                           filename);
+                SCLogError("%s: Does not exist, or name is an empty string", filename);
                 break;
             case ENOMEM:
-                SCLogError(SC_ERR_FOPEN,
-                           "%s: Insufficient memory to complete the operation",
-                           filename);
+                SCLogError("%s: Insufficient memory to complete the operation", filename);
                 break;
 
             case ENOTDIR: //no error checking the directory, just is a plain file
@@ -214,7 +206,7 @@ TmEcode PcapDetermineDirectoryOrFile(char *filename, DIR **directory)
                 break;
 
             default:
-                SCLogError(SC_ERR_FOPEN, "%s: %" PRId32, filename, errno);
+                SCLogError("%s: %" PRId32, filename, errno);
         }
     } else {
         //no error, filename references a directory
@@ -265,17 +257,17 @@ TmEcode PcapDirectoryInsertFile(PcapFileDirectoryVars *pv,
     PendingFile *next_file_to_compare = NULL;
 
     if (unlikely(pv == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "No directory vars passed");
+        SCLogError("No directory vars passed");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
     if (unlikely(file_to_add == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "File passed was null");
+        SCLogError("File passed was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
     if (unlikely(file_to_add->filename == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "File was passed with null filename");
+        SCLogError("File was passed with null filename");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -307,11 +299,11 @@ TmEcode PcapDirectoryPopulateBuffer(PcapFileDirectoryVars *pv,
                                     struct timespec *older_than
 ) {
     if (unlikely(pv == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "No directory vars passed");
+        SCLogError("No directory vars passed");
         SCReturnInt(TM_ECODE_FAILED);
     }
     if (unlikely(pv->filename == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "No directory filename was passed");
+        SCLogError("No directory filename was passed");
         SCReturnInt(TM_ECODE_FAILED);
     }
     struct dirent * dir = NULL;
@@ -335,7 +327,7 @@ TmEcode PcapDirectoryPopulateBuffer(PcapFileDirectoryVars *pv,
         written = snprintf(pathbuff, PATH_MAX, "%s/%s", pv->filename, dir->d_name);
 
         if (written <= 0 || written >= PATH_MAX) {
-            SCLogError(SC_ERR_SPRINTF, "Could not write path");
+            SCLogError("Could not write path");
 
             SCReturnInt(TM_ECODE_FAILED);
         } else {
@@ -365,14 +357,14 @@ TmEcode PcapDirectoryPopulateBuffer(PcapFileDirectoryVars *pv,
 
             file_to_add = SCCalloc(1, sizeof(PendingFile));
             if (unlikely(file_to_add == NULL)) {
-                SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate pending file");
+                SCLogError("Failed to allocate pending file");
 
                 SCReturnInt(TM_ECODE_FAILED);
             }
 
             file_to_add->filename = SCStrdup(pathbuff);
             if (unlikely(file_to_add->filename == NULL)) {
-                SCLogError(SC_ERR_MEM_ALLOC, "Failed to copy filename");
+                SCLogError("Failed to copy filename");
                 CleanupPendingFile(file_to_add);
 
                 SCReturnInt(TM_ECODE_FAILED);
@@ -385,7 +377,7 @@ TmEcode PcapDirectoryPopulateBuffer(PcapFileDirectoryVars *pv,
                        (uintmax_t)SCTimespecAsEpochMillis(&file_to_add->modified_time));
 
             if (PcapDirectoryInsertFile(pv, file_to_add) == TM_ECODE_FAILED) {
-                SCLogError(SC_ERR_INVALID_ARGUMENT, "Failed to add file");
+                SCLogError("Failed to add file");
                 CleanupPendingFile(file_to_add);
 
                 SCReturnInt(TM_ECODE_FAILED);
@@ -401,7 +393,7 @@ TmEcode PcapDirectoryDispatchForTimeRange(PcapFileDirectoryVars *pv,
                                           struct timespec *older_than)
 {
     if (PcapDirectoryPopulateBuffer(pv, older_than) == TM_ECODE_FAILED) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Failed to populate directory buffer");
+        SCLogError("Failed to populate directory buffer");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -424,31 +416,29 @@ TmEcode PcapDirectoryDispatchForTimeRange(PcapFileDirectoryVars *pv,
             TAILQ_REMOVE(&pv->directory_content, current_file, next);
 
             if (unlikely(current_file == NULL)) {
-                SCLogWarning(SC_ERR_PCAP_DISPATCH, "Current file was null");
+                SCLogWarning("Current file was null");
             } else if (unlikely(current_file->filename == NULL)) {
-                SCLogWarning(SC_ERR_PCAP_DISPATCH, "Current file filename was null");
+                SCLogWarning("Current file filename was null");
             } else {
                 SCLogDebug("Processing file %s", current_file->filename);
 
                 PcapFileFileVars *pftv = SCMalloc(sizeof(PcapFileFileVars));
                 if (unlikely(pftv == NULL)) {
-                    SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate PcapFileFileVars");
+                    SCLogError("Failed to allocate PcapFileFileVars");
                     SCReturnInt(TM_ECODE_FAILED);
                 }
                 memset(pftv, 0, sizeof(PcapFileFileVars));
 
                 pftv->filename = SCStrdup(current_file->filename);
                 if (unlikely(pftv->filename == NULL)) {
-                    SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate filename");
+                    SCLogError("Failed to allocate filename");
                     CleanupPcapFileFileVars(pftv);
                     SCReturnInt(TM_ECODE_FAILED);
                 }
                 pftv->shared = pv->shared;
 
                 if (InitPcapFile(pftv) == TM_ECODE_FAILED) {
-                    SCLogWarning(SC_ERR_PCAP_DISPATCH,
-                                 "Failed to init pcap file %s, skipping",
-                                 current_file->filename);
+                    SCLogWarning("Failed to init pcap file %s, skipping", current_file->filename);
                     CleanupPendingFile(current_file);
                     CleanupPcapFileFileVars(pftv);
                     status = TM_ECODE_OK;
@@ -542,7 +532,7 @@ TmEcode PcapDirectoryDispatch(PcapFileDirectoryVars *ptv)
     StatsSyncCountersIfSignalled(ptv->shared->tv);
 
     if (status == TM_ECODE_FAILED) {
-        SCLogError(SC_ERR_PCAP_DISPATCH, "Directory %s run mode failed", ptv->filename);
+        SCLogError("Directory %s run mode failed", ptv->filename);
         status = PcapDirectoryFailure(ptv);
     } else {
         SCLogInfo("Directory run mode complete");
