@@ -24,12 +24,14 @@
  */
 
 #include "source-pcap-file-helper.h"
+#include "suricata.h"
+#include "util-datalink.h"
 #include "util-checksum.h"
 #include "util-profiling.h"
 #include "source-pcap-file.h"
 #include "util-exception-policy.h"
 
-extern intmax_t max_pending_packets;
+extern uint16_t max_pending_packets;
 extern PcapFileGlobalVars pcap_g;
 
 static void PcapFileCallbackLoop(char *user, struct pcap_pkthdr *h, u_char *pkt);
@@ -45,8 +47,7 @@ void CleanupPcapFileFileVars(PcapFileFileVars *pfv)
             if (pfv->shared != NULL && pfv->shared->should_delete) {
                 SCLogDebug("Deleting pcap file %s", pfv->filename);
                 if (unlink(pfv->filename) != 0) {
-                    SCLogWarning(SC_ERR_PCAP_FILE_DELETE_FAILED,
-                                 "Failed to delete %s", pfv->filename);
+                    SCLogWarning("Failed to delete %s: %s", pfv->filename, strerror(errno));
                 }
             }
             SCFree(pfv->filename);
@@ -76,9 +77,8 @@ void PcapFileCallbackLoop(char *user, struct pcap_pkthdr *h, u_char *pkt)
     PACKET_PROFILING_TMM_START(p, TMM_RECEIVEPCAPFILE);
 
     PKT_SET_SRC(p, PKT_SRC_WIRE);
-    p->ts.tv_sec = h->ts.tv_sec;
-    p->ts.tv_usec = h->ts.tv_usec % 1000000;
-    SCLogDebug("p->ts.tv_sec %"PRIuMAX"", (uintmax_t)p->ts.tv_sec);
+    p->ts = SCTIME_FROM_TIMEVAL(&h->ts);
+    SCLogDebug("p->ts.tv_sec %" PRIuMAX "", (uintmax_t)SCTIME_SECS(p->ts));
     p->datalink = ptv->datalink;
     p->pcap_cnt = ++pcap_g.cnt;
 
@@ -129,7 +129,7 @@ TmEcode PcapFileDispatch(PcapFileFileVars *ptv)
 
     /* initialize all the thread's initial timestamp */
     if (likely(ptv->first_pkt_hdr != NULL)) {
-        TmThreadsInitThreadsTimestamp(&ptv->first_pkt_ts);
+        TmThreadsInitThreadsTimestamp(SCTIME_FROM_TIMEVAL(&ptv->first_pkt_ts));
         PcapFileCallbackLoop((char *)ptv, ptv->first_pkt_hdr,
                 (u_char *)ptv->first_pkt_data);
         ptv->first_pkt_hdr = NULL;
@@ -153,8 +153,8 @@ TmEcode PcapFileDispatch(PcapFileFileVars *ptv)
         int r = pcap_dispatch(ptv->pcap_handle, packet_q_len,
                           (pcap_handler)PcapFileCallbackLoop, (u_char *)ptv);
         if (unlikely(r == -1)) {
-            SCLogError(SC_ERR_PCAP_DISPATCH, "error code %" PRId32 " %s for %s",
-                       r, pcap_geterr(ptv->pcap_handle), ptv->filename);
+            SCLogError("error code %" PRId32 " %s for %s", r, pcap_geterr(ptv->pcap_handle),
+                    ptv->filename);
             if (ptv->shared->cb_result == TM_ECODE_FAILED) {
                 SCReturnInt(TM_ECODE_FAILED);
             }
@@ -165,8 +165,7 @@ TmEcode PcapFileDispatch(PcapFileFileVars *ptv)
             ptv->shared->files++;
             loop_result = TM_ECODE_DONE;
         } else if (ptv->shared->cb_result == TM_ECODE_FAILED) {
-            SCLogError(SC_ERR_PCAP_DISPATCH,
-                       "Pcap callback PcapFileCallbackLoop failed for %s", ptv->filename);
+            SCLogError("Pcap callback PcapFileCallbackLoop failed for %s", ptv->filename);
             loop_result = TM_ECODE_FAILED;
         }
         StatsSyncCountersIfSignalled(ptv->shared->tv);
@@ -184,8 +183,7 @@ static bool PeekFirstPacketTimestamp(PcapFileFileVars *pfv)
 {
     int r = pcap_next_ex(pfv->pcap_handle, &pfv->first_pkt_hdr, &pfv->first_pkt_data);
     if (r <= 0 || pfv->first_pkt_hdr == NULL) {
-        SCLogError(SC_ERR_PCAP_OPEN_OFFLINE,
-                "failed to get first packet timestamp. pcap_next_ex(): %d", r);
+        SCLogError("failed to get first packet timestamp. pcap_next_ex(): %d", r);
         return false;
     }
     /* timestamp in pfv->first_pkt_hdr may not be 'struct timeval' so
@@ -200,13 +198,13 @@ TmEcode InitPcapFile(PcapFileFileVars *pfv)
     char errbuf[PCAP_ERRBUF_SIZE] = "";
 
     if(unlikely(pfv->filename == NULL)) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Filename was null");
+        SCLogError("Filename was null");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
     pfv->pcap_handle = pcap_open_offline(pfv->filename, errbuf);
     if (pfv->pcap_handle == NULL) {
-        SCLogError(SC_ERR_FOPEN, "%s", errbuf);
+        SCLogError("%s", errbuf);
         SCReturnInt(TM_ECODE_FAILED);
     }
 
@@ -214,14 +212,14 @@ TmEcode InitPcapFile(PcapFileFileVars *pfv)
         SCLogInfo("using bpf-filter \"%s\"", pfv->shared->bpf_string);
 
         if (pcap_compile(pfv->pcap_handle, &pfv->filter, pfv->shared->bpf_string, 1, 0) < 0) {
-            SCLogError(SC_ERR_BPF, "bpf compilation error %s for %s",
-                       pcap_geterr(pfv->pcap_handle), pfv->filename);
+            SCLogError("bpf compilation error %s for %s", pcap_geterr(pfv->pcap_handle),
+                    pfv->filename);
             SCReturnInt(TM_ECODE_FAILED);
         }
 
         if (pcap_setfilter(pfv->pcap_handle, &pfv->filter) < 0) {
-            SCLogError(SC_ERR_BPF,"could not set bpf filter %s for %s",
-                       pcap_geterr(pfv->pcap_handle), pfv->filename);
+            SCLogError("could not set bpf filter %s for %s", pcap_geterr(pfv->pcap_handle),
+                    pfv->filename);
             pcap_freecode(&pfv->filter);
             SCReturnInt(TM_ECODE_FAILED);
         }
@@ -230,6 +228,7 @@ TmEcode InitPcapFile(PcapFileFileVars *pfv)
 
     pfv->datalink = pcap_datalink(pfv->pcap_handle);
     SCLogDebug("datalink %" PRId32 "", pfv->datalink);
+    DatalinkSetGlobalType(pfv->datalink);
 
     if (!PeekFirstPacketTimestamp(pfv))
         SCReturnInt(TM_ECODE_FAILED);
@@ -265,9 +264,8 @@ TmEcode ValidateLinkType(int datalink, DecoderFunc *DecoderFn)
             break;
 
         default:
-            SCLogError(SC_ERR_UNIMPLEMENTED,
-                    "datalink type %"PRId32" not (yet) supported in module PcapFile.",
-                    datalink);
+            SCLogError(
+                    "datalink type %" PRId32 " not (yet) supported in module PcapFile.", datalink);
             SCReturnInt(TM_ECODE_FAILED);
     }
 
