@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -36,6 +36,20 @@
 #define _GNU_SOURCE
 #define __USE_GNU
 
+#if defined(__clang_analyzer__)
+/* clang analyzer acts as DEBUG_VALIDATION in some places, so
+ * force this so #ifdef DEBUG_VALIDATION code gets included */
+#define DEBUG_VALIDATION 1
+#endif
+
+#if defined(__has_feature)
+#if __has_feature(address_sanitizer)
+#define SC_ADDRESS_SANITIZER 1
+#endif
+#elif defined(__SANITIZE_ADDRESS__)
+#define SC_ADDRESS_SANITIZER 1
+#endif
+
 #if HAVE_CONFIG_H
 #include <autoconf.h>
 #endif
@@ -126,19 +140,19 @@
 #endif
 
 #ifdef HAVE_TYPE_U_LONG_NOT_DEFINED
-typedef unsigned long int u_long
+typedef unsigned long int u_long;
 #endif
 #ifdef HAVE_TYPE_U_INT_NOT_DEFINED
-typedef unsigned int u_int
+typedef unsigned int u_int;
 #endif
 #ifdef HAVE_TYPE_U_SHORT_NOT_DEFINED
-typedef unsigned short u_short
+typedef unsigned short u_short;
 #endif
 #ifdef HAVE_TYPE_U_CHAR_NOT_DEFINED
-typedef unsigned char u_char
+typedef unsigned char u_char;
 #endif
 
-#include <pcre.h>
+#include <pcre2.h>
 
 #ifdef HAVE_SYSLOG_H
 #include <syslog.h>
@@ -261,11 +275,6 @@ typedef unsigned char u_char
 #ifndef JSON_ESCAPE_SLASH
 #define JSON_ESCAPE_SLASH 0
 #endif
-/* Appears not all current distros have jansson that defines this. */
-#ifndef json_boolean
-#define json_boolean(val)      SCJsonBool((val))
-//#define json_boolean(val)      ((val) ? json_true() : json_false())
-#endif
 
 #ifdef HAVE_MAGIC
 #include <magic.h>
@@ -373,10 +382,6 @@ typedef unsigned char u_char
     #error "byte order: can't figure out big or little"
 #endif
 
-#ifndef HAVE_PCRE_FREE_STUDY
-#define pcre_free_study pcre_free
-#endif
-
 #ifndef MIN
 #define MIN(x, y) (((x)<(y))?(x):(y))
 #endif
@@ -422,6 +427,10 @@ typedef unsigned char u_char
         (b) = t;                        \
     } while (0)
 
+#include <ctype.h>
+#define u8_tolower(c) ((uint8_t)tolower((uint8_t)(c)))
+#define u8_toupper(c) ((uint8_t)toupper((uint8_t)(c)))
+
 typedef enum PacketProfileDetectId_ {
     PROF_DETECT_SETUP,
     PROF_DETECT_GETSGH,
@@ -431,6 +440,7 @@ typedef enum PacketProfileDetectId_ {
     PROF_DETECT_PF_PKT,
     PROF_DETECT_PF_PAYLOAD,
     PROF_DETECT_PF_TX,
+    PROF_DETECT_PF_RECORD,
     PROF_DETECT_PF_SORT1,
     PROF_DETECT_PF_SORT2,
     PROF_DETECT_NONMPMLIST,
@@ -441,46 +451,26 @@ typedef enum PacketProfileDetectId_ {
     PROF_DETECT_SIZE,
 } PacketProfileDetectId;
 
-/** \note update PacketProfileLoggertIdToString if you change anything here */
+/** \note update PacketProfileLoggerIdToString if you change anything here */
 typedef enum {
     LOGGER_UNDEFINED,
 
     /* TX loggers first for low logger IDs */
-    LOGGER_DNS_TS,
-    LOGGER_DNS_TC,
     LOGGER_HTTP,
     LOGGER_TLS_STORE,
     LOGGER_TLS,
-    LOGGER_JSON_DNS_TS,
-    LOGGER_JSON_DNS_TC,
-    LOGGER_JSON_HTTP,
-    LOGGER_JSON_SMTP,
-    LOGGER_JSON_TLS,
-    LOGGER_JSON_NFS,
-    LOGGER_JSON_TFTP,
-    LOGGER_JSON_FTP,
-    LOGGER_JSON_DNP3_TS,
-    LOGGER_JSON_DNP3_TC,
-    LOGGER_JSON_SSH,
-    LOGGER_JSON_SMB,
-    LOGGER_JSON_IKEV2,
-    LOGGER_JSON_KRB5,
-    LOGGER_JSON_DHCP,
-    LOGGER_JSON_SNMP,
-    LOGGER_JSON_SIP,
-    LOGGER_JSON_TEMPLATE_RUST,
-    LOGGER_JSON_RFB,
-    LOGGER_JSON_MQTT,
-    LOGGER_JSON_TEMPLATE,
-    LOGGER_JSON_RDP,
-    LOGGER_JSON_DCERPC,
-    LOGGER_JSON_HTTP2,
+    LOGGER_JSON_TX,
+    LOGGER_FILE,
+    LOGGER_FILEDATA,
+
+    /** \warning Note that transaction loggers here with a value > 31
+        will not work. */
+
+    /* non-tx loggers below */
 
     LOGGER_ALERT_DEBUG,
     LOGGER_ALERT_FAST,
-    LOGGER_UNIFIED2,
     LOGGER_ALERT_SYSLOG,
-    LOGGER_DROP,
     LOGGER_JSON_ALERT,
     LOGGER_JSON_ANOMALY,
     LOGGER_JSON_DROP,
@@ -491,37 +481,39 @@ typedef enum {
     LOGGER_JSON_NETFLOW,
     LOGGER_STATS,
     LOGGER_JSON_STATS,
-    LOGGER_PRELUDE,
     LOGGER_PCAP,
     LOGGER_JSON_METADATA,
+    LOGGER_JSON_FRAME,
+    LOGGER_JSON_STREAM,
     LOGGER_SIZE,
 } LoggerId;
 
-#include "util-optimize.h"
-#ifndef SURICATA_PLUGIN
-#include <htp/htp.h>
-#endif
-#include "threads.h"
-#include "tm-threads-common.h"
-#include "util-debug.h"
-#include "util-error.h"
-#include "util-mem.h"
-#ifndef SURICATA_PLUGIN
-#include "detect-engine-alert.h"
-#endif
-#include "util-path.h"
-#include "util-conf.h"
+#ifndef HAVE_LUA
 
-#ifdef HAVE_LUA
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
-#else
 /* If we don't have Lua, create a typedef for lua_State so the
  * exported Lua functions don't fail the build. */
 typedef void lua_State;
+
+#else
+
+#include <lua.h>
+#include <lualib.h>
+#include <lauxlib.h>
+
 #endif
 
+#include "tm-threads-common.h"
+#include "util-optimize.h"
+#include "util-time.h"
+#include "util-mem.h"
+#include "util-memcmp.h"
+#include "util-atomic.h"
+#include "util-unittest.h"
+
+// pseudo system headers
+#include "queue.h"
+#include "tree.h"
+
 #ifndef HAVE_STRLCAT
 size_t strlcat(char *, const char *src, size_t siz);
 #endif
@@ -549,9 +541,4 @@ extern int g_ut_covered;
 
 #define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(arr[0]))
 
-#ifndef NAME_MAX
-#define NAME_MAX 255
-#endif
-
 #endif /* __SURICATA_COMMON_H__ */
-
