@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2016 Open Information Security Foundation
+/* Copyright (C) 2011-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -20,12 +20,14 @@
 #include "util-device.h"
 #include "util-ioctl.h"
 #include "util-misc.h"
+#include "util-dpdk.h"
 
 #include "device-storage.h"
+#include "util-debug.h"
 
 #define MAX_DEVNAME 10
 
-static int g_bypass_storage_id = -1;
+static LiveDevStorageId g_bypass_storage_id = { .id = -1 };
 
 /**
  * \file
@@ -130,6 +132,12 @@ int LiveRegisterDevice(const char *dev)
         return -1;
     }
 
+    int id = LiveGetDeviceCount();
+    if (id > UINT16_MAX) {
+        SCFree(pd);
+        return -1;
+    }
+
     pd->dev = SCStrdup(dev);
     if (unlikely(pd->dev == NULL)) {
         SCFree(pd);
@@ -141,7 +149,7 @@ int LiveRegisterDevice(const char *dev)
     SC_ATOMIC_INIT(pd->pkts);
     SC_ATOMIC_INIT(pd->drop);
     SC_ATOMIC_INIT(pd->invalid_checksums);
-    pd->id = LiveGetDeviceCount();
+    pd->id = (uint16_t)id;
     TAILQ_INSERT_TAIL(&live_devices, pd, next);
 
     SCLogDebug("Device \"%s\" registered and created.", dev);
@@ -189,49 +197,6 @@ const char *LiveGetDeviceName(int number)
     return NULL;
 }
 
-/**
- *  \brief Get the number of pre registered devices
- *
- *  \retval cnt the number of pre registered devices
- */
-int LiveGetDeviceNameCount(void)
-{
-    int i = 0;
-    LiveDeviceName *pd;
-
-    TAILQ_FOREACH(pd, &pre_live_devices, next) {
-        i++;
-    }
-
-    return i;
-}
-
-/**
- *  \brief Get a pointer to the pre device name at idx
- *
- *  \param number idx of the pre device in our list
- *
- *  \retval ptr pointer to the string containing the device
- *  \retval NULL on error
- */
-const char *LiveGetDeviceNameName(int number)
-{
-    int i = 0;
-    LiveDeviceName *pd;
-
-    TAILQ_FOREACH(pd, &pre_live_devices, next) {
-        if (i == number) {
-            return pd->dev;
-        }
-
-        i++;
-    }
-
-    return NULL;
-}
-
-
-
 /** \internal
  *  \brief Shorten a device name that is to long
  *
@@ -245,6 +210,11 @@ static int LiveSafeDeviceName(const char *devname, char *newdevname, size_t dest
 
     /* If we have to shorten the interface name */
     if (devnamelen > MAX_DEVNAME) {
+        /* special mode for DPDK pci addresses */
+        if (devnamelen >= 5 && strncmp(devname, "0000:", 5) == 0) {
+            strlcpy(newdevname, devname + 5, destlen);
+            return 0;
+        }
 
         /* IF the dest length is over 10 chars long it will not do any
          * good for the shortening. The shortening is done due to the
@@ -260,7 +230,7 @@ static int LiveSafeDeviceName(const char *devname, char *newdevname, size_t dest
 
         ShortenString(devname, newdevname, destlen, '.');
 
-        SCLogInfo("Shortening device name to: %s", newdevname);
+        SCLogInfo("%s: shortening device name to %s", devname, newdevname);
     } else {
         strlcpy(newdevname, devname, destlen);
     }
@@ -280,7 +250,7 @@ LiveDevice *LiveGetDevice(const char *name)
     LiveDevice *pd;
 
     if (name == NULL) {
-        SCLogWarning(SC_ERR_INVALID_VALUE, "Name of device should not be null");
+        SCLogWarning("Name of device should not be null");
         return NULL;
     }
 
@@ -348,15 +318,17 @@ int LiveDeviceListClean(void)
 
     TAILQ_FOREACH_SAFE(pd, &live_devices, next, tpd) {
         if (live_devices_stats) {
-            SCLogNotice("Stats for '%s':  pkts: %" PRIu64", drop: %" PRIu64 " (%.2f%%), invalid chksum: %" PRIu64,
-                    pd->dev,
-                    SC_ATOMIC_GET(pd->pkts),
-                    SC_ATOMIC_GET(pd->drop),
-                    100 * (SC_ATOMIC_GET(pd->drop) * 1.0) / SC_ATOMIC_GET(pd->pkts),
+            SCLogNotice("%s: packets: %" PRIu64 ", drops: %" PRIu64
+                        " (%.2f%%), invalid chksum: %" PRIu64,
+                    pd->dev, SC_ATOMIC_GET(pd->pkts), SC_ATOMIC_GET(pd->drop),
+                    SC_ATOMIC_GET(pd->pkts) > 0 ? 100 * ((double)SC_ATOMIC_GET(pd->drop)) /
+                                                          (double)SC_ATOMIC_GET(pd->pkts)
+                                                : 0,
                     SC_ATOMIC_GET(pd->invalid_checksums));
         }
 
         RestoreIfaceOffloading(pd);
+        DPDKCloseDevice(pd);
 
         if (pd->dev)
             SCFree(pd->dev);
@@ -499,7 +471,7 @@ int LiveDevUseBypass(LiveDevice *dev)
 {
     BypassInfo *bpinfo = SCCalloc(1, sizeof(*bpinfo));
     if (bpinfo == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Can't allocate bypass info structure");
+        SCLogError("Can't allocate bypass info structure");
         return -1;
     }
 
@@ -587,7 +559,7 @@ void LiveDevAddBypassFail(LiveDevice *dev, uint64_t cnt, int family)
 }
 
 /**
- * Increase number of currently succesfully bypassed flows for a protocol family
+ * Increase number of currently successfully bypassed flows for a protocol family
  *
  * \param dev pointer to LiveDevice to set stats for
  * \param cnt number of flows to add
