@@ -18,8 +18,11 @@
 #ifndef __UTIL_DEVICE_H__
 #define __UTIL_DEVICE_H__
 
+#ifdef HAVE_DPDK
+#include <rte_mempool.h>
+#endif /* HAVE_DPDK */
+
 #include "queue.h"
-#include "unix-manager.h"
 
 #define OFFLOAD_FLAG_SG     (1<<0)
 #define OFFLOAD_FLAG_TSO    (1<<1)
@@ -36,13 +39,19 @@ int LiveGetOffload(void);
 
 #define MAX_DEVNAME 10
 
+#ifdef HAVE_DPDK
+typedef struct {
+    struct rte_mempool *pkt_mp;
+} DPDKDeviceResources;
+#endif /* HAVE_DPDK */
+
 /** storage for live device names */
 typedef struct LiveDevice_ {
     char *dev;  /**< the device (e.g. "eth0") */
     char dev_short[MAX_DEVNAME + 1];
     bool tenant_id_set;
 
-    int id;
+    uint16_t id;
 
     SC_ATOMIC_DECLARE(uint64_t, pkts);
     SC_ATOMIC_DECLARE(uint64_t, drop);
@@ -52,6 +61,10 @@ typedef struct LiveDevice_ {
 
     uint32_t tenant_id;     /**< tenant id in multi-tenancy */
     uint32_t offload_orig;  /**< original offload settings to restore @exit */
+#ifdef HAVE_DPDK
+    // DPDK resources that needs to be cleaned after workers are stopped and devices closed
+    DPDKDeviceResources dpdk_vars;
+#endif
 } LiveDevice;
 
 typedef struct LiveDeviceName_ {
@@ -62,8 +75,6 @@ typedef struct LiveDeviceName_ {
 void LiveDevRegisterExtension(void);
 
 int LiveRegisterDeviceName(const char *dev);
-int LiveGetDeviceNameCount(void);
-const char *LiveGetDeviceNameName(int number);
 int LiveRegisterDevice(const char *dev);
 int LiveDevUseBypass(LiveDevice *dev);
 void LiveDevSetBypassStats(LiveDevice *dev, uint64_t cnt, int family);
