@@ -23,8 +23,10 @@
  */
 
 #include "suricata-common.h"
+#include "util-ioctl.h"
 #include "conf.h"
-#include "util-device.h"
+#include "decode.h"
+#include "decode-sll.h"
 
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
@@ -48,8 +50,6 @@
 #include "win32-syscall.h"
 #endif
 
-#include "util-ioctl.h"
-
 /**
  * \brief output a majorant of hardware header length
  *
@@ -101,15 +101,13 @@ int GetIfaceMTU(const char *pcap_dev)
     }
 
     if (ioctl(fd, SIOCGIFMTU, (char *)&ifr) < 0) {
-        SCLogWarning(SC_ERR_SYSCALL,
-                "Failure when trying to get MTU via ioctl for '%s': %s (%d)",
-                pcap_dev, strerror(errno), errno);
+        SCLogWarning("Failure when trying to get MTU via ioctl for '%s': %s (%d)", pcap_dev,
+                strerror(errno), errno);
         close(fd);
         return -1;
     }
     close(fd);
-    SCLogInfo("Found an MTU of %d for '%s'", ifr.ifr_mtu,
-            pcap_dev);
+    SCLogInfo("%s: MTU %d", pcap_dev, ifr.ifr_mtu);
     return ifr.ifr_mtu;
 #elif defined OS_WIN32
     return GetIfaceMTUWin32(pcap_dev);
@@ -151,7 +149,7 @@ int GetIfaceMaxPacketSize(const char *pcap_dev)
 #ifdef SIOCGIFFLAGS
 /**
  * \brief Get interface flags.
- * \param ifname Inteface name.
+ * \param ifname Interface name.
  * \return Interface flags or -1 on error
  */
 int GetIfaceFlags(const char *ifname)
@@ -167,9 +165,7 @@ int GetIfaceFlags(const char *ifname)
     strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 
     if (ioctl(fd, SIOCGIFFLAGS, &ifr) == -1) {
-        SCLogError(SC_ERR_SYSCALL,
-                   "Unable to get flags for iface \"%s\": %s",
-                   ifname, strerror(errno));
+        SCLogError("%s: failed to get device flags: %s", ifname, strerror(errno));
         close(fd);
         return -1;
     }
@@ -187,7 +183,7 @@ int GetIfaceFlags(const char *ifname)
 #ifdef SIOCSIFFLAGS
 /**
  * \brief Set interface flags.
- * \param ifname Inteface name.
+ * \param ifname Interface name.
  * \param flags Flags to set.
  * \return Zero on success.
  */
@@ -206,13 +202,11 @@ int SetIfaceFlags(const char *ifname, int flags)
     ifr.ifr_flags = flags & 0xffff;
     ifr.ifr_flagshigh = flags >> 16;
 #else
-    ifr.ifr_flags = flags;
+    ifr.ifr_flags = (uint16_t)flags;
 #endif
 
     if (ioctl(fd, SIOCSIFFLAGS, &ifr) == -1) {
-        SCLogError(SC_ERR_SYSCALL,
-                   "Unable to set flags for iface \"%s\": %s",
-                   ifname, strerror(errno));
+        SCLogError("%s: unable to set device flags: %s", ifname, strerror(errno));
         close(fd);
         return -1;
     }
@@ -236,9 +230,7 @@ int GetIfaceCaps(const char *ifname)
     strlcpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
 
     if (ioctl(fd, SIOCGIFCAP, &ifr) == -1) {
-        SCLogError(SC_ERR_SYSCALL,
-                   "Unable to get caps for iface \"%s\": %s",
-                   ifname, strerror(errno));
+        SCLogError("%s: unable to get device caps: %s", ifname, strerror(errno));
         close(fd);
         return -1;
     }
@@ -262,9 +254,7 @@ int SetIfaceCaps(const char *ifname, int caps)
     ifr.ifr_reqcap = caps;
 
     if (ioctl(fd, SIOCSIFCAP, &ifr) == -1) {
-        SCLogError(SC_ERR_SYSCALL,
-                   "Unable to set caps for iface \"%s\": %s",
-                   ifname, strerror(errno));
+        SCLogError("%s: unable to set caps: %s", ifname, strerror(errno));
         close(fd);
         return -1;
     }
@@ -291,9 +281,7 @@ static int GetEthtoolValue(const char *dev, int cmd, uint32_t *value)
     ethv.cmd = cmd;
     ifr.ifr_data = (void *) &ethv;
     if (ioctl(fd, SIOCETHTOOL, (char *)&ifr) < 0) {
-        SCLogWarning(SC_ERR_SYSCALL,
-                  "Failure when trying to get feature via ioctl for '%s': %s (%d)",
-                  dev, strerror(errno), errno);
+        SCLogWarning("%s: failed to get SIOCETHTOOL ioctl: %s", dev, strerror(errno));
         close(fd);
         return -1;
     }
@@ -319,9 +307,7 @@ static int SetEthtoolValue(const char *dev, int cmd, uint32_t value)
     ethv.data = value;
     ifr.ifr_data = (void *) &ethv;
     if (ioctl(fd, SIOCETHTOOL, (char *)&ifr) < 0) {
-        SCLogWarning(SC_ERR_SYSCALL,
-                  "Failure when trying to set feature via ioctl for '%s': %s (%d)",
-                  dev, strerror(errno), errno);
+        SCLogWarning("%s: failed to set SIOCETHTOOL ioctl: %s", dev, strerror(errno));
         close(fd);
         return -1;
     }
@@ -351,11 +337,10 @@ static int GetIfaceOffloadingLinux(const char *dev, int csum, int other)
         }
 #endif
         if (csum_ret == 0)
-            SCLogPerf("NIC offloading on %s: RX %s TX %s", dev, rx, tx);
+            SCLogPerf("%s: NIC offloading: RX %s TX %s", dev, rx, tx);
         else {
-            SCLogWarning(SC_ERR_NIC_OFFLOADING,
-                    "NIC offloading on %s: RX %s TX %s. Run: "
-                    "ethtool -K %s rx off tx off", dev, rx, tx, dev);
+            SCLogWarning("%s: NIC offloading: RX %s TX %s. Run: ethtool -K %s rx off tx off", dev,
+                    rx, tx, dev);
             ret = 1;
         }
     }
@@ -397,12 +382,11 @@ static int GetIfaceOffloadingLinux(const char *dev, int csum, int other)
         }
 #endif
         if (other_ret == 0) {
-            SCLogPerf("NIC offloading on %s: SG: %s, GRO: %s, LRO: %s, "
-                    "TSO: %s, GSO: %s", dev, sg, gro, lro, tso, gso);
+            SCLogPerf("%s: NIC offloading: SG: %s, GRO: %s, LRO: %s, TSO: %s, GSO: %s", dev, sg,
+                    gro, lro, tso, gso);
         } else {
-            SCLogWarning(SC_ERR_NIC_OFFLOADING, "NIC offloading on %s: SG: %s, "
-                    " GRO: %s, LRO: %s, TSO: %s, GSO: %s. Run: "
-                    "ethtool -K %s sg off gro off lro off tso off gso off",
+            SCLogWarning("%s: NIC offloading: SG: %s, GRO: %s, LRO: %s, TSO: %s, GSO: %s. Run: "
+                         "ethtool -K %s sg off gro off lro off tso off gso off",
                     dev, sg, gro, lro, tso, gso, dev);
             ret = 1;
         }
@@ -546,24 +530,21 @@ static int GetIfaceOffloadingBSD(const char *ifname)
     SCLogDebug("if_caps %X", if_caps);
 
     if (if_caps & IFCAP_RXCSUM) {
-        SCLogWarning(SC_ERR_NIC_OFFLOADING,
-                "Using %s with RXCSUM activated can lead to capture "
-                "problems. Run: ifconfig %s -rxcsum", ifname, ifname);
+        SCLogWarning("%s: RXCSUM activated can lead to capture problems. Run: ifconfig %s -rxcsum",
+                ifname, ifname);
         ret = 1;
     }
 #ifdef IFCAP_TOE
     if (if_caps & (IFCAP_TSO|IFCAP_TOE|IFCAP_LRO)) {
-        SCLogWarning(SC_ERR_NIC_OFFLOADING,
-                "Using %s with TSO, TOE or LRO activated can lead to "
-                "capture problems. Run: ifconfig %s -tso -toe -lro",
+        SCLogWarning("%s: TSO, TOE or LRO activated can lead to capture problems. Run: ifconfig %s "
+                     "-tso -toe -lro",
                 ifname, ifname);
         ret = 1;
     }
 #else
     if (if_caps & (IFCAP_TSO|IFCAP_LRO)) {
-        SCLogWarning(SC_ERR_NIC_OFFLOADING,
-                "Using %s with TSO or LRO activated can lead to "
-                "capture problems. Run: ifconfig %s -tso -lro",
+        SCLogWarning(
+                "%s: TSO or LRO activated can lead to capture problems. Run: ifconfig %s -tso -lro",
                 ifname, ifname);
         ret = 1;
     }
@@ -734,19 +715,17 @@ void RestoreIfaceOffloading(LiveDevice *dev)
     }
 }
 
-int GetIfaceRSSQueuesNum(const char *pcap_dev)
+int GetIfaceRSSQueuesNum(const char *dev)
 {
 #if defined HAVE_LINUX_ETHTOOL_H && defined ETHTOOL_GRXRINGS
     struct ifreq ifr;
     struct ethtool_rxnfc nfccmd;
     int fd;
 
-    (void)strlcpy(ifr.ifr_name, pcap_dev, sizeof(ifr.ifr_name));
+    (void)strlcpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
     fd = socket(AF_INET, SOCK_DGRAM, 0);
     if (fd == -1) {
-        SCLogWarning(SC_ERR_SYSCALL,
-                "Failure when opening socket for ioctl: %s (%d)",
-                strerror(errno), errno);
+        SCLogWarning("%s: failed to open socket for ioctl: %s", dev, strerror(errno));
         return -1;
     }
 
@@ -755,16 +734,13 @@ int GetIfaceRSSQueuesNum(const char *pcap_dev)
 
     if (ioctl(fd, SIOCETHTOOL, (char *)&ifr) < 0) {
         if (errno != ENOTSUP) {
-            SCLogWarning(SC_ERR_SYSCALL,
-                         "Failure when trying to get number of RSS queue ioctl for '%s': %s (%d)",
-                         pcap_dev, strerror(errno), errno);
+            SCLogWarning("%s: failed get number of RSS queue ioctl: %s", dev, strerror(errno));
         }
         close(fd);
         return 0;
     }
     close(fd);
-    SCLogInfo("Found %d RX RSS queues for '%s'", (int)nfccmd.data,
-            pcap_dev);
+    SCLogInfo("%s: RX RSS queues: %d", dev, (int)nfccmd.data);
     return (int)nfccmd.data;
 #else
     return 0;
