@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2022 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -29,45 +29,40 @@
 #include "util-misc.h"
 
 #define PARSE_REGEX "^\\s*(\\d+(?:.\\d+)?)\\s*([a-zA-Z]{2})?\\s*$"
-static pcre *parse_regex = NULL;
-static pcre_extra *parse_regex_study = NULL;
+static pcre2_code *parse_regex = NULL;
+static pcre2_match_data *parse_regex_match = NULL;
 
 void ParseSizeInit(void)
 {
-    const char *eb = NULL;
-    int eo;
+    int en;
+    PCRE2_SIZE eo;
     int opts = 0;
 
-    parse_regex = pcre_compile(PARSE_REGEX, opts, &eb, &eo, NULL);
+    parse_regex =
+            pcre2_compile((PCRE2_SPTR8)PARSE_REGEX, PCRE2_ZERO_TERMINATED, opts, &en, &eo, NULL);
     if (parse_regex == NULL) {
-        SCLogError(SC_ERR_PCRE_COMPILE, "Compile of \"%s\" failed at offset "
-                   "%" PRId32 ": %s", PARSE_REGEX, eo, eb);
-        exit(EXIT_FAILURE);
-    }
-    parse_regex_study = pcre_study(parse_regex, 0, &eb);
-    if (eb != NULL) {
-        SCLogError(SC_ERR_PCRE_STUDY, "pcre study failed: %s", eb);
+        PCRE2_UCHAR errbuffer[256];
+        pcre2_get_error_message(en, errbuffer, sizeof(errbuffer));
+        SCLogError("pcre2 compile of \"%s\" failed at "
+                   "offset %d: %s",
+                PARSE_REGEX, (int)eo, errbuffer);
         exit(EXIT_FAILURE);
     }
+    parse_regex_match = pcre2_match_data_create_from_pattern(parse_regex, NULL);
 }
 
 void ParseSizeDeinit(void)
 {
-
-    if (parse_regex != NULL)
-        pcre_free(parse_regex);
-    if (parse_regex_study != NULL)
-        pcre_free_study(parse_regex_study);
+    pcre2_code_free(parse_regex);
+    pcre2_match_data_free(parse_regex_match);
 }
 
 /* size string parsing API */
 
 static int ParseSizeString(const char *size, double *res)
 {
-#define MAX_SUBSTRINGS 30
-    int pcre_exec_ret;
+    int pcre2_match_ret;
     int r;
-    int ov[MAX_SUBSTRINGS];
     int retval = 0;
     char str[128];
     char str2[128];
@@ -75,35 +70,35 @@ static int ParseSizeString(const char *size, double *res)
     *res = 0;
 
     if (size == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENTS,"invalid size argument - NULL. Valid size "
+        SCLogError("invalid size argument - NULL. Valid size "
                    "argument should be in the format - \n"
                    "xxx <- indicates it is just bytes\n"
                    "xxxkb or xxxKb or xxxKB or xxxkB <- indicates kilobytes\n"
                    "xxxmb or xxxMb or xxxMB or xxxmB <- indicates megabytes\n"
-                   "xxxgb or xxxGb or xxxGB or xxxgB <- indicates gigabytes.\n"
-			    );
+                   "xxxgb or xxxGb or xxxGB or xxxgB <- indicates gigabytes.\n");
         retval = -2;
         goto end;
     }
 
-    pcre_exec_ret = pcre_exec(parse_regex, parse_regex_study, size, strlen(size), 0, 0,
-                    ov, MAX_SUBSTRINGS);
-    if (!(pcre_exec_ret == 2 || pcre_exec_ret == 3)) {
-        SCLogError(SC_ERR_PCRE_MATCH, "invalid size argument - %s. Valid size "
+    pcre2_match_ret = pcre2_match(
+            parse_regex, (PCRE2_SPTR8)size, strlen(size), 0, 0, parse_regex_match, NULL);
+
+    if (!(pcre2_match_ret == 2 || pcre2_match_ret == 3)) {
+        SCLogError("invalid size argument - %s. Valid size "
                    "argument should be in the format - \n"
                    "xxx <- indicates it is just bytes\n"
                    "xxxkb or xxxKb or xxxKB or xxxkB <- indicates kilobytes\n"
                    "xxxmb or xxxMb or xxxMB or xxxmB <- indicates megabytes\n"
                    "xxxgb or xxxGb or xxxGB or xxxgB <- indicates gigabytes.\n",
-                   size);
+                size);
         retval = -2;
         goto end;
     }
 
-    r = pcre_copy_substring((char *)size, ov, MAX_SUBSTRINGS, 1,
-                             str, sizeof(str));
+    size_t copylen = sizeof(str);
+    r = pcre2_substring_copy_bynumber(parse_regex_match, 1, (PCRE2_UCHAR8 *)str, &copylen);
     if (r < 0) {
-        SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_copy_substring failed");
+        SCLogError("pcre2_substring_copy_bynumber failed");
         retval = -2;
         goto end;
     }
@@ -112,20 +107,21 @@ static int ParseSizeString(const char *size, double *res)
     errno = 0;
     *res = strtod(str_ptr, &endptr);
     if (errno == ERANGE) {
-        SCLogError(SC_ERR_NUMERIC_VALUE_ERANGE, "Numeric value out of range");
+        SCLogError("Numeric value out of range");
         retval = -1;
         goto end;
     } else if (endptr == str_ptr) {
-        SCLogError(SC_ERR_INVALID_NUMERIC_VALUE, "Invalid numeric value");
+        SCLogError("Invalid numeric value");
         retval = -1;
         goto end;
     }
 
-    if (pcre_exec_ret == 3) {
-        r = pcre_copy_substring((char *)size, ov, MAX_SUBSTRINGS, 2,
-                                 str2, sizeof(str2));
+    if (pcre2_match_ret == 3) {
+        copylen = sizeof(str2);
+        r = pcre2_substring_copy_bynumber(parse_regex_match, 2, (PCRE2_UCHAR8 *)str2, &copylen);
+
         if (r < 0) {
-            SCLogError(SC_ERR_PCRE_GET_SUBSTRING, "pcre_copy_substring failed");
+            SCLogError("pcre2_substring_copy_bynumber failed");
             retval = -2;
             goto end;
         }
@@ -1168,14 +1164,9 @@ static int UtilMiscParseSizeStringTest01(void)
     return 0;
 }
 
-#endif /* UNITTESTS */
-
 void UtilMiscRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("UtilMiscParseSizeStringTest01",
                    UtilMiscParseSizeStringTest01);
-#endif /* UNITTESTS */
-
-    return;
 }
+#endif /* UNITTESTS */
