@@ -54,14 +54,14 @@ static int PoolMemset(void *pitem, void *initdata)
 
 /**
  * \brief Check if data is preallocated
- * \retval 0 if not inside the prealloc'd block, 1 if inside */
-static int PoolDataPreAllocated(Pool *p, void *data)
+ * \retval false if not inside the prealloc'd block, true if inside */
+static bool PoolDataPreAllocated(Pool *p, void *data)
 {
     ptrdiff_t delta = data - p->data_buffer;
     if ((delta < 0) || (delta > p->data_buffer_size)) {
-        return 0;
+        return false;
     }
-    return 1;
+    return true;
 }
 
 /** \brief Init a Pool
@@ -85,25 +85,31 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
         void *(*Alloc)(void), int (*Init)(void *, void *), void *InitData,
         void (*Cleanup)(void *), void (*Free)(void *))
 {
+    sc_errno = SC_OK;
+
     Pool *p = NULL;
 
     if (size != 0 && prealloc_size > size) {
-        SCLogError(SC_ERR_POOL_INIT, "size error");
+        sc_errno = SC_EINVAL;
         goto error;
     }
     if (size != 0 && elt_size == 0) {
-        SCLogError(SC_ERR_POOL_INIT, "size != 0 && elt_size == 0");
+        sc_errno = SC_EINVAL;
         goto error;
     }
     if (elt_size && Free) {
-        SCLogError(SC_ERR_POOL_INIT, "elt_size && Free");
+        sc_errno = SC_EINVAL;
+        goto error;
+    }
+    if (elt_size == 0 && Alloc == NULL) {
+        sc_errno = SC_EINVAL;
         goto error;
     }
 
     /* setup the filter */
     p = SCMalloc(sizeof(Pool));
     if (unlikely(p == NULL)) {
-        SCLogError(SC_ERR_POOL_INIT, "alloc error");
+        sc_errno = SC_ENOMEM;
         goto error;
     }
 
@@ -128,7 +134,7 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
     if (size > 0) {
         PoolBucket *pb = SCCalloc(size, sizeof(PoolBucket));
         if (unlikely(pb == NULL)) {
-            SCLogError(SC_ERR_POOL_INIT, "alloc error");
+            sc_errno = SC_ENOMEM;
             goto error;
         }
         memset(pb, 0, size * sizeof(PoolBucket));
@@ -145,7 +151,7 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
         p->data_buffer = SCCalloc(prealloc_size, elt_size);
         /* FIXME better goto */
         if (p->data_buffer == NULL) {
-            SCLogError(SC_ERR_POOL_INIT, "alloc error");
+            sc_errno = SC_ENOMEM;
             goto error;
         }
     }
@@ -154,7 +160,7 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
         if (size == 0) { /* unlimited */
             PoolBucket *pb = SCMalloc(sizeof(PoolBucket));
             if (unlikely(pb == NULL)) {
-                SCLogError(SC_ERR_POOL_INIT, "alloc error");
+                sc_errno = SC_ENOMEM;
                 goto error;
             }
             memset(pb, 0, sizeof(PoolBucket));
@@ -165,17 +171,17 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
                 pb->data = SCMalloc(p->elt_size);
             }
             if (pb->data == NULL) {
-                SCLogError(SC_ERR_POOL_INIT, "alloc error");
                 SCFree(pb);
+                sc_errno = SC_ENOMEM;
                 goto error;
             }
             if (p->Init(pb->data, p->InitData) != 1) {
-                SCLogError(SC_ERR_POOL_INIT, "init error");
                 if (p->Free)
                     p->Free(pb->data);
                 else
                     SCFree(pb->data);
                 SCFree(pb);
+                sc_errno = SC_EINVAL;
                 goto error;
             }
             p->allocated++;
@@ -186,14 +192,14 @@ Pool *PoolInit(uint32_t size, uint32_t prealloc_size, uint32_t elt_size,
         } else {
             PoolBucket *pb = p->empty_stack;
             if (pb == NULL) {
-                SCLogError(SC_ERR_POOL_INIT, "alloc error");
+                sc_errno = SC_ENOMEM;
                 goto error;
             }
 
             pb->data = (char *)p->data_buffer + u32 * elt_size;
             if (p->Init(pb->data, p->InitData) != 1) {
-                SCLogError(SC_ERR_POOL_INIT, "init error");
                 pb->data = NULL;
+                sc_errno = SC_EINVAL;
                 goto error;
             }
 
@@ -227,7 +233,7 @@ void PoolFree(Pool *p)
         p->alloc_stack = pb->next;
         if (p->Cleanup)
             p->Cleanup(pb->data);
-        if (PoolDataPreAllocated(p, pb->data) == 0) {
+        if (!PoolDataPreAllocated(p, pb->data)) {
             if (p->Free)
                 p->Free(pb->data);
             else
@@ -245,7 +251,7 @@ void PoolFree(Pool *p)
         if (pb->data!= NULL) {
             if (p->Cleanup)
                 p->Cleanup(pb->data);
-            if (PoolDataPreAllocated(p, pb->data) == 0) {
+            if (!PoolDataPreAllocated(p, pb->data)) {
                 if (p->Free)
                     p->Free(pb->data);
                 else
@@ -344,7 +350,7 @@ void PoolReturn(Pool *p, void *data)
         if (p->Cleanup != NULL) {
             p->Cleanup(data);
         }
-        if (PoolDataPreAllocated(p, data) == 0) {
+        if (!PoolDataPreAllocated(p, data)) {
             if (p->Free)
                 p->Free(data);
             else
@@ -372,7 +378,12 @@ void PoolReturn(Pool *p, void *data)
 
 void PoolPrintSaturation(Pool *p)
 {
-    SCLogDebug("pool %p is using %"PRIu32" out of %"PRIu32" items (%02.1f%%), max %"PRIu32" (%02.1f%%): pool struct memory %"PRIu64".", p, p->outstanding, p->max_buckets, (float)(p->outstanding/(float)(p->max_buckets))*100, p->max_outstanding, (float)(p->max_outstanding/(float)(p->max_buckets))*100, (uint64_t)(p->max_buckets * sizeof(PoolBucket)));
+    SCLogDebug("pool %p is using %" PRIu32 " out of %" PRIu32 " items (%02.1f%%), max %" PRIu32
+               " (%02.1f%%): pool struct memory %" PRIu64 ".",
+            p, p->outstanding, p->max_buckets,
+            (float)(p->outstanding) / (float)(p->max_buckets) * 100, p->max_outstanding,
+            (float)(p->max_outstanding) / (float)(p->max_buckets) * 100,
+            (uint64_t)(p->max_buckets * sizeof(PoolBucket)));
 }
 
 /*
