@@ -35,7 +35,9 @@
 #include "detect.h"
 #include "detect-parse.h"
 #include "detect-engine.h"
+#include "detect-engine-alert.h"
 #include "detect-engine-sigorder.h"
+#include "detect-engine-build.h"
 
 #include "stream-tcp.h"
 #include "stream-tcp-private.h"
@@ -143,7 +145,7 @@ uint32_t UTHSetIPv4Address(const char *str)
  * \brief UTHBuildPacketReal is a function that create tcp/udp packets for unittests
  * specifying ip and port sources and destinations (IPV6)
  *
- * \param payload pointer to the payloadd buffer
+ * \param payload pointer to the payload buffer
  * \param payload_len pointer to the length of the payload
  * \param ipproto Protocols allowed atm are IPPROTO_TCP and IPPROTO_UDP
  * \param src pointer to a string containing the ip source
@@ -163,7 +165,7 @@ Packet *UTHBuildPacketIPV6Real(uint8_t *payload, uint16_t payload_len,
     if (unlikely(p == NULL))
         return NULL;
 
-    TimeGet(&p->ts);
+    p->ts = TimeGet();
 
     p->src.family = AF_INET6;
     p->dst.family = AF_INET6;
@@ -229,7 +231,7 @@ error:
  * \brief UTHBuildPacketReal is a function that create tcp/udp packets for unittests
  * specifying ip and port sources and destinations
  *
- * \param payload pointer to the payloadd buffer
+ * \param payload pointer to the payload buffer
  * \param payload_len pointer to the length of the payload
  * \param ipproto Protocols allowed atm are IPPROTO_TCP and IPPROTO_UDP
  * \param src pointer to a string containing the ip source
@@ -249,9 +251,7 @@ Packet *UTHBuildPacketReal(uint8_t *payload, uint16_t payload_len,
     if (unlikely(p == NULL))
         return NULL;
 
-    struct timeval tv;
-    TimeGet(&tv);
-    COPY_TIMESTAMP(&tv, &p->ts);
+    p->ts = TimeGet();
 
     p->src.family = AF_INET;
     p->dst.family = AF_INET;
@@ -328,7 +328,7 @@ error:
  * \brief UTHBuildPacket is a wrapper that build packets with default ip
  * and port fields
  *
- * \param payload pointer to the payloadd buffer
+ * \param payload pointer to the payload buffer
  * \param payload_len pointer to the length of the payload
  * \param ipproto Protocols allowed atm are IPPROTO_TCP and IPPROTO_UDP
  *
@@ -358,8 +358,8 @@ Packet **UTHBuildPacketArrayFromEth(uint8_t *raw_eth[], int *pktsize, int numpkt
     DecodeThreadVars dtv;
     ThreadVars th_v;
     if (raw_eth == NULL || pktsize == NULL || numpkts <= 0) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "The arrays cant be null, and the number"
-                                        " of packets should be grater thatn zero");
+        SCLogError("The arrays cant be null, and the number"
+                   " of packets should be grater thatn zero");
         return NULL;
     }
     Packet **p = NULL;
@@ -409,7 +409,7 @@ Packet *UTHBuildPacketFromEth(uint8_t *raw_eth, uint16_t pktsize)
  * \brief UTHBuildPacketSrcDst is a wrapper that build packets specifying IPs
  * and defaulting ports
  *
- * \param payload pointer to the payloadd buffer
+ * \param payload pointer to the payload buffer
  * \param payload_len pointer to the length of the payload
  * \param ipproto Protocols allowed atm are IPPROTO_TCP and IPPROTO_UDP
  *
@@ -445,7 +445,7 @@ Packet *UTHBuildPacketIPV6SrcDst(uint8_t *payload, uint16_t payload_len,
  * \brief UTHBuildPacketSrcDstPorts is a wrapper that build packets specifying
  * src and dst ports and defaulting IPs
  *
- * \param payload pointer to the payloadd buffer
+ * \param payload pointer to the payload buffer
  * \param payload_len pointer to the length of the payload
  * \param ipproto Protocols allowed atm are IPPROTO_TCP and IPPROTO_UDP
  *
@@ -540,7 +540,7 @@ int UTHAddStreamToFlow(Flow *f, int direction,
 
     StreamingBufferSegment seg;
     TcpStream *stream = direction == 0 ? &ssn->client : &ssn->server;
-    int r = StreamingBufferAppend(&stream->sb, &seg, data, data_len);
+    int r = StreamingBufferAppend(&stream->sb, &stream_config.sbcnf, &seg, data, data_len);
     FAIL_IF_NOT(r == 0);
     stream->last_ack += data_len;
     return 1;
@@ -555,7 +555,7 @@ int UTHAddSessionToFlow(Flow *f,
     TcpSession *ssn = SCCalloc(1, sizeof(*ssn));
     FAIL_IF_NULL(ssn);
 
-    StreamingBuffer x = STREAMING_BUFFER_INITIALIZER(&stream_config.sbcnf);
+    StreamingBuffer x = STREAMING_BUFFER_INITIALIZER;
     ssn->client.sb = x;
     ssn->server.sb = x;
 
@@ -579,10 +579,10 @@ int UTHRemoveSessionFromFlow(Flow *f)
 }
 
 /**
- * \brief UTHGenericTest: function that perfom a generic check taking care of
+ * \brief UTHGenericTest: function that perform a generic check taking care of
  *                      as maximum common unittest elements as possible.
  *                      It will create a detection engine, append an array
- *                      of signatures an check the spected results for each
+ *                      of signatures an check the expected results for each
  *                      of them, it check matches for an array of packets
  *
  * \param pkt pointer to the array of packets
@@ -607,7 +607,8 @@ int UTHGenericTest(Packet **pkt, int numpkts, const char *sigs[], uint32_t sids[
     int result = 0;
     if (pkt == NULL || sigs == NULL || numpkts == 0
         || sids == NULL || results == NULL || numsigs == 0) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Arguments invalid, that the pointer/arrays are not NULL, and the number of signatures and packets is > 0");
+        SCLogError("Arguments invalid, that the pointer/arrays are not NULL, and the number of "
+                   "signatures and packets is > 0");
         goto end;
     }
     DetectEngineCtx *de_ctx = DetectEngineCtxInit();
@@ -622,11 +623,7 @@ int UTHGenericTest(Packet **pkt, int numpkts, const char *sigs[], uint32_t sids[
     result = UTHMatchPacketsWithResults(de_ctx, pkt, numpkts, sids, results, numsigs);
 
 cleanup:
-    if (de_ctx != NULL) {
-        SigGroupCleanup(de_ctx);
-        SigCleanSignatures(de_ctx);
-        DetectEngineCtxFree(de_ctx);
-    }
+    DetectEngineCtxFree(de_ctx);
 end:
     return result;
 }
@@ -644,25 +641,24 @@ end:
  * \retval int 1 if the match of all the sids is the specified has the
  *             specified results; 0 if not
  */
-int UTHCheckPacketMatchResults(Packet *p, uint32_t sids[],
-        uint32_t results[], int numsids)
+int UTHCheckPacketMatchResults(Packet *p, uint32_t sids[], uint32_t results[], int numsigs)
 {
     if (p == NULL || sids == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Arguments invalid, check if the "
-                "packet is NULL, and if the array contain sids is set");
+        SCLogError("Arguments invalid, check if the "
+                   "packet is NULL, and if the array contain sids is set");
         return 0;
     }
 
     int i = 0;
     int res = 1;
-    for (; i < numsids; i++) {
-        uint16_t r = PacketAlertCheck(p, sids[i]);
+    for (; i < numsigs; i++) {
+        uint32_t r = PacketAlertCheck(p, sids[i]);
         if (r != results[i]) {
-            SCLogInfo("Sid %"PRIu32" matched %"PRIu16" times, and not %"PRIu32
-                    " as expected", sids[i], r, results[i]);
+            SCLogInfo("Sid %" PRIu32 " matched %" PRIu32 " times, and not %" PRIu32 " as expected",
+                    sids[i], r, results[i]);
             res = 0;
         } else {
-            SCLogInfo("Sid %"PRIu32" matched %"PRIu16" times, as expected", sids[i], r);
+            SCLogInfo("Sid %" PRIu32 " matched %" PRIu32 " times, as expected", sids[i], r);
         }
     }
     return res;
@@ -676,39 +672,35 @@ int UTHCheckPacketMatchResults(Packet *p, uint32_t sids[],
  * \param numsigs number of signatures to load from the array
  *                (size of the array)
  *
- * \retval int 0 if we have errors; 1 if all the signatures loaded succesfuly
+ * \retval int 0 if we have errors; 1 if all the signatures loaded successfully
  */
 int UTHAppendSigs(DetectEngineCtx *de_ctx, const char *sigs[], int numsigs)
 {
-    if (de_ctx == NULL || numsigs <= 0 || sigs == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Arguments invalid, check if sigs or de_ctx are NULL, and if the array contain sigs");
-        return 0;
-    }
-    //SCLogDebug("Adding %d signatures for the current unittest", numsigs);
+    BUG_ON(de_ctx == NULL);
+    BUG_ON(numsigs <= 0);
+    BUG_ON(sigs == NULL);
 
-    Signature *s;
-    int i = 0;
-
-    for ( ; i < numsigs; i++) {
+    for (int i = 0; i < numsigs; i++) {
         if (sigs[i] == NULL) {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Check the signature"
-                       " at position %d", i);
+            SCLogError("Check the signature"
+                       " at position %d",
+                    i);
             return 0;
         }
-        s = DetectEngineAppendSig(de_ctx, sigs[i]);
+        Signature *s = DetectEngineAppendSig(de_ctx, sigs[i]);
         if (s == NULL) {
-            SCLogError(SC_ERR_INVALID_ARGUMENT, "Check the signature at"
-                       " position %d (%s)", i, sigs[i]);
+            SCLogError("Check the signature at"
+                       " position %d (%s)",
+                    i, sigs[i]);
             return 0;
         }
     }
-    //SCLogDebug("Added %d signatures to the de_ctx of the unittest", i);
     return 1;
 }
 
 /**
  * \test UTHMatchPacketsWithResults Match a packet or a array of packets against sigs
- * of a de_ctx, checking that each signature match match X times for certain packets
+ * of a de_ctx, checking that each signature matches X times for certain packets
  *
  * \param de_ctx pointer with the signatures loaded
  * \param p pointer to the array of packets
@@ -719,39 +711,28 @@ int UTHAppendSigs(DetectEngineCtx *de_ctx, const char *sigs[], int numsigs)
  */
 int UTHMatchPacketsWithResults(DetectEngineCtx *de_ctx, Packet **p, int num_packets, uint32_t sids[], uint32_t *results, int numsigs)
 {
+    BUG_ON(de_ctx == NULL);
+    BUG_ON(p == NULL);
+
     int result = 0;
-
-    if (de_ctx == NULL || p == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "packet or de_ctx was null");
-        result = 0;
-        goto end;
-    }
-
     DecodeThreadVars dtv;
     ThreadVars th_v;
     DetectEngineThreadCtx *det_ctx = NULL;
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&th_v, 0, sizeof(th_v));
 
-    //de_ctx->flags |= DE_QUIET;
-
     SigGroupBuild(de_ctx);
     DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
 
-    int i = 0;
-    for (; i < num_packets; i++) {
+    for (int i = 0; i < num_packets; i++) {
         SigMatchSignatures(&th_v, de_ctx, det_ctx, p[i]);
         if (UTHCheckPacketMatchResults(p[i], sids, &results[(i * numsigs)], numsigs) == 0)
             goto cleanup;
     }
 
-    /* so far, so good ;) */
     result = 1;
-
 cleanup:
-    if (det_ctx != NULL)
-        DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-end:
+    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     return result;
 }
 
@@ -769,42 +750,29 @@ end:
  */
 int UTHMatchPackets(DetectEngineCtx *de_ctx, Packet **p, int num_packets)
 {
+    BUG_ON(de_ctx == NULL);
+    BUG_ON(p == NULL);
     int result = 1;
-
-    if (de_ctx == NULL || p == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "packet or de_ctx was null");
-        result = 0;
-        goto end;
-    }
-
     DecodeThreadVars dtv;
     ThreadVars th_v;
     DetectEngineThreadCtx *det_ctx = NULL;
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&th_v, 0, sizeof(th_v));
-
-    //de_ctx->flags |= DE_QUIET;
-
     SCSigRegisterSignatureOrderingFuncs(de_ctx);
     SCSigOrderSignatures(de_ctx);
     SCSigSignatureOrderingModuleCleanup(de_ctx);
     SigGroupBuild(de_ctx);
     DetectEngineThreadCtxInit(&th_v, (void *)de_ctx, (void *)&det_ctx);
 
-    int i = 0;
-    for (; i < num_packets; i++)
+    for (int i = 0; i < num_packets; i++)
         SigMatchSignatures(&th_v, de_ctx, det_ctx, p[i]);
 
     /* Here we don't check if the packet matched or not, because
      * the de_ctx can have multiple signatures, and some of them may match
      * and others may not. That check will be outside
      */
-    if (det_ctx != NULL) {
-        DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-    }
-end:
+    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
     if (de_ctx != NULL) SigGroupCleanup(de_ctx);
-
     return result;
 }
 
@@ -857,15 +825,8 @@ int UTHPacketMatchSigMpm(Packet *p, char *sig, uint16_t mpm_type)
 
     result = 1;
 end:
-    SigGroupCleanup(de_ctx);
-    SigCleanSignatures(de_ctx);
-
-    if (det_ctx != NULL)
-        DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
-
-    if (de_ctx != NULL)
-        DetectEngineCtxFree(de_ctx);
-
+    DetectEngineThreadCtxDeinit(&th_v, (void *)det_ctx);
+    DetectEngineCtxFree(de_ctx);
     SCReturnInt(result);
 }
 
@@ -946,11 +907,10 @@ uint32_t UTHBuildPacketOfFlows(uint32_t start, uint32_t end, uint8_t dir)
         }
         FlowHandlePacket(NULL, &fls, p);
         if (p->flow != NULL) {
-            p->flow->use_cnt = 0;
             FLOWLOCK_UNLOCK(p->flow);
         }
 
-        /* Now the queues shoul be updated */
+        /* Now the queues should be updated */
         UTHFreePacket(p);
     }
 
